local apolloengine = require "apolloengine"
local mathfunction = require "mathfunction"
local venuscore = require "venuscore"
local Object = require "classic"

--虚拟节点，不包含可渲染的组件
local PostEffect = Object:extend();

function PostEffect:new(post)
  self.post = post;
end

function PostEffect:GetPostEffect()
  return self.post;
end

function PostEffect:CreateResource(scriptpath)
  self.post:PushMetadata(
    apolloengine.PathMetadata(scriptpath));
  return self.post:CreateResource();  
end

function PostEffect:SetScriptParameter(parameter, value)
  self.post:SetParameter(parameter, value);
end

function PostEffect:_RegisterParameter(slotname)
  local ps = apolloengine.IMaterialSystem:GetParameterSlot(slotname);
  local mats = self.post:GetMaterials()
  for _, mat in pairs(mats) do
    if mat:hasParameter(ps) then
      self[slotname] = function(self, value)
        mat:SetParameter(ps, value);
      end
      return
    end
  end
  self[slotname] = function(self, value)
    self.post:SetParameter(slotname, value);
  end
end


function PostEffect:_RegisterParameterWithMaterial(slotname, material)
  local ps = apolloengine.IMaterialSystem:GetParameterSlot(slotname);
  local mat = self.post:GetMaterial(material);
  self[slotname] = function(self, value)
    mat:SetParameter(ps, value);
  end  
end

function PostEffect:RegisterParameter(slotname, material)
  if material then
    self:_RegisterParameterWithMaterial(slotname, material);
  else
    self:_RegisterParameter(slotname);
  end  
end

function PostEffect:Enable()
  self.post:Enable();
end

function PostEffect:Disable()
  self.post:Disable();
end

function PostEffect:SetQueue(queue)
  self.post:SetQueue(queue);
end


return PostEffect;