local venuscore = require "venuscore"
local apolloengine = require "apolloengine"
local mathfunction = require "mathfunction"
local venusjson = require "venusjson"

local likeapp
if not _PLATFORM_WINDOWS then
  likeapp = require "likeapp"
end

local CustomGiftControllerBehavior = venuscore.VenusBehavior:extend("CustomGiftControllerBehavior"); 
local maxConfigGroup = 50; 
local configSavePath = "proj:"
function CustomGiftControllerBehavior:new()
  self.playRoot = nil;  --播放礼物控制根节点
  self.playAniName = ""; --播放部分动画
  self.previewRoot = nil; --预览礼物控制根节点
  self.previewAniName = "" --预览部分动画
  self.captureCamera = nil;  --用于生产礼物定制图的相机
  self._captureCameraUpdateCnt = 0;
  
  --礼物模型根节点
  --在预览模式下，礼物可以旋转缩放
  self.previewGiftModel = nil; 
  self._previewGiftModelTrans = nil;
  self._cachedLocalPosition = mathfunction.vector3(0,0,0);
  self._cachedLocalRotation = mathfunction.Quaternion(0,0,0,1);
  self._cachedLocalScale = mathfunction.vector3(1,1,1);

  -- 记录无头像和有头像的截屏相机位置和欧拉角
  self.captureCamPos = mathfunction.vector3(0,0,0);
  self.captureCamRot = mathfunction.vector3(0,0,0);

  self.captureCamPosAvatar = mathfunction.vector3(0,0,0);
  self.captureCamRotAvatar = mathfunction.vector3(0,0,0);
  --礼物的骨骼动画Component
  --上边会挂多个动画(play/preview模式下有不同的动画)
  self.skeletonAnimationComponent = nil; --礼物的骨骼动画组件
  self.curAnimationName = nil;

  self.captureAvatar1 = nil;
  self.captureAvatar2 = nil;
  self.captureAvatar2_1 = nil;
  self.captureAvatar2_2 = nil;
end

function CustomGiftControllerBehavior:_Initialize()
  if self.previewGiftModel then
    local transComp = self.previewGiftModel:GetComponent(apolloengine.Node.CT_TRANSFORM);
    if transComp then
      self._previewGiftModelTrans = transComp;
      self._cachedLocalPosition = transComp:GetLocalPosition();
      self._cachedLocalRotation = transComp:GetLocalRotation();
      self._cachedLocalScale = transComp:GetLocalScale();
    end
  end
  if self.captureCamera then
    self.captureCamera.Active = false;
    local captureCamComp = self.captureCamera:GetComponent(apolloengine.Node.CT_CAMERA);
  end
  
end


function CustomGiftControllerBehavior:_OnAwake()
  self:_Initialize();
end


function CustomGiftControllerBehavior:_OnStart()
  
end


function CustomGiftControllerBehavior:_OnUpdate(def)
  if not _KRATOSEDITOR then
    self:_CaptureProcess();
  end
end


function CustomGiftControllerBehavior:_CaptureProcess()
  --处理截帧的相机
  if self.captureCamera then
    if self.captureCamera.Active == true then
      if self._captureCameraUpdateCnt > 1 then
        local captureCamComp = self.captureCamera:GetComponent(apolloengine.Node.CT_CAMERA);
        if captureCamComp then
          self._captureCameraUpdateCnt = 0;
          local captureRT = captureCamComp:GetAttachedRenderTarget();
          if captureRT then
            local stream = captureRT:GetAttachmentStream(apolloengine.RenderTargetEntity.TA_COLOR_0);
            if stream then
              if likeapp then
                likeapp.GiftInvokerProxy:onCaptureCallback(stream);
              else  --PC上模拟端内保存成一张图片
                apolloengine.IGraphicSystem:SaveToPng("E:\\test_capture.png",stream);
              end
            end
          end
          self.captureCamera.Active = false;
        else
          ERROR("Capture camera is not a valid camera node");
        end
      else
        self._captureCameraUpdateCnt = self._captureCameraUpdateCnt + 1;
      end
    end
  end
end


function CustomGiftControllerBehavior:_OnPackage()
end

function CustomGiftControllerBehavior:PlayGift()

  self.previewRoot.Active = false;
  
  if self._previewGiftModelTrans then
    self._previewGiftModelTrans:SetLocalPosition(self._cachedLocalPosition);
    self._previewGiftModelTrans:SetLocalRotation(self._cachedLocalRotation);
    self._previewGiftModelTrans:SetLocalScale(self._cachedLocalScale);
  end
  
  if self.previewAniName ~= "" then
    self.skeletonAnimationComponent:Reset(self.previewAniName);
    self.skeletonAnimationComponent:Stop(self.previewAniName);
  end
  if self.playAniName ~= "" then
    self.skeletonAnimationComponent:Reset(self.playAniName);
    self.skeletonAnimationComponent:Play(self.playAniName);
  end


  self.playRoot.Active = true;

end

function CustomGiftControllerBehavior:PreviewGift()
  self.playRoot.Active = false;

  
  if self.playAniName ~= "" then
    self.skeletonAnimationComponent:Reset(self.playAniName);
    self.skeletonAnimationComponent:Stop(self.playAniName);
  end
  if self.previewAniName ~= "" then
    self.skeletonAnimationComponent:Reset(self.previewAniName);
    self.skeletonAnimationComponent:Play(self.previewAniName);
  end

  self.previewRoot.Active = true;

end

function CustomGiftControllerBehavior:PreviewSetRotation(xAngle,yAngle,zAngle)
  if self.previewGiftModel then
    local locaQuat=  mathfunction.Mathutility:YawPitchRoll(yAngle, xAngle, zAngle)
    self._previewGiftModelTrans:SetLocalRotation(locaQuat);
  end
end

function CustomGiftControllerBehavior:PreviewSetZoom(zoom)
  if self.previewGiftModel then
    self._previewGiftModelTrans:SetLocalScale(mathfunction.vector3(zoom,zoom,zoom));
  end
end

function CustomGiftControllerBehavior:PreviewReset()
  if self._previewGiftModelTrans then
    self._previewGiftModelTrans:SetLocalPosition(self._cachedLocalPosition);
    self._previewGiftModelTrans:SetLocalRotation(self._cachedLocalRotation);
    self._previewGiftModelTrans:SetLocalScale(self._cachedLocalScale);
  end
end

function CustomGiftControllerBehavior:StartCapture(newSetting)
  if self.captureCamera then
    WARNING("StartCapture called")
    local captureCamComp = self.captureCamera:GetComponent(apolloengine.Node.CT_CAMERA);
    if captureCamComp then
      captureCamComp.isFixedResolution = true;
      captureCamComp.CameraResolution = mathfunction.vector2(200,200);
      local CameraResolution = captureCamComp.CameraResolution;
      local captureCameraRT = apolloengine.RenderTargetEntity();--创建一个FBO
      captureCameraRT:SetJobType(venuscore.IJob.JT_SYNCHRONOUS)
      local viewport = mathfunction.vector4(0.0,0.0,CameraResolution:x(),CameraResolution:y())
      captureCameraRT:PushMetadata(--设置FBO格式
          apolloengine.RenderTargetMetadata(
          apolloengine.RenderTargetEntity.RT_RENDER_TARGET_2D,
          apolloengine.RenderTargetEntity.ST_SWAP_UNIQUE,
          viewport,
          CameraResolution, --分辨率
          mathfunction.vector2(1.0, 1.0)));
      local tex = captureCameraRT:MakeTextureAttachment(apolloengine.RenderTargetEntity.TA_COLOR_0);--增加color0纹理
      local metadata = apolloengine.TextureRenderMetadata(
         apolloengine.RenderTargetEntity.ST_SWAP_UNIQUE,
         apolloengine.TextureEntity.TU_READ,
         CameraResolution);
      metadata:SetNeedExport(true);
      tex:PushMetadata(metadata);

      --深度
      local depth = captureCameraRT:MakeTextureAttachment(apolloengine.RenderTargetEntity.TA_DEPTH_STENCIL);
      depth:PushMetadata(
        apolloengine.DepthRenderBufferMetadata(
        apolloengine.RenderTargetEntity.ST_SWAP_UNIQUE,
        CameraResolution,
        apolloengine.TextureEntity.PF_DEPTH24_STENCIL8));
    
      captureCameraRT:CreateResource();
      captureCameraRT:PauseResumeRead(true);    
      captureCamComp:AttachRenderTarget(captureCameraRT);

      if newSetting["avatar_1"] ~= nil then 
        if self.captureAvatar1 then
          self.captureAvatar1.Active = newSetting["avatar_1"];
        else
          ERROR("CustomGiftControllerBehavior:StartCapture anchor node is nil");
        end
      end
      if newSetting["avatar_2"] ~= nil then 
        if self.captureAvatar2 then
          self.captureAvatar2.Active = newSetting["avatar_2"];
        else
          ERROR("CustomGiftControllerBehavior:StartCapture audience node is nil");
        end
      end
      if newSetting["avatar2_1"] ~= nil then 
        if self.captureAvatar2_1 then
          self.captureAvatar2_1.Active = newSetting["avatar2_1"];
        else
          ERROR("CustomGiftControllerBehavior:StartCapture anchor node is nil when two avatars are shown");
        end
      end
      if newSetting["avatar2_2"] ~= nil then 
        if self.captureAvatar2_2 then
          self.captureAvatar2_2.Active = newSetting["avatar2_2"];
        else
          ERROR("CustomGiftControllerBehavior:StartCapture audience node is nil when two avatars are shown");
        end
      end
      local hasAvatar = false;
      for key, value in pairs(newSetting) do
        hasAvatar = hasAvatar or value;
      end
      local trans = self.captureCamera:GetComponent(apolloengine.Node.CT_TRANSFORM);
      if hasAvatar == true then
        trans:SetLocalPosition(self.captureCamPosAvatar);
        trans.LocalEularAngle = self.captureCamRotAvatar;
      else
        trans:SetLocalPosition(self.captureCamPos);
        trans.LocalEularAngle = self.captureCamRot;
      end

      self.captureCamera.Active = true;

    end
  else
    ERROR("Capture camera do not correctly setup!");
  end
   
end


function CustomGiftControllerBehavior:GetPlayRootNode()
  return self.playRoot;
end

function CustomGiftControllerBehavior:SetPlayRootNode(rootNode)
  self.playRoot = rootNode;
end

function CustomGiftControllerBehavior:GetPreviewRootNode()
  return self.previewRoot;
end

function CustomGiftControllerBehavior:SetPreviewRootNode(rootNode)
  self.previewRoot = rootNode;
end

function CustomGiftControllerBehavior:SetAnimationComponent(aniComp)
  self.skeletonAnimationComponent = aniComp;
end

function CustomGiftControllerBehavior:GetAnimationComponent()
  return self.skeletonAnimationComponent;
end

function CustomGiftControllerBehavior:SetGiftModel(gift)
  self.previewGiftModel = gift;
end

function CustomGiftControllerBehavior:GetGiftModel()
  return self.previewGiftModel;
end

function CustomGiftControllerBehavior:GetCaptureCamera()
  return self.captureCamera;
end

function CustomGiftControllerBehavior:SetCaptureCamera(cameraRef)
  self.captureCamera = cameraRef; 
end

function CustomGiftControllerBehavior:SetCaptureAvatar1(avatar1)
  self.captureAvatar1 = avatar1;
end

function CustomGiftControllerBehavior:GetCaptureAvatar1()
  return self.captureAvatar1;
end

function CustomGiftControllerBehavior:SetCaptureAvatar2(avatar2)
  self.captureAvatar2 = avatar2;
end

function CustomGiftControllerBehavior:GetCaptureAvatar2()
  return self.captureAvatar2;
end

function CustomGiftControllerBehavior:SetCaptureAvatar2_1(avatar1)
  self.captureAvatar2_1 = avatar1;
end

function CustomGiftControllerBehavior:GetCaptureAvatar2_1()
  return self.captureAvatar2_1;
end

function CustomGiftControllerBehavior:SetCaptureAvatar2_2(avatar2)
  self.captureAvatar2_2 = avatar2;
end

function CustomGiftControllerBehavior:GetCaptureAvatar2_2()
  return self.captureAvatar2_2;
end



CustomGiftControllerBehavior:MemberRegister("playAniName");
CustomGiftControllerBehavior:MemberRegister("previewAniName");

CustomGiftControllerBehavior:MemberRegister("captureCamRot");
CustomGiftControllerBehavior:MemberRegister("captureCamPos");
CustomGiftControllerBehavior:MemberRegister("captureCamRotAvatar");
CustomGiftControllerBehavior:MemberRegister("captureCamPosAvatar");

CustomGiftControllerBehavior:MemberRegister("skeletonAnimationComponent",
  venuscore.ScriptTypes.ReferenceType(
    apolloengine.AnimationComponent:RTTI(),
    CustomGiftControllerBehavior.GetAnimationComponent,
    CustomGiftControllerBehavior.SetAnimationComponent
  ));
  
 CustomGiftControllerBehavior:MemberRegister("previewGiftModel",
  venuscore.ScriptTypes.ReferenceType(
    apolloengine.Node:RTTI(),
    CustomGiftControllerBehavior.GetGiftModel,
    CustomGiftControllerBehavior.SetGiftModel
  )); 

 CustomGiftControllerBehavior:MemberRegister("captureCamera",
  venuscore.ScriptTypes.ReferenceType(
    apolloengine.Node:RTTI(),
    CustomGiftControllerBehavior.GetCaptureCamera,
    CustomGiftControllerBehavior.SetCaptureCamera
  )); 

CustomGiftControllerBehavior:MemberRegister("playRoot",
  venuscore.ScriptTypes.ReferenceType(
    apolloengine.Node:RTTI(),
    CustomGiftControllerBehavior.GetPlayRootNode,
    CustomGiftControllerBehavior.SetPlayRootNode
  )); 

CustomGiftControllerBehavior:MemberRegister("previewRoot",
  venuscore.ScriptTypes.ReferenceType(
    apolloengine.Node:RTTI(),
    CustomGiftControllerBehavior.GetPreviewRootNode,
    CustomGiftControllerBehavior.SetPreviewRootNode
  )); 

 CustomGiftControllerBehavior:MemberRegister("captureAvatar1",
  venuscore.ScriptTypes.ReferenceType(
    apolloengine.Node:RTTI(),
    CustomGiftControllerBehavior.GetCaptureAvatar1,
    CustomGiftControllerBehavior.SetCaptureAvatar1
  )); 
  CustomGiftControllerBehavior:MemberRegister("captureAvatar2",
  venuscore.ScriptTypes.ReferenceType(
    apolloengine.Node:RTTI(),
    CustomGiftControllerBehavior.GetCaptureAvatar2,
    CustomGiftControllerBehavior.SetCaptureAvatar2
  )); 
   CustomGiftControllerBehavior:MemberRegister("captureAvatar2_1",
  venuscore.ScriptTypes.ReferenceType(
    apolloengine.Node:RTTI(),
    CustomGiftControllerBehavior.GetCaptureAvatar2_1,
    CustomGiftControllerBehavior.SetCaptureAvatar2_1
  )); 
    CustomGiftControllerBehavior:MemberRegister("captureAvatar2_2",
  venuscore.ScriptTypes.ReferenceType(
    apolloengine.Node:RTTI(),
    CustomGiftControllerBehavior.GetCaptureAvatar2_2,
    CustomGiftControllerBehavior.SetCaptureAvatar2_2
  )); 


return CustomGiftControllerBehavior;