
local venuscore = require "venuscore"
local venusjson = require "venusjson"
local apolloengine = require "apolloengine"
local mathfunction = require "mathfunction"
local EditorSystem = require "window.editor.system.editsystem"
local ContentManager = require "apolloengine.content.contentmanager"
local CommandManager = require "window.editor.command.command_manager"
local SetParameterCmd = require "window.editor.command.commands.rendercomponent.set_parameter_command"
local mattool = require "window.editor.widget.assets.mattool"
local mattoolWrap = require "window.editor.widget.inspector.panels.assets.mat"
local lfs = require("lfs")
local filelfs = require "window.editor.widget.assets.fileutility"


local ImportMatParam = venuscore.VenusBehavior:extend("ImportMatParam");


local shaderMappingList = {
    ["cel_hair"]        = "comm:documents/editor_shaders/3d/cel_hair_old.material",
    ["cel_cloth"]       = "comm:documents/editor_shaders/3d/cel_cloth_old.material",
    ["Cloth"]       = "comm:documents/editor_shaders/3d/cel_cloth_old.material",
    ["cel_metal"]       = "comm:documents/editor_shaders/3d/cel_metal_old.material",
    ["cel_cloth_part4"] = "comm:documents/editor_shaders/3d/cel_metal_old.material",
    ["cel_transparentcloth"]	    = "comm:documents/editor_shaders/3d/cel_transparentcloth_old.material",
    ["vtuber_body"]        = "comm:documents/editor_shaders/3d/vtuber_body_old.material",
    ["venus_vyou_head"]	    = "comm:documents/editor_shaders/3d/venus_vyou_head_old.material",
   
    ["cel_hair_multichannel"]        = "comm:documents/editor_shaders/3d/cel_hair.material",
    ["cel_metal_multichannel"]        = "comm:documents/editor_shaders/3d/cel_metal.material",
    ["cel_transparentcloth_multichannel"]        = "comm:documents/editor_shaders/3d/cel_transparentcloth.material",
    ["Cloth_multichannel"]        = "comm:documents/editor_shaders/3d/cel_cloth.material",
   
    ["head"]        = "comm:documents/editor_shaders/3d/venus_vyou_head.material",
    ["eye"]        = "comm:documents/editor_shaders/3d/cel_eye.material",
    ["cel_eyebrow"]        = "comm:documents/editor_shaders/3d/cel_eyebrow.material",
    ["cel_nose"]        = "comm:documents/editor_shaders/3d/cel_nose.material",
    ["cel_eyelashes"]        = "comm:documents/editor_shaders/3d/cel_eyelashes.material",
    ["cel_eyebottom"]        = "comm:documents/editor_shaders/3d/cel_eyebottom.material",
    ["cel_tongue"]        = "comm:documents/editor_shaders/3d/cel_tongue.material",

    ["ColorShader"]        = "comm:documents/editor_shaders/3d/fur_smooth_edge.material",
    ["cel_highlight"]        = "comm:documents/editor_shaders/3d/cel_highlight_old.material",

}




function ImportMatParam:new()
    self.MaterialParamsConfig = "";
    self.tryImportParam = false;
    self.paramImported = self.tryImportParam;
    self.SceneDirectory = "";
    self.mat_params = {};
    self.assetHierarchy = {};
end

function ImportMatParam:_OnAwake()
end

function ImportMatParam:_OnStart()
    self.SceneDirectory = self:ParseScenePath(EditorSystem.sceneFile);
end

function ImportMatParam:_OnUpdate()

    if self.tryImportParam ~= self.paramImported then
        self:_GenerateMatfiles();
        self.paramImported = self.tryImportParam;
    end
end

--得到所有的rendercomponent的node
function ImportMatParam:GetMeshNodes(node, meshNodes)
    if node.Active == false then
        return;
    end
    local childs = {};
    if node.GetChildrens then
      childs = node:GetChildrens();
    end
    if childs == nil then
      return;
    end
    for _,v in pairs(childs) do
      local renderCom = v:GetComponent(apolloengine.Node.CT_RENDER);
      if renderCom then
        table.insert(meshNodes, v);
      else
        self:GetMeshNodes(v, meshNodes);
      end
    end
end

function ImportMatParam:ParseScenePath(path)
    return string.match(path,"[%w%:_/]+/assets/[%w_]+/");
end

function ImportMatParam:ParseMaterialName(matPath)
    return string.match(matPath,"[%w_]+%.mat");
end

function ImportMatParam:ParseTexturePath(texPath)
    local idx = string.find(texPath, "/", 1);
    texPath = string.sub(texPath, idx+1, -1);
    idx = string.find(texPath, "/", 1);
    texPath = string.sub(texPath, idx+1, -1);
    idx = string.find(texPath, "/", 1);
--    texPath = string.sub(texPath, idx+1, -1);
--    idx = string.find(texPath, "/", 1);
    return string.sub(texPath, idx+1, -1);
end

function ImportMatParam:AssemblePath(name)
    return self.SceneDirectory.."dress/XGYJ/"..name;
end

function ImportMatParam:LoadMaterialParamsConfig()
    if self.MaterialParamsConfig == "" then
      return;
    end
    local file =  venusjson.LaodJsonFile(self.MaterialParamsConfig);
    if file ~= nil then
      self.mat_params = file;
    else
        ERROR("config json file doesn't exist");
    end

    
    --- parse hierarchy path
    local idx = self.MaterialParamsConfig:match('^.*()/');
    local relativepath = string.sub(self.MaterialParamsConfig, 1, idx);
    local absolutepath = venuscore.IFileSystem:PathAssembly(relativepath);

    if absolutepath ~= nil then
        self.assetHierarchy["root"] = absolutepath;
        self.assetHierarchy["mat"] = absolutepath.."mat/";
        self.assetHierarchy["fbx"] = absolutepath.."fbx/";
        self.assetHierarchy["prefab"] = absolutepath.."prefab/";
        self.assetHierarchy["texture"] = absolutepath.."texture/";
    end
end


-------------------------------------------------------------------------------------------------------------
-- generate mat file
-------------------------------------------------------------------------------------------------------------


function ImportMatParam:_GenerateMatfiles()

    if self.MaterialParamsConfig == nil then
        ERROR(" Config file path not set");
        return;
    end

    if self.assetHierarchy["mat"] == nil then
        ERROR(" 'mat' folder doesn't exist");
        return;
    end

    
    local matFileList = self:_FetchMatList();
    
    for matName, matproperty in pairs(matFileList) do
        mattool:CreateMat(self.assetHierarchy["mat"]..matName);
        local materialEntity = apolloengine.MaterialEntity();
        local meta = apolloengine.MaterialMetadata(self.assetHierarchy["mat"]..matName);
        materialEntity:PushMetadata(meta);
        materialEntity:SetJobType(venuscore.IJob.JT_SYNCHRONOUS);
        materialEntity:CreateResource();


        local shaderPath = shaderMappingList[matproperty["_ShaderName"]] or shaderMappingList["cel_hair"];
        materialEntity:ChangeShaderPath(shaderPath);
        mattoolWrap.SaveOneMat(materialEntity, shaderPath);


        self:_EditMaterial(materialEntity, matproperty);
    end
    
end


function ImportMatParam:_EditMaterial(mat, matProperty)
    for name, value in pairs(matProperty) do
        local hasparam = mat:hasParameter(name);
        if hasparam then
            if value["type"] == "Float" then
                --mat:SetParameter(name, mathfunction.vector1(value["data"]));
                local data = mathfunction.vector1(value["data"]);
                mat:SetParameter(name, data);
            elseif value["type"] == "Vector" then
                 local d = value["data"];
                 local data = mathfunction.vector4(d[1],d[2],d[3],d[4]);
                 mat:SetParameter(name, data);
            elseif value["type"] == "Color" then
                 local d = value["data"];
                 local data = mathfunction.Color(d[1],d[2],d[3],d[4]);
                 mat:SetParameter(name, data);
            elseif value["type"] == "Texture" then
                local mipmap = (value["MipmapCount"]>1) and true or false;
                local wrapmode;
                if value["WrapMode"]=="Repeat" then
                    wrapmode = apolloengine.TextureEntity.TW_REPEAT;
                elseif value["WrapMode"]=="Clamp" then
                    wrapmode = apolloengine.TextureEntity.TW_CLAMP_TO_EDGE;
                elseif value["WrapMode"]=="Mirror" then
                    wrapmode = apolloengine.TextureEntity.TW_MIRRORED_REPEAT;
                end
                local filtermode;
                if value["FilterMode"]=="Bilinear" then
                    filtermode = apolloengine.TextureEntity.TF_LINEAR;
                elseif value["FilterMode"]=="Point" then
                    filtermode = apolloengine.TextureEntity.TF_NEAREST;
                elseif value["FilterMode"]=="Trilinear" then
                    filtermode = apolloengine.TextureEntity.TF_LINEAR_MIPMAP_LINEAR;
                end
                local anisoLevel = value["AnisoLevel"];
                local isSRGB = (string.find(value["GraphicsFormat"], "SRGB", 1)~=nil) and true or false;

                local imagePath = self.assetHierarchy["root"]..value["Path"];

                if venuscore.IFileSystem:isFileExist(imagePath) then
                    local metaPath = filelfs:GetTextureMeta(imagePath);
                    metaPath = filelfs:EditTextureMeta(imagePath,mipmap,wrapmode,filtermode,anisoLevel,isSRGB);
                    local rMetaPath = filelfs:ConverToRelativePath(metaPath);
                    local tex = apolloengine.TextureEntity();
                    tex:PushMetadata(apolloengine.TextureDescribeFileMetadata(rMetaPath));
                    --tex:SetKeepSource(true);
                    tex:SetJobType(venuscore.IJob.JT_SYNCHRONOUS);
                    tex:CreateResource();
                    mat:SetParameter(name, tex);
                    local data = mathfunction.vector4(value["ST"][1],value["ST"][2],value["ST"][3],value["ST"][4]);
                    if mat:hasParameter(name.."_ST") then
                        mat:SetParameter(name.."_ST", data);
                    end
                else
                    ERROR("Failed copy From *****" .. value["Path"])
                    ERROR("Failed copy to   *****" .. imagePath)
                end
            end
        end
    end

    mattoolWrap.SaveOneMat(mat);
end


function ImportMatParam:_FetchMatList()
    local MatList = {};

    for _, mesh in pairs(self.mat_params) do
        for mat, matproperty in pairs(mesh) do
            MatList[mat] = matproperty;
        end
    end

    return MatList;
end


function ImportMatParam:GetImportnnn()
    return self.tryImportParam;
end

function ImportMatParam:SetImportnnn(f)
    self.tryImportParam = f;
end

ImportMatParam:MemberRegister(
    "MaterialParamsConfig",
    venuscore.ScriptTypes.FilePathType(
        {"json"},
        nil,
        function(thiz, value)
            thiz.MaterialParamsConfig = value;
            thiz:LoadMaterialParamsConfig();
        end
));


ImportMatParam:MemberRegister("tryImportParam",
  venuscore.ScriptTypes.ImageButtonType(
  {
    textName = "Import Asset",
    icon = "comm:documents/icons/particular.png",
    size = {30,30},
    tip = "Import Asset ",
    sameline = false,
  },
  ImportMatParam.GetImportnnn,
  ImportMatParam.SetImportnnn
));




return ImportMatParam;