
local venuscore = require "venuscore"
local apolloengine = require "apolloengine"
local mathfunction = require "mathfunction"
local ContentManager = require "apolloengine.content.contentmanager"
local MotionSwitchLogic = venuscore.VenusBehavior:extend("MotionSwitchLogic");

local headDriverPath = "scrs:behavior/vtuber_behavior/bodymotiondriver/headdriver.lua"
local animationswitchpath = "scrs:behavior/animation/animationSwitch.lua"
local halfBodyPosePath = "scrs:behavior/vtuber_behavior/bodymotiondriver/halfBodyPose.lua"
local handPose3DPath = "scrs:behavior/vtuber_behavior/bodymotiondriver/handPose3D.lua"

MotionSwitchLogic.initHalfBodyBones = {};
table.insert(MotionSwitchLogic.initHalfBodyBones, "RightArm");
table.insert(MotionSwitchLogic.initHalfBodyBones, "RightForeArm");
table.insert(MotionSwitchLogic.initHalfBodyBones, "RightHand");
table.insert(MotionSwitchLogic.initHalfBodyBones, "LeftArm");
table.insert(MotionSwitchLogic.initHalfBodyBones, "LeftForeArm");
table.insert(MotionSwitchLogic.initHalfBodyBones, "LeftHand");

MotionSwitchLogic.initHeadBones = {};
table.insert(MotionSwitchLogic.initHeadBones, "Head");
table.insert(MotionSwitchLogic.initHeadBones, "HeadSke");
table.insert(MotionSwitchLogic.initHeadBones, "Spine1");
table.insert(MotionSwitchLogic.initHeadBones, "Spine2");
table.insert(MotionSwitchLogic.initHeadBones, "Spine3");

MotionSwitchLogic.initHandBones = {};
table.insert(MotionSwitchLogic.initHandBones, "RightHand");
table.insert(MotionSwitchLogic.initHandBones, "RightHandThumb1")
table.insert(MotionSwitchLogic.initHandBones, "RightHandThumb2")
table.insert(MotionSwitchLogic.initHandBones, "RightHandThumb3")
table.insert(MotionSwitchLogic.initHandBones, "RightHandThumb4")

table.insert(MotionSwitchLogic.initHandBones, "RightHandIndex1")
table.insert(MotionSwitchLogic.initHandBones, "RightHandIndex2")
table.insert(MotionSwitchLogic.initHandBones, "RightHandIndex3")
table.insert(MotionSwitchLogic.initHandBones, "RightHandIndex4")

table.insert(MotionSwitchLogic.initHandBones, "RightHandMiddle1")
table.insert(MotionSwitchLogic.initHandBones, "RightHandMiddle2")
table.insert(MotionSwitchLogic.initHandBones, "RightHandMiddle3")
table.insert(MotionSwitchLogic.initHandBones, "RightHandMiddle4")

table.insert(MotionSwitchLogic.initHandBones, "RightHandRing1")
table.insert(MotionSwitchLogic.initHandBones, "RightHandRing2")
table.insert(MotionSwitchLogic.initHandBones, "RightHandRing3")
table.insert(MotionSwitchLogic.initHandBones, "RightHandRing4")

table.insert(MotionSwitchLogic.initHandBones, "RightHandPinky1")
table.insert(MotionSwitchLogic.initHandBones, "RightHandPinky2")
table.insert(MotionSwitchLogic.initHandBones, "RightHandPinky3")
table.insert(MotionSwitchLogic.initHandBones, "RightHandPinky4")

table.insert(MotionSwitchLogic.initHandBones, "LeftHand");
table.insert(MotionSwitchLogic.initHandBones, "LeftHandThumb1")
table.insert(MotionSwitchLogic.initHandBones, "LeftHandThumb2")
table.insert(MotionSwitchLogic.initHandBones, "LeftHandThumb3")
table.insert(MotionSwitchLogic.initHandBones, "LeftHandThumb4")

table.insert(MotionSwitchLogic.initHandBones, "LeftHandIndex1")
table.insert(MotionSwitchLogic.initHandBones, "LeftHandIndex2")
table.insert(MotionSwitchLogic.initHandBones, "LeftHandIndex3")
table.insert(MotionSwitchLogic.initHandBones, "LeftHandIndex4")

table.insert(MotionSwitchLogic.initHandBones, "LeftHandMiddle1")
table.insert(MotionSwitchLogic.initHandBones, "LeftHandMiddle2")
table.insert(MotionSwitchLogic.initHandBones, "LeftHandMiddle3")
table.insert(MotionSwitchLogic.initHandBones, "LeftHandMiddle4")

table.insert(MotionSwitchLogic.initHandBones, "LeftHandRing1")
table.insert(MotionSwitchLogic.initHandBones, "LeftHandRing2")
table.insert(MotionSwitchLogic.initHandBones, "LeftHandRing3")
table.insert(MotionSwitchLogic.initHandBones, "LeftHandRing4")

table.insert(MotionSwitchLogic.initHandBones, "LeftHandPinky1")
table.insert(MotionSwitchLogic.initHandBones, "LeftHandPinky2")
table.insert(MotionSwitchLogic.initHandBones, "LeftHandPinky3")
table.insert(MotionSwitchLogic.initHandBones, "LeftHandPinky4")

function MotionSwitchLogic:new()
    self.curAniIdx = 0;
    self.animationIndex = 0;

    self.prevMotionIndex = 0;

    self.halfBodyPoseDriverScript = nil;
    self.handPose3DriverScript = nil;

    self.initHalfBodyRot = {};
    self.initHandRot = {};
    self.initHeadRot = {};
    self.UpdateEnable = true;
    self.dftAnimIdx = 0;
    
    self.dftplayed = false;
    
    self.RightNipple = nil;
    self.LeftNipple = nil;
end

function MotionSwitchLogic:_OnAwake()
    if self.headdriver ~= nil then
        self.HeadDriverScript = self.headdriver:GetScript(headDriverPath);    
        if self.HeadDriverScript ~= nil then
            local animationComp = self.Node:GetComponent(apolloengine.Node.CT_ANIMATION);
            if animationComp ~= nil then
                if self.HeadDriverScript.HeadSke ~= nil then
                    local headnode = self.HeadDriverScript.HeadSke.ChildNodes[1];
                    local trans = headnode:GetComponent(apolloengine.Node.CT_TRANSFORM);
                    animationComp:AddFullMask(trans);
                end
                if self.RightNipple ~= nil then
                    local trans = self.RightNipple:GetComponent(apolloengine.Node.CT_TRANSFORM);
                    animationComp:AddFullMask(trans);
                end
                if self.LeftNipple ~= nil then
                    local trans = self.LeftNipple:GetComponent(apolloengine.Node.CT_TRANSFORM);
                    animationComp:AddFullMask(trans);
                end
            end
        end
    end
    if self.animationswitch ~= nil then
        self.AnimationSwitchScript = self.animationswitch:GetScript(animationswitchpath);
    end
    local scriptCom = self.Node:GetComponent(apolloengine.Node.CT_SCRIPT);
    self.halfBodyPoseDriverScript = scriptCom:GetScript(halfBodyPosePath);
    self.handPose3DriverScript = scriptCom:GetScript(handPose3DPath);
end

function MotionSwitchLogic:_OnStart()
    local animationComp = self.Node:GetComponent(apolloengine.Node.CT_ANIMATION);

    if self.halfBodyPoseDriverScript ~= nil then
        self:GetInitHalfBodyPoses();
    end
    if self.handPose3DriverScript ~= nil then
        self:GetInitHandPoses();
    end
    if self.HeadDriverScript ~= nil then
        self:GetInitHeadPoses();
    end
    
    local animationComp = self.Node:GetComponent(apolloengine.Node.CT_ANIMATION);
    local aninames = nil;
    if self.AnimationSwitchScript then
      aninames = self.AnimationSwitchScript.animationInfo["name"];
    end
    if self.dftAnimIdx>0 and aninames and aninames[self.dftAnimIdx] then
      animationComp:Loop(aninames[self.dftAnimIdx],true);
    end
end

function MotionSwitchLogic:_OnUpdate(def)
    if self.AnimationSwitchScript == nil then
        WARNING("MotionSwitchLogic AnimationSwitchScript is nil");
        return;
    end

    if self.dftplayed==false then
       if _KRATOSEDITOR then
        self.animationIndex = self.dftAnimIdx
       end
       self:PlayAnimation(self.dftAnimIdx);
       self.dftplayed = true;
    end

    if _KRATOSEDITOR then
       self:PlayAnimation(self.animationIndex);
    end
    
    if self.UpdateEnable then
      self:UpdateMotion(def);
    end
end

function MotionSwitchLogic:GetInitHalfBodyPoses()
    for i = 1, #self.initHalfBodyBones do
        if self.halfBodyPoseDriverScript[self.initHalfBodyBones[i]] then
            local trans = self.halfBodyPoseDriverScript[self.initHalfBodyBones[i]]:GetComponent(apolloengine.Node.CT_TRANSFORM);
            self.initHalfBodyRot[i] = trans:GetLocalRotation();
        end
    end
end

function MotionSwitchLogic:GetInitHandPoses()
    for i = 1, #self.initHandBones do
        if self.handPose3DriverScript[self.initHandBones[i]] then
            local trans = self.handPose3DriverScript[self.initHandBones[i]]:GetComponent(apolloengine.Node.CT_TRANSFORM);
            self.initHandRot[i] = trans:GetLocalRotation();
        end
    end
end

function MotionSwitchLogic:GetInitHeadPoses()
  if self.HeadDriverScript.HeadNode and self.HeadDriverScript.HeadSke then
    local headtrans = self.HeadDriverScript.HeadNode:GetComponent(apolloengine.Node.CT_TRANSFORM);
    self.initHeadRot[1] = headtrans:GetLocalRotation();
    local sketrans = self.HeadDriverScript.HeadSke:GetComponent(apolloengine.Node.CT_TRANSFORM);
    self.initHeadRot[2] = sketrans:GetLocalRotation();
    for i = 3, #self.initHeadBones do
      if self.HeadDriverScript[self.initHeadBones[i]] then
        local trans = self.HeadDriverScript[self.initHeadBones[i]]:GetComponent(apolloengine.Node.CT_TRANSFORM);
        self.initHeadRot[i] = trans:GetLocalRotation();
       end
    end
  end
end

function MotionSwitchLogic:RestoreHalfBoneBones()
  if self.handPose3DriverScript then
    for i = 1, #self.initHalfBodyBones do
        local trans = self.halfBodyPoseDriverScript[self.initHalfBodyBones[i]]:GetComponent(apolloengine.Node.CT_TRANSFORM);
        trans:SetLocalRotation(self.initHalfBodyRot[i]);
    end
  end
end

function MotionSwitchLogic:RestoreHandBones()
  if self.handPose3DriverScript then
    for i = 1, #self.initHandBones do
        local trans = self.handPose3DriverScript[self.initHandBones[i]]:GetComponent(apolloengine.Node.CT_TRANSFORM);
        trans:SetLocalRotation(self.initHandRot[i]);
    end
  end
end

function MotionSwitchLogic:RestoreHeadBones() 
  if self.HeadDriverScript.HeadNode and self.HeadDriverScript.HeadSke then
    local headtrans = self.HeadDriverScript.HeadNode:GetComponent(apolloengine.Node.CT_TRANSFORM);
    headtrans:SetLocalRotation(self.initHeadRot[1]);
    local sketrans = self.HeadDriverScript.HeadSke:GetComponent(apolloengine.Node.CT_TRANSFORM);
    sketrans:SetLocalRotation(self.initHeadRot[2]);
    for i = 3, #self.initHeadBones do
      local trans = self.HeadDriverScript[self.initHeadBones[i]]:GetComponent(apolloengine.Node.CT_TRANSFORM);
      trans:SetLocalRotation(self.initHeadRot[i]);
    end
  end
end

function MotionSwitchLogic:UpdateMotion(def)
  -- halfBodyPose and handPose3D update
  if self.curAniIdx == -1 then
    if self.HeadDriverScript then self.HeadDriverScript.AddOnAnimaion = false; end
    if self.halfBodyPoseDriverScript and self.handPose3DriverScript then
        self.halfBodyPoseDriverScript:UpdateHalfBody();
        self.handPose3DriverScript:UpdateHandPose();
    else
        WARNING("halfBodyPoseDriverScript or handPose3DriverScript is nil");
    end
  -- headdriver update
  elseif self.curAniIdx == 0 then
    if self.HeadDriverScript ~= nil then
        self.HeadDriverScript.AddOnAnimaion = false;
        self.HeadDriverScript:UpdateMotion(def);
    else
        WARNING("headDriverScript is nil");
    end
  elseif self.curAniIdx > 0 then
    if self.HeadDriverScript then self.HeadDriverScript.AddOnAnimaion = true; end
    if self.curAniIdx~=self.dftAnimIdx then
      local isFinish = self.AnimationSwitchScript:IsCurStatePlayFinish();
      if isFinish == true then
          self:PlayAnimation(self.dftAnimIdx);
          if _KRATOSEDITOR then
              self.animationIndex = self.dftAnimIdx;
          end
      end
    end
  end
end

function MotionSwitchLogic:PlayAnimation(animIdx)
    if self.AnimationSwitchScript:IsInited()==false then
        WARNING("AnimationSwitchScript is not Inited!");
        return;
    end
  
    if self.curAniIdx == animIdx then
        return;
    end
    local animationComp = self.Node:GetComponent(apolloengine.Node.CT_ANIMATION);
    
    if animIdx <= 0 then
        if animationComp ~= nil then
            animationComp:ResetAll(false);
            local trans_root = self.RootNode:GetComponent(apolloengine.Node.CT_TRANSFORM);
            animationComp:SetDelegateMask(trans_root, true);
        end
        self.AnimationSwitchScript:Stop();
        self.AnimationSwitchScript.curState = nil;
        if animIdx == 0 then
            self:RestoreHandBones();
            self:RestoreHalfBoneBones();
        end
    elseif animIdx > 0 then
        self.AnimationSwitchScript:SwitchAnimationByIdx(animIdx);
        local bRunning = self.AnimationSwitchScript:IsRunning();
       
        if not bRunning then
            WARNING("StateMachine is not running, need to start!");
        end

        if self.curAniIdx<=0  or not bRunning then
            self.AnimationSwitchScript:Run();
        end
    end

    self.curAniIdx = animIdx;
end

function MotionSwitchLogic:PlayAnimationFrame(aniIdx, frameIdx)
    if self.AnimationSwitchScript:IsAniAndFrameValid(aniIdx) == false then
        aniIdx = self.dftAnimIdx;
        frameIdx = 1;
        ERROR("vtuber MotionSwitchLogic PlayAnimationFrame Error aniIdx");
    end
    self.AnimationSwitchScript:Stop();
    self.AnimationSwitchScript:Play(aniIdx);
    self.AnimationSwitchScript:Reset(aniIdx, frameIdx);
    self.AnimationSwitchScript:Pause(aniIdx, true);
end


MotionSwitchLogic:MemberRegister("dftAnimIdx");
MotionSwitchLogic:MemberRegister("animationIndex");

MotionSwitchLogic:MemberRegister("RootNode",
    venuscore.ScriptTypes.ReferenceType(
    apolloengine.Node:RTTI()
));

MotionSwitchLogic:MemberRegister("LeftShoulderNode",
    venuscore.ScriptTypes.ReferenceType(
    apolloengine.Node:RTTI()
));

MotionSwitchLogic:MemberRegister("RightShoulderNode",
    venuscore.ScriptTypes.ReferenceType(
    apolloengine.Node:RTTI()
));

MotionSwitchLogic:MemberRegister("RightNipple",
    venuscore.ScriptTypes.ReferenceType(
    apolloengine.Node:RTTI()
));

MotionSwitchLogic:MemberRegister("LeftNipple",
    venuscore.ScriptTypes.ReferenceType(
    apolloengine.Node:RTTI()
));

MotionSwitchLogic:MemberRegister("headdriver",
venuscore.ScriptTypes.ReferenceType(
apolloengine.ScriptComponent:RTTI()
));

MotionSwitchLogic:MemberRegister("animationswitch",
venuscore.ScriptTypes.ReferenceType(
apolloengine.ScriptComponent:RTTI()
));

return MotionSwitchLogic;
