
local apolloengine  = require "apolloengine"
local mathfunction  = require "mathfunction"
local venuscore     = require "venuscore"
local defined       = require "apolloutility.defiend"
local BundleSystem = require "venuscore.bundle.bundlesystem"
local CreateComponentCmd = require "window.editor.command.commands.node.create_component_command"
local AddScriptCommand = require "window.editor.command.commands.scriptcomponent.add_script_command"
local CommandManager = require "window.editor.command.command_manager"
local AddScriptCmd = require "window.editor.command.commands.scriptcomponent.add_script_command"
local SetScriptValueByKeys = require "window.editor.command.commands.scriptcomponent.set_script_value_by_keys_command"
local CopyVtuberBehavior = venuscore.VenusBehavior:extend("CopyVtuberBehavior")


function _CopyFaceMgr(self,cpto,cpfrom)
    if cpto~= nil then
        self:CopyParams(cpto,"bsPairPath",cpfrom.bsPairPath)
        self:CopyComponent(cpto,cpfrom.morph,apolloengine.Node.CT_MORPH,"morph")
        self:CopyNode(cpto,cpfrom.boneRootNode,"boneRootNode")
    end
end
function _CopyHairMgr(self,cpto,cpfrom)

end
function _CopyDressMgr(self,cpto,cpfrom)

end
function _CopyMatMgr(self,cpto,cpfrom)

end
function _CopyTrackBS(self,cpto,cpfrom)
    self:CopyParams(cpto,"Sending",cpfrom.Sending)
    self:CopyParams(cpto,"Receiving",cpfrom.Receiving)
    self:CopyParams(cpto,"BsCorrectConfig",cpfrom.BsCorrectConfig)

end
function _CopyEyeRotate(self,cpto,cpfrom)
    self:CopyParams(cpto,"Sending",cpfrom.Sending)
    self:CopyParams(cpto,"Receiving",cpfrom.Receiving)
    self:CopyParams(cpto,"NodesConfig",cpfrom.NodesConfig)

end
function _CopyHeadDriver(self,cpto,cpfrom)


end
function _CopyVtuberLoader(self,cpto,cpfrom)

end
function _CopyJewelleryMgr(self,cpto,cpfrom)

end
function _CopyAnimationSwitch(self,cpto,cpfrom)
    self:CopyParams(cpto,"animationInfoPath",cpfrom.animationInfoPath)
end
function _CopyMotionSwitchLogic(self,cpto,cpfrom)
    self:CopyParams(cpto,"dftAnimIdx",cpfrom.dftAnimIdx)
    self:CopyParams(cpto,"animationIndex",cpfrom.animationIndex)
end
function _CopyDisneyIdentify(self,cpto,cpfrom)

end
function _CopyViewCFG(self,cpto,cpfrom)

end

local cp_vyou = {
    ["headdriver"] = _CopyHeadDriver,
    ["vtuberLoader"] = _CopyVtuberLoader,
    ["eye_rotate_behavior"] = _CopyEyeRotate,
    ["vtuberFaceMgr_behavior"] = _CopyFaceMgr,
    ["vtuberHairMgr_behavior"] = _CopyHairMgr,
    ["animationSwitch"] = _CopyAnimationSwitch,
    ["vtuberDressMgr_behavior"] = _CopyDressMgr,
    ["track_blendshape_behavior"] = _CopyTrackBS,
    ["vtuberMaterialMgr_behavior"] = _CopyMatMgr,
    ["motionSwitchLogic"] = _CopyMotionSwitchLogic,
    ["vtuberJewelleryMgr_behavior"] = _CopyJewelleryMgr,
}

local cp_timeprincess = {
    ["headdriver"] = _CopyHeadDriver,
    ["eye_rotate_behavior"] = _CopyEyeRotate,
    ["vtuberFaceMgr_behavior"] = _CopyFaceMgr,
    ["animationSwitch"] = _CopyAnimationSwitch,
    ["vtuberDressMgr_behavior"] = _CopyDressMgr,
    ["track_blendshape_behavior"] = _CopyTrackBS,
    ["vtuberMaterialMgr_behavior"] = _CopyMatMgr,
    ["motionSwitchLogic"] = _CopyMotionSwitchLogic,
    ["vtuber_viewCFG"] = _CopyViewCFG,
    ["vtuber_disney_identify"] = _CopyDisneyIdentify,
}


function CopyVtuberBehavior:new()
    self.copyFromNode = nil
    self.copyToNode = nil
    self.tryCopy = false
end

function CopyVtuberBehavior:_OnAwake()
    self.copyFinished = self.tryCopy
end

function CopyVtuberBehavior:_OnUpdate()
    if self.tryCopy ~= self.copyFinished then
        self:_DoCopy()
        self.copyFinished = self.tryCopy
    end
end

function CopyVtuberBehavior:_DoCopy()
    if self.copyFromNode ~= nil and self.copyToNode ~= nil then
     
        local cmdsc = CreateComponentCmd(self.copyToNode:GetContentPath(),"ScriptComponent");
        local ressc = CommandManager:DoIt(cmdsc);

        local cmdani = CreateComponentCmd(self.copyToNode:GetContentPath(),"AnimationComponent");
        local resani = CommandManager:DoIt(cmdani);

        local scriptCom = self.copyFromNode:GetComponent(apolloengine.Node.CT_SCRIPT);
        local paraSet = scriptCom.Instances;

        local cpFuncs = cp_timeprincess   --cp_vyou
        for bvrName,cp in pairs(cpFuncs) do
            for bvrPath, cpfrom in pairs(paraSet) do
                if string.find(tostring(bvrPath),bvrName) then 
                    local cpto = self:_AddBehavior(tostring(bvrPath))
                    --cp(self,cpto,cpfrom)
                    break
                end
              end
        end
    end
end


function CopyVtuberBehavior:_AddBehavior(path)
    local scriptCom =  self.copyToNode:GetComponent(apolloengine.Node.CT_SCRIPT);
    local cmd = AddScriptCmd(scriptCom:GetContentPath(), path);
    CommandManager:DoIt(cmd);
    local bvrIns = scriptCom:GetScript(path);
    return bvrIns
end

function CopyVtuberBehavior:GetChildNode(node,childName)
    local childs = node:GetChildrens();
    for _,v in pairs(childs) do
      local name = v:GetName();
      if name == childName then
        return v
      end
    end
end


function CopyVtuberBehavior:CopyComponent(cpto,targetComponent,type,key)
    ERROR("_CopyComponent : " .. tostring(targetComponent:GetHostNode():GetName()))
    local node = self:GetChildNode(self.copyToNode,targetComponent:GetHostNode():GetName())
    local component = node:GetComponent(type)
    self:CopyParams(cpto,key,component)
end


function CopyVtuberBehavior:CopyNode(cpto,targetNode,key)
    ERROR("_CopyNode : " .. tostring(targetNode:GetName()))
    local node = self:GetChildNode(self.copyToNode,targetNode:GetName())
    self:CopyParams(cpto,key,node)
end

function CopyVtuberBehavior:CopyParams(behavior,key,value)
    local params = {};
    table.insert(params, key); 
    local cmd = SetScriptValueByKeys(
        behavior._hostID,
        behavior:GetTypeName(),
        params,
        value
    );
    CommandManager:DoIt(cmd);
end
  

function CopyVtuberBehavior:GetTryCopy()
    return self.tryCopy;
end

function CopyVtuberBehavior:SetTryCopy(i)
    self.tryCopy = i;
end

CopyVtuberBehavior:MemberRegister(
    "copyFromNode",
    venuscore.ScriptTypes.ReferenceType(
            apolloengine.Node:RTTI()
    )
)

CopyVtuberBehavior:MemberRegister(
    "copyToNode",
    venuscore.ScriptTypes.ReferenceType(
            apolloengine.Node:RTTI()
    )
)


CopyVtuberBehavior:MemberRegister("tryCopy",
  venuscore.ScriptTypes.ImageButtonType(
  {
    textName = "copy",
    icon = "comm:documents/icons/play.png",
    size = {30,30},
    tip = "copy",
    sameline = false,
  },
  CopyVtuberBehavior.GetTryCopy,
  CopyVtuberBehavior.SetTryCopy
));


return CopyVtuberBehavior;

