local venuscore = require "venuscore"
local venusjson = require "venusjson"
local apolloengine = require "apolloengine"
local libmasque = require "libmasque"
local Hub = require "behavior.vtuber_behavior.vtuber_msg_hub"
local ValueTransit = require "behavior.tracking_behavior.value_transit"
local likeapp = require "likeapp"
local defined = require "behavior.tracking_behavior.defined"

local TrackBlendshape = venuscore.VenusBehavior:extend("TrackBlendshape")

local use_arkit = false
local init_arkit = false

function TrackBlendshape:new()
  self.BsCorrectConfig = ""
  self.bs_correct = {}
  self.Sending = false
  self.Receiving = false
  self.EnableUpdate = true;
  self.weight_dict = {}
  self.bsTransit = {}
  for name, region in pairs(defined.confidence_regions) do
    if region.all_low_transit then
      self.bsTransit[name] = ValueTransit(region.all_low_transit.time)
    end
  end
end

function TrackBlendshape:_OnAwake()
  if not self.Tracking then
    if self.Node:HaveComponent(apolloengine.Node.CT_TRACK) then
      self.Tracking = self.Node:GetComponent(apolloengine.Node.CT_TRACK);
    end
  end
  self.morphers = {}
  self:GetSubMorpherComponents(self.Node)
  self:LoadBsCorrectConfig()
end

function TrackBlendshape:_OnUpdate(delta)

  if self.EnableUpdate~=true then
    return;
  end
  if not self.Tracking and not self.Receiving then
    return
  end

  if _KRATOSEDITOR then
    use_arkit = false
  else
    use_arkit = likeapp.AI:GetARKitFlag()
  end

  if init_arkit == false and use_arkit then
    self.Tracking:GetHostNode().Active=false
    init_arkit = true
  end

  self:UpdateBsWeight(delta)

  if _KRATOSEDITOR then
    self.morphers = {}
    self:GetSubMorpherComponents(self.Node)
  end
  for _, morpher_info in ipairs(self.morphers) do
    if morpher_info.matched == nil or #morpher_info.matched == 0 then
      morpher_info.matched = self:FindMatchedTrackBs(morpher_info.morpher)
    end
    for _, matched_data in ipairs(morpher_info.matched) do
      
      local w = self.weight_dict[matched_data.track]
      if w == nil then
        w = 0
      end

      if _KRATOSEDITOR then
        self:UpdateChannelPercent(morpher_info.morpher:GetContentPath(),
        matched_data.group, matched_data.channel, self.weight_dict[matched_data.track])
      else
        morpher_info.morpher:SetChannelPercents(
          matched_data.group, 
          matched_data.channel, 
          w
        )
      end
    end
  end
end

-- function printDict(weight_dict)
--   for k, v in pairs(weight_dict) do
--     ERROR(string.format( "WJC: [%s, %f]!", k, v))
--   end
-- end

local arkitBlendshapeName = {
  "browDown_L",
  "browDown_R",
  "browInnerUp" ,
  "browOuterUp_L",
  "browOuterUp_R",
  "cheekPuff",
  "cheekSquint_L", --
  "cheekSquint_R", -- 
  "eyeBlink_L",
  "eyeBlink_R",
  "eyeLookDown_L",
  "eyeLookDown_R",
  "eyeLookIn_L",
  "eyeLookIn_R",
  "eyeLookOut_L",
  "eyeLookOut_R",
  "eyeLookUp_L",
  "eyeLookUp_R",
  "eyeSquint_L",
  "eyeSquint_R",
  "eyeWide_L",
  "eyeWide_R",
  "jawForward",
  "jawLeft",
  "jawOpen",
  "jawRight",
  "mouthClose",
  "mouthDimple_L",
  "mouthDimple_R",
  "mouthFrown_L",
  "mouthFrown_R",
  "mouthFunnel",
  "mouthLeft",
  "mouthLowerDown_L",
  "mouthLowerDown_R",
  "mouthPress_L",--
  "mouthPress_R",--
  "mouthPucker",
  "mouthRight",
  "mouthRollLower",
  "mouthRollUpper",
  "mouthShrugLower",
  "mouthShrugUpper",
  "mouthSmile_L",
  "mouthSmile_R",
  "mouthStretch_L",--
  "mouthStretch_R",--
  "mouthUpperUp_L",
  "mouthUpperUp_R",
  "noseSneer_L",
  "noseSneer_R",
  "tongueOut"--
}

function TrackBlendshape:refreshWeightFromARKit() 
  local arkitBS = likeapp.AI:GetARKitParams()
  --  ERROR(string.format("WJC: blendshape length = %d", #arkitBS))
  for i = 1, #arkitBlendshapeName do
    local weight = arkitBS[i]
    self.weight_dict[arkitBlendshapeName[i]] = weight
  end
end

if _KRATOSEDITOR then
  local CommandManager = require "window.editor.command.command_manager"
  local SetMorpherChannelPercents = require "window.editor.command.commands.morphcomponent.morphcomponent_setchannelpercents_command"
  function TrackBlendshape:UpdateChannelPercent(uuid, grp, channel, percent)
     local cmdSetPercent = SetMorpherChannelPercents(
        uuid,
        grp, 
        channel, 
        percent
      )  
      CommandManager:DoIt(cmdSetPercent)
  end
end

function TrackBlendshape:UpdateBsWeight(delta)
  if self.Receiving then
    self:UpdateReceivedBsWeight()
    return
  end

  if use_arkit then
    local faceNum = likeapp.AI:GetARKitFaceNum()
    if faceNum > 0 then
      self:refreshWeightFromARKit()
    else
      self:UpdateDefaultBsWeight()
    end
  else
    local isTracked = self.Tracking:IsTracked()
    if isTracked then
      self:UpdateTrackedBsWeight()
    else
      self:UpdateDefaultBsWeight()
    end
  
    self:UpdateWeightByConfidence(delta, isTracked)
  end

  if self.Sending then
    self:SendBsWeight()
  end
end

function TrackBlendshape:UpdateWeightByConfidence(delta, isTracked)
  local bvt_confidence = self.Tracking:GetConfidence()
  for name, region in pairs(defined.confidence_regions) do
    local transit_status = TransitStatus.START
    if isTracked then
      if self:AllLowConfidence(bvt_confidence, region.keys) then
        transit_status = TransitStatus.TRANSIT
      else
        transit_status = TransitStatus.START
        if region.one_low_copy then
          for key, blendshapes in pairs(region.one_low_copy) do
            if self:IsLowConfidence(bvt_confidence, key) then
              self:UpdateWeightByCopy(blendshapes)
            end
          end
        end
      end
    else
      transit_status = TransitStatus.END
    end

    local region_weight = {}
    if region.all_low_transit then
      for _, key in ipairs(region.all_low_transit.blendshape) do
        region_weight[key] = self.weight_dict[key]
      end

      self.bsTransit[name]:UpdateValue(delta, transit_status, region_weight)
      region_weight = self.bsTransit[name]:GetValue()
  
      for _, key in ipairs(region.all_low_transit.blendshape) do
        self.weight_dict[key] = region_weight[key]
      end
    end
  end
end

function TrackBlendshape:UpdateWeightByCopy(blendshape_pairs)
  for bs, target in ipairs(blendshape_pairs) do
    if self.weight_dict[bs] and self.weight_dict[target] then
      self.weight_dict[bs] = self.weight_dict[target]
    end
  end
end

function TrackBlendshape:UpdateDefaultBsWeight()
  for i = 1, self.Tracking:GetBlendshapeNumber() do     
    local name = self.Tracking:GetBlendshapeName(i - 1)
    self.weight_dict[name] = 0
  end
  if self.isSleep then
    self.weight_dict["eyeBlink_L"] = 1.0;
    self.weight_dict["eyeBlink_R"] = 1.0;
  end
end

function TrackBlendshape:UpdateReceivedBsWeight()
  local msg = Hub:Get(Hub.ReceivingId)
  for i = 1, #Hub.BSMap do
    self.weight_dict[Hub.BSMap[i]] = 0.0
  end
  if msg.bsWeightList ~= nil then
    if #msg.bsWeightList ~= #Hub.BSMap then
      ERROR(string.format(
        "SUNTYLOG: TrackBlendshape:UpdateReceivedBsWeight: unexpected bs-list-length received: %d %d",
        #msg.bsWeightList,
        #Hub.BSMap
      ))
    end
    for i = 1, #msg.bsWeightList do
      local weight = msg.bsWeightList[i]
      local name = Hub.BSMap[i]
      if name ~= nil then
        self.weight_dict[name] = weight
      end
    end
  else
    ERROR(string.format(
      "SUNTYLOG: TrackBlendshape:UpdateReceivedBsWeight: nil bs-list received"
    ))
  end
end

function TrackBlendshape:SendBsWeight()
  local weight_list = {}
  for i = 1, #Hub.BSMap do
    weight_list[i] = 0.0
  end
  for name, weight in pairs(self.weight_dict) do
    local inx = Hub.BSRevMap[name]
    if inx == nil then
      ERROR(string.format(
        "SUNTYLOG: TrackBlendshape:SendBsWeight: unexpected bs-name: %s",
        name
      ))
    else
      weight_list[inx] = weight
    end
  end
  Hub:PutBlendShapeWeights(Hub.SendingId, weight_list)
end

function TrackBlendshape:UpdateTrackedBsWeight()

  for i = 1, self.Tracking:GetBlendshapeNumber() do
    local name = self.Tracking:GetBlendshapeName(i - 1)
    local weight = self.Tracking:GetBlendshapeWeight(i - 1)
    local weight_avg = self.Tracking:GetBlendshapeWeightAvg(i - 1)
    self.weight_dict[name] = self:CorrectBsWeight(name, weight, weight_avg)
  end

  self:ProcessSymmetry()
  self:ProcessCorrelation()
end

function TrackBlendshape:GetSubMorpherComponents(node)
  local sub_nodes = node:GetChildrens()
  for _, sub_node in ipairs(sub_nodes) do
    if sub_node:HaveComponent(apolloengine.Node.CT_MORPH) then
      table.insert(self.morphers, { morpher = sub_node:GetComponent(apolloengine.Node.CT_MORPH) })
    end
    self:GetSubMorpherComponents(sub_node)
  end
end

function TrackBlendshape:FindMatchedTrackBs(morpher)
  local res = {}
  local num_group = morpher:GetNumberOfGroup()

  for i = 1, num_group do
    local group_index = i-1
    local group_name = morpher:GetGroupName(group_index)

    local num_channels = morpher:GetNumberOfChannel(group_index)

    for j = 1, num_channels do
      local channel_index = j-1
      local channel_name = morpher:GetChannelName(group_index, channel_index)
      local track_name = self:FindMatchedTrackBsName(channel_name)
      if track_name ~= nil then
        table.insert(res, {track = track_name, group = group_index, channel = channel_index})
      end
    end

  end
  
  return res
end

function TrackBlendshape:FindMatchedTrackBsName(name)
  local morpher_lower = name:lower()
  local res = {len = 0, track = nil}
  for track_name, track_weight in pairs(self.weight_dict) do
    local track_lower = track_name:lower()
    if morpher_lower:find(track_lower, 1, true) ~= nil then
      local len = string.len(track_lower)
      if len > res.len then
        res.len = len
        res.track = track_name
      end
    end
  end
  return res.track
end

function TrackBlendshape:LoadBsCorrectConfig()
  if self.BsCorrectConfig == "" then
    return
  end
  local file =  venusjson.LaodJsonFile(self.BsCorrectConfig);
  if file ~=nil then 
    self.bs_correct = file
  end
end

function TrackBlendshape:CorrectBsWeight(name, weight, weight_avg)
  if self.bs_correct[name] then
    local min_weight = self.bs_correct[name].min_weight or 0
    local max_weight = self.bs_correct[name].max_weight or 1
    local correct = self.bs_correct[name].correct or 1
    local use_avg = self.bs_correct[name].use_avg

    if use_avg == "min" then
      min_weight = math.max(weight_avg, min_weight)
    elseif use_avg == "max" then
      max_weight = math.min(weight_avg, max_weight)
    end

    if weight < min_weight then
      weight = 0
    elseif weight > max_weight then
      weight = correct
    elseif min_weight == max_weight then
      if use_avg == "min" then
        weight = 0
      elseif use_avg == "max" then
        weight = correct
      else
        weight = 0  --default
      end 
    else
      weight = (weight - min_weight) / (max_weight - min_weight) * correct
    end

    weight = self:MakeValid(weight)
  end
  return weight
end

function TrackBlendshape:ProcessSymmetry()
  if self.bs_correct.symmetry then
    for _, element in ipairs(self.bs_correct.symmetry) do
      if element.pair and #element.pair == 2 then
        local delta = element.threshold or 2
        local name_1 = element.pair[1]
        local name_2 = element.pair[2]
        if self.weight_dict[name_1] and self.weight_dict[name_2] then
          local weight = (self.weight_dict[name_1] + self.weight_dict[name_2]) / 2
          if math.abs(self.weight_dict[name_1] - self.weight_dict[name_2]) < delta then
            self.weight_dict[name_1] = weight
            self.weight_dict[name_2] = weight
          end
        end
      end
    end
  end
end

function TrackBlendshape:ProcessCorrelation()
  if self.bs_correct.correlate then
    for bs, element in pairs(self.bs_correct.correlate) do
      if self.weight_dict[bs] then
        if element.mul and self.weight_dict[element.mul] then
          self.weight_dict[bs] = self.weight_dict[bs] * self.weight_dict[element.mul]
        end
        if element.mul_reverse and self.weight_dict[element.mul_reverse] then
          self.weight_dict[bs] = self.weight_dict[bs] * (1 - self.weight_dict[element.mul_reverse])
        end
        if element.weighted_sum then
          self.weight_dict[bs] = 0
          for target_bs, target_rate in pairs(element.weighted_sum) do
            if self.weight_dict[target_bs] then
              self.weight_dict[bs] = self.weight_dict[bs] + target_rate * self.weight_dict[target_bs]
            end
          end
        end
        if element.less_plus then
          for target_bs, target_plus in pairs(element.less_plus) do
            if self.weight_dict[target_bs] then
              self.weight_dict[bs] = math.min(self.weight_dict[bs], target_plus + self.weight_dict[target_bs])
            end
          end
        end
        if element.more_clamp and self.weight_dict[bs] > element.more_clamp then
          self.weight_dict[bs] = 1.0
        end
        if element.equal and self.weight_dict[element.equal] then
          self.weight_dict[bs] = self.weight_dict[element.equal]
        end

        self.weight_dict[bs] = self:MakeValid(self.weight_dict[bs])
      end
    end
  end
end

function TrackBlendshape:MakeValid(weight)
  if weight < 0 then
    weight = 0
  elseif weight > 1 then
    weight = 1
  elseif weight ~= weight then  --nan
    weight = 0  --default
  end
  return weight
end

function TrackBlendshape:IsLowConfidence(bvt_confidence, key)
  return bvt_confidence[key] < defined.confidence_threshold
end

function TrackBlendshape:AllLowConfidence(bvt_confidence, keys)
  for _, key in ipairs(keys) do
    if not self:IsLowConfidence(bvt_confidence, key) then
      return false
    end
  end
  return true
end

function TrackBlendshape:GetBlendShapeWeights()
  return self.weight_dict
end

TrackBlendshape:MemberRegister(
  "Tracking",
  venuscore.ScriptTypes.ReferenceType(
    libmasque.TrackingComponent:RTTI()
  )
)
TrackBlendshape:MemberRegister("BsCorrectConfig",  
  venuscore.ScriptTypes.FilePathType(
    {"json"},
    nil,
    function(thiz, value)
        thiz.BsCorrectConfig = value
        thiz:LoadBsCorrectConfig()
    end
))


TrackBlendshape:MemberRegister("Sending");
TrackBlendshape:MemberRegister("Receiving");

return TrackBlendshape
