local venuscore = require "venuscore"
local venusjson = require "venusjson"
local apolloengine = require "apolloengine"
local mathfunction = require "mathfunction"
local libmasque = require "libmasque"
local cv = require "computervisionfunction"
local quatfilter = require "math.lowpassfilter.oneeurofilter_quat"

local TrackHeadRot = venuscore.VenusBehavior:extend("TrackHeadRot");

function TrackHeadRot:new()
  self.recognition = nil;
 
end

function TrackHeadRot:_OnAwake()
  self.Trans = self.Node:GetComponent(apolloengine.Node.CT_TRANSFORM);
  self.startrot = self.Trans:GetWorldRotation(); 
  self.starTime = venuscore.ITimerSystem:GetTimevalue();
  self.SmoothFactorA  = 3;   
  self.SmoothFactorB  = 1; 
  self.SmoothFactorC  = 2; 
end

function TrackHeadRot:_OnUpdate(delta)
  
  if self.recognition  then
    local results = self.recognition:GetResult();
    if results[1]~=nil then
    
      local pitch = math.rad(results[1][1][109 * 2-1]);
      local yaw = math.rad(results[1][1][110 * 2-1]);
      local roll = math.rad(results[1][1][111 * 2-1]);
      local rotquat = mathfunction.Quaternion();
      rotquat:YawPitchRoll(yaw,pitch,roll);
      
      
      if self.quatFilter == nil then    
        self.quatFilter = quatfilter(venuscore.ITimerSystem:GetTimevalue() - self.starTime,rotquat,self.SmoothFactorA,self.SmoothFactorB,self.SmoothFactorC);
      else
        rotquat = self.quatFilter:filter(venuscore.ITimerSystem:GetTimevalue() - self.starTime, rotquat);
      end
      
      self.Trans = self.Node:GetComponent(apolloengine.Node.CT_TRANSFORM);
      self.Trans:SetWorldRotation(self.startrot*rotquat);
      
    end
  end
  
end

TrackHeadRot:MemberRegister("recognition",
venuscore.ScriptTypes.ReferenceType(
cv.RecognitionComponent:RTTI()
)); 

return TrackHeadRot