local Venuscore = require "venuscore"
local Apolloengine = require "apolloengine"
local Defined = require "apolloutility.defiend"
local HeadBlendBehavior = Venuscore.VenusBehavior:extend("HeadBlendBehavior"); 
local Venusjson = require "venusjson"
local Mathfunction = require "mathfunction"
local Libmasque = require "libmasque"

function HeadBlendBehavior:new() 
    self.HeadConfig = ""
    self.Tracking = nil;
    self.Track_Ani_Rate = 0.0;
end

function  HeadBlendBehavior:_OnAwake()
  self:LoadConfig()
  self:Init()
end

function HeadBlendBehavior:Init()
  if not self.Node or not self.bonesNames then
    return
  end
  self.BoneTrans = {};
  self.BoneInitialRot = {};
  for i = 1, #self.bonesNames do
    local node = self.Node:GetRoot():GetNodeByName(self.bonesNames[i]);
    if node then
      local trans = node:GetComponent(Apolloengine.Node.CT_TRANSFORM);
      table.insert(self.BoneTrans, trans);
      table.insert(self.BoneInitialRot, trans:GetLocalRotation());

    end
  end
    

end

function HeadBlendBehavior:LoadConfig()
  if self.HeadConfig == "" then
    return
  end
  local file = Venusjson.LaodJsonFile(self.HeadConfig);
  if file ~=nil then
    self.bonesNames = file.nodes;
    self.bonesWeight = file.weight;
  end
end

function HeadBlendBehavior:_OnUpdate(delta)
  if self.bonesNames == nil or #self.bonesNames == 0 then 
    return;
  end
  if self.Tracking then
    -- local rotmat = self.Tracking:GetBilinearTracking():GetRotation();
    -- local headRot = rotmat:ToQuaternion();
    local headRot = self.Tracking:GetTrackingRotation();
    local num = #self.bonesNames;
    local deltaRot = headRot;
    local rot = self:_CalTrackingRot(deltaRot);
    local slerpRot = self:_SlerpWithBone(rot);
    for i = 1, num do
      self.BoneTrans[i]:SetLocalRotation(slerpRot[i]);
    end
  end

end

function HeadBlendBehavior:_CalTrackingRot(deltaRot)
  local rotNew = {};
  local initialRot = {};
  for i = 1, #self.bonesNames do
    local rot = Mathfunction.Mathutility:Slerp(Mathfunction.Quaternion(), deltaRot, self.bonesWeight[i]);
    if self.BoneInitialRot then
      local rotInit = self.BoneInitialRot[i];
      rot = rotInit * rot;
    end
    rotNew[i] = rot;
  end
  return rotNew
end

function HeadBlendBehavior:_SlerpWithBone(rot)
  local slerpRot = {};
  for i = 1, #self.bonesNames do
    slerpRot[i] = Mathfunction.Mathutility:Slerp(rot[i], self.BoneTrans[i]:GetLocalRotation(), self.Track_Ani_Rate);
  end
  return slerpRot;
end

HeadBlendBehavior:MemberRegister("HeadConfig",  
  Venuscore.ScriptTypes.FilePathType(
    {"json"},
    nil,
    function(this, value)
        this.HeadConfig = value
        this:LoadConfig()
        this:Init()
    end
))

HeadBlendBehavior:MemberRegister(
  "Tracking",
  Venuscore.ScriptTypes.ReferenceType(
    Libmasque.TrackingComponent:RTTI()
  )
)


HeadBlendBehavior:MemberRegister("Track_Ani_Rate");
return HeadBlendBehavior;