local Object = require "classic"
local FrameTimer = require "behavior.tracking_behavior.frame_timer"

local ValueTransit = Object:extend()

TransitStatus = {
  START = "start",
  END = "end",
  TRANSIT = "transit"
}

function ValueTransit:new(time)
  self.timer = FrameTimer(time)
  self.target_values = {}
  self.current_status = TransitStatus.START
  self.current_value_list = {}
end

function ValueTransit:GetValue()
  return self.current_value_list
end

function ValueTransit:UpdateValue(timespan, value_status, start_value_list, end_value_list, defalut_value)
  if self:Finished() then
    if value_status == TransitStatus.TRANSIT then
      if self.current_status == TransitStatus.START then
        self:Start(start_value_list, end_value_list, defalut_value)
      end
    else
      self:SetValue(start_value_list)
    end
  else
    self:Update(timespan)
  end
  self.current_status = value_status
end

function ValueTransit:SetValue(value_list)
  self.current_value_list = {}
  for key, value in pairs(value_list) do
    self.current_value_list[key] = value
  end
end

function ValueTransit:Start(start_value_list, end_value_list, defalut_value)
  end_value_list = end_value_list or {}
  defalut_value = defalut_value or 0
  self.current_value_list = {}
  for key, value in pairs(start_value_list) do
    self.current_value_list[key] = value
    local start_value = value
    local end_value = end_value_list[key] or defalut_value
    self.target_values[key] = {start_value, end_value}
  end
  self.timer:Start()
end

function ValueTransit:Finished()
  return self.timer:Finished()
end

function ValueTransit:Update(timespan)
  self.timer:Update(timespan)
  local progress = self.timer:Progress()
  for key, value in pairs(self.target_values) do
    self.current_value_list[key] = value[1] * (1 - progress) + value[2] * progress
  end
end

return ValueTransit