local cjson = require "cjson"
local properties = require "cam.CameraProperties"

local CameraDeviceManager = {}
local deviceJson = ""

function CameraDeviceManager.Init(text)
    deviceJson = text;
    CameraDeviceManager.data = CameraDeviceManager.LoadJsonString(text)
end

function CameraDeviceManager.LoadJsonString(text)
    local result = {}
    if text then
        result = cjson.decode(text)
    end
    return result
end

function CameraDeviceManager.GetCameraAPI()
    local result = {}
    local apiobj = CameraDeviceManager.data;
    if apiobj then
        result = apiobj["api"]
    end
    return result
end

function CameraDeviceManager.GetAllDevices(api)
    local result = {}
    local apiobj = CameraDeviceManager.data;
    if apiobj then
        result = apiobj["apiMap"][api]
    end
    return result
end

function CameraDeviceManager.GetCameraDevice(api,deviceId)
    local device = {};
    local apiobj = CameraDeviceManager.data;
    if apiobj then
        local devices = apiobj["apiMap"][api]
        for i,v in pairs(devices) do 
            if deviceId == v["id"] then
                device = v
                ERROR("GetDevice:"..device);
                break
            end
        end 
    end
    return device;
end

function CameraDeviceManager.GetCameraDevice2(deviceId)
    local device = {};
    local apiobj = CameraDeviceManager.data;
    if apiobj then
        local api = apiobj["api"]
        local devices = apiobj["apiMap"][api]
        for i,v in pairs(devices) do 
            if deviceId == v["id"] then
                device = v
                ERROR("GetDevice:"..device);
                break
            end
        end 
    end
    return device;
end

function CameraDeviceManager.DevGetCameraId(camera)
    local result = ""
    if camera then
        result = camera["id"]
    end
    return result
end

function CameraDeviceManager.DevGetCameraPosition(camera)
    local result = properties.CameraPosition.CP_FRONT
    if camera then
        result = camera["pos"]
    end
    return result
end

function CameraDeviceManager.DevGetCameraType(camera)
    local result = properties.CameraDeviceType.CDT_UNKNOWN
    if camera then
        result = camera["t"]
    end
    return result
end

function CameraDeviceManager.DevGetCameraCharacteristics(camera)
    local result = {}
    if camera then
        result = camera["chs"]
    end
    return result
end

function CameraDeviceManager.ChaIsZoomSupported(characteristics)
    local result = false
    if characteristics then
        result = characteristics["z"]
    end
    return result
end

function CameraDeviceManager.ChaIsSmoothZoomSupported(characteristics)
    local result = false
    if characteristics then
        result = characteristics["sz"]
    end
    return result
end

function CameraDeviceManager.ChaGetMaxZoom(characteristics)
    local result = 0
    if characteristics then
        result = characteristics["maxz"]
    end
    return result
end

function CameraDeviceManager.ChaGetDefaultZoom(characteristics)
    local result = 0
    if characteristics then
        result = characteristics["zval"]
    end
    return result
end

function CameraDeviceManager.ChaGetSupportedPreviewSizes(characteristics)
    local result = {}
    if characteristics then
        result = characteristics["preSizes"]
    end
    return result
end

function CameraDeviceManager.GetSizeByIndex(sizeList,index)
    local result = nil;
    if sizeList then
        result = sizeList[index]
    end
    return result;
end

function CameraDeviceManager.SizeGetWidth(size)
    local result = 0
    if size then
        result = size["w"]
    end
    return result
end

function CameraDeviceManager.SizeGetHeight(size)
    local result = 0
    if size then
        result = size["h"]
    end
    return result
end

return CameraDeviceManager