local apolloengine = require "apolloengine"
local apollocore = require "apollocore"
local camSystem = apollocore.CameraSystem
local deviceHelper = require "cam.CameraDeviceManager"

local CameraSystem = {}

function CameraSystem:CamOpen(deviceId)
    ERROR("[main:CamOpen]")
    local result = false;
    if camSystem then
        local status = camSystem:GetCameraStatus()
        ERROR("camera status = " .. tostring(status))
        camSystem:Open(deviceId);
        ERROR("main:CamOpen");
        result = true;
    end

    return result;
end

function CameraSystem:CamClose()
    local result = false;
    if camSystem then
        camSystem:Close();
        result = camSystem:PiplineStop(); -- 按照设计 每个'摄像头预览' 对应 一个pipeline
        if not result then
            ERROR("camSystem:PiplineStop ERROR");
        end
    end
    return result;
end

function CameraSystem:CamStartPreview()
    ERROR("[main:CamStartPreview]")
    local result = false;
    if camSystem then
        local status = camSystem:GetCameraStatus()
        ERROR("camera status = " .. tostring(status))
        camSystem:StartPreview();
        

        local ret = false;
        --PreocessPipeLine创建移动至camerasystem初始化时
        --ret = camSystem:CreatPreocessPipeLine();
        --if not ret then
        --    ERROR("camSystem:CreatPreocessPipeLine ERROR");
        --end

        local transfromProcess = apolloengine.ProcessEntityInfo();
        transfromProcess:SetType(apolloengine.ProcessEntityInfo.PET_TRANSFORM);

        local facePoint106Process = apolloengine.ProcessEntityInfo();
        facePoint106Process:SetType(apolloengine.ProcessEntityInfo.PET_FACE_POINT106);
        facePoint106Process:SetDependType({apolloengine.ProcessEntityInfo.PET_TRANSFORM});

        --测试用例1  加上一个锐化效果
        --local laplaceSharpProcess = apolloengine.ProcessEntityInfo();
        --laplaceSharpProcess:SetType(apolloengine.ProcessEntityInfo.PET_LAPLACE_SHARP);
        --laplaceSharpProcess:SetDependType({apolloengine.ProcessEntityInfo.PET_TRANSFORM, apolloengine.ProcessEntityInfo.PET_FACE_POINT106});

        --测试用例2  再之前锐化的基础上再加一个滤镜
        --local invertAlphaProcess = apolloengine.ProcessEntityInfo();
        --invertAlphaProcess:SetType(apolloengine.ProcessEntityInfo.PET_INVERT_ALPHA);
        --invertAlphaProcess:SetDependType({apolloengine.ProcessEntityInfo.PET_LAPLACE_SHARP});

        ret = camSystem:AddPiplineProcessEntity(transfromProcess);
        ret = camSystem:AddPiplineProcessEntity(facePoint106Process);
        --ret = camSystem:AddPiplineProcessEntity(laplaceSharpProcess);
        --ret = camSystem:AddPiplineProcessEntity(invertAlphaProcess);
        if not ret then
            ERROR("camSystem:AddPiplineProcessEntity ERROR");
        end

        ret = camSystem:PiplineStart();
        if not ret then
            ERROR("camSystem:Start ERROR");
        end

        -- 需要在pre-pipeline创建之后再调用
        -- 按照设计 应该pre-component对应一个 pre-pipeline 负责存demo设置的参数 目前不释放 共用
        -- 并在 渲染逻辑更新开头 设置到pre-pipeline (RefreshBufferToRender)
        if not self.camSystemScene then
            self.camSystemScene = apolloengine.SceneManager:GetOrCreateScene("CameraSystem");
            local preprocessNode = self.camSystemScene:CreateNode(apolloengine.Node.CT_NODE)
            self.preprocessComp = preprocessNode:CreateComponent(apolloengine.Node.CT_PREPROCESS)
        end

        self.preprocessComp:SetResultCallBack("");

        ERROR("main:StartPreview");
        result = true;
    end
    return result;
end

function CameraSystem:GetCameraStatus()
    local result = -1;
    if camSystem then
        result = camSystem:GetCameraStatus();
    end
    return result;
end

function CameraSystem:ExecuteParameter(parameter)
    local result = false;
    if camSystem then
        result = camSystem:ApplyParameter(parameter);
    end
    return result;
end

function CameraSystem:CamGetDevice()
    local device = {};
    if camSystem then
        device = camSystem:GetCurrentDevice();
        ERROR("GetCameraDevice device:"..device);
        local character = deviceHelper.DevGetCameraCharacteristics(deviceHelper.LoadJsonString(device));
        ERROR("GetCameraDevice defaultZoom:"..tostring(deviceHelper.ChaGetDefaultZoom(character)));
        ERROR("GetCameraDevice maxZoom:"..tostring(deviceHelper.ChaGetMaxZoom(character)));
    end
    ERROR("main:CamGetDevice4")
    return device
end

function CameraSystem:QueryParameter(parameterType)
    ERROR("CamQueryParameters parameterType:"..parameterType);
    local result = "";
    if camSystem then
        result = camSystem:QueryParameter(parameterType);
        ERROR("CamQueryParameters parameter:"..result);
    end
    return result;
end

function CameraSystem:CamGetAPI()
    local api = 0;
    if camSystem then
        api = camSystem:GetCameraApi();
    end
    return api
end

function CameraSystem:CamSetListener(listener)
    local res = false;
    ERROR("CamSetListener 0:"..tostring(listener))
    if camSystem then
        ERROR("CamSetListener 1")
        res = camSystem:SetCameraListener(listener);
        ERROR("CamSetListener 2")
    end
    return res;
end

function CameraSystem:Render()
    if camSystem then
        local res = camSystem:RefreshBufferToRender();
        if not res then
            ERROR("camSystem:RefreshBufferToRender")
        end
    end
    return true;
end

function CameraSystem:SetOutputSize(width, height)
    if self.preprocessComp then
        WARNING(string.format("[SetOutputSize] w = %f h = %f",  width, height));
        self.preprocessComp:SetOutputSize(width, height)
    end
end

function CameraSystem:SetFaceSmoothParams(strength, modelLevel)
    if self.preprocessComp then
        WARNING(string.format("[SetFaceSmoothParams] strength %f modelLevel %f ", strength, modelLevel));
        self.preprocessComp:SetFaceSmoothParams(strength, modelLevel)
    end
end

function CameraSystem:SetTheaParams(enableVnr, vnrLightValue, enableVls, vlsScale, vlsFullRange)
    if self.preprocessComp then
        WARNING(string.format("[SetTheaParams] enableVnr %s ", tostring(enableVnr)));
        self.preprocessComp:SetTheaParams(enableVnr, vnrLightValue, enableVls, vlsScale, vlsFullRange)
    end
end

return CameraSystem