local BundleSystem = require "venuscore.bundle.bundlesystem"
local renderqueue = require "apolloutility.renderqueue"
local defined = require "apolloutility.defiend"
local apolloengine = require "apollocore"
local venuscore = require "venuscore"
local venusjson = require "venusjson"
local editorscene = require "editorscene"
local apolloDefine = require "apolloutility.defiend"

local facemeshmakeupmanager = {}

function facemeshmakeupmanager:Initialize(layer, resourceType)
  self.layer = layer;
  self.resourceType = resourceType;
  self.scene = nil;
  self.renderquequ_active = false;
end

function facemeshmakeupmanager:LoadConfig(path)
  if not self.renderquequ_active then
    self:_ActivateRenderQueue();
  end
  if self.scene ~= nil then
    self:_ReleaseScene();
  end
  return self:_LoadScene(path)
end

function facemeshmakeupmanager:ReleaseResource()
  if self.scene ~= nil then
    self:_ReleaseScene();
  end
  if self.renderquequ_active then
    self:_DeactivateRenderQueue();
  end
  
  return true;
end

function facemeshmakeupmanager:Update(def)
  if self.scene ~= nil then
    self.scene:Update(def);
  end
end

function facemeshmakeupmanager:_ActivateRenderQueue()
  renderqueue:Activate(self.layer);
  local inputTex = renderqueue:GetTexture(self.layer);
  apolloengine.DeviceResource:PushDeviceResource(
    defined.LAST_QUEUE_TEXTURE,
    inputTex);
  self.renderquequ_active = true;
end

function facemeshmakeupmanager:_DeactivateRenderQueue()
  renderqueue:Deactivate(self.layer)
  self.renderquequ_active = false;
end

function facemeshmakeupmanager:_LoadScene(path)
  local pre_resource_path = venuscore.IFileSystem:GetResourcePath();
  local path_dir= string.match(path, "(.+)/[^/]*%.%w+$");
  local current_resource_path = path_dir.."/";  
  local config = venusjson.LaodJsonFile(path);

  venuscore.IFileSystem:SetResourcePath(current_resource_path);
  local _, base_objects = self:_CreateInstanceFromBundle(config, BundleSystem.DeserializeMode.Scene)
  venuscore.IFileSystem:SetResourcePath(pre_resource_path);

  if base_objects == nil then
    return false
  end
  self.scene = editorscene(self.layer, self.resourceType);
  return self.scene:LoadConfig(base_objects);
end

function facemeshmakeupmanager:_CreateInstanceFromBundle(config, mode)
  local bundlepath = venuscore.IFileSystem:PathAssembly(config.scene);
  local scene = apolloengine.SceneManager:GetOrCreateScene(apolloDefine.default_scene_name);
  local native_scene, deserialize_res = BundleSystem:DeserializeFromPath(bundlepath, mode, scene);
  return native_scene, deserialize_res;
end

function facemeshmakeupmanager:_ReleaseScene()
  self.scene:ReleaseResource();
  self.scene = nil;
end

return facemeshmakeupmanager;
