local apolloDefine = require "apolloutility.defiend"
apolloDefine.default_scene_name = "camera"

local deviceHelper = require "cam.CameraDeviceManager"
local venuscore = require "venuscore"
local apolloengine = require "apolloengine"
local apollocore = require "apollocore"
local apollonode = require "apolloutility.apollonode"
local mathfunction = require "mathfunction"

local TAG = "main_camera"

local STATUS_PREVIEW = 5

--暂时关闭JIT防止JIT失败导致的效率下降
if _PLATFORM_ANDROID or _PROFILER then
    jit.off();
    jit.flush();
    LOG("turn off jit")
end

local main = {}


if _KRATOSEDITOR then
    main._Agent = main
else
    g_callbackhandle = main
    gMain = main

    --初始化回调
    main.servicecallbacks = {}
    main.callbackindex = 1
    setmetatable(main.servicecallbacks, {__mode = "v"}) --弱引用
    --以下为固定接口
    function main:RegisterCallback(func)
        local index = self.callbackindex
        self.callbackindex = self.callbackindex + 1
        self.servicecallbacks[index] = func
        return self._Agent, index
    end

    function main:CallbackFunction(index, ...)
        --_COROUTINES_ON();
        local func = self.servicecallbacks[index]
        local res = true
        if func then
            res = func(...)
        end
        --_COROUTINES_OFF();
        return res
    end
end

function main:Initialize()
    LOG(string.format("[%s::initialize]", TAG))

    _COROUTINES_ON()

    --创建一个场景
    self.cameraScene = apolloengine.SceneManager:GetOrCreateScene(apolloDefine.default_scene_name)

    self.mainCamera = self:_CreateCameras(
            0.01,
            20,
            mathfunction.vector3(0, 0, 0),
            mathfunction.vector3(0, 0, -1),
            mathfunction.vector3(0, 1, 0)
        )

    apolloengine.Framework:AddSynchronizeUpdateCallback(self._Agent, "SyncUpdate") --添加同步更新

    self.begintime = venuscore.ITimerSystem:GetTimevalue()
    self.cumtime = 0
    self.fps = 0

    self.frameIndex = 0
    self.lastpts = 0
    self.curdef = 0

    LOG(string.format("[%s::initialize] done", TAG))

    _COROUTINES_OFF()

    return true
end

function main:Exit()
    apolloengine.Framework:RemoveSynchronizeUpdateCallback(self._Agent, "SyncUpdate")
    apolloengine.SceneManager:DeleteScene(apolloDefine.default_scene_name)
    return true
end

function main:AttachRenderTarget(fbo, texture)
    LOG(string.format("[%s::AttachRenderTarget]platform android, fbo: %d", TAG, fbo))
    self.rendertarget = apolloengine.RenderTargetEntity() --创建一个FBO
    self.rendertarget:PushMetadata(
            apolloengine.RenderTargetResourceMetadata(
                    apolloengine.RenderTargetEntity.RT_RENDER_TARGET_2D,
                    apolloengine.Framework:GetResolution(),
                    fbo
            )
    )
    self.outputtexture = self.rendertarget:MakeTextureAttachment(apolloengine.RenderTargetEntity.TA_COLOR_0) --增加color0纹理
    self.outputtexture:PushMetadata(
        apolloengine.TextureResourceMetadata(apolloengine.Framework:GetResolution(), texture)   --创建纹理
    )
    self.rendertarget:CreateResource()
    self.mainCamera:AttachRenderTarget(self.rendertarget);
    return true
end

function main:SyncUpdate()
    --_COROUTINES_ON();
    self:_Running()
    --_COROUTINES_OFF();
end

function main:Update()
    self:_Running()
end

function main:_Running()
    _COROUTINES_ON()

    --local delts = self:Timespan()

    if _PLATFORM_WINDOWS then --windows测试代码
        self.curdef = delts;
        self.frameIndex = (self.frameIndex + 24 * self.curdef) % 100;
    end

    --if self.node == nil then
    --    self:_CreateBackgroundNode("comm:documents/texture/posteffect/lut.png")
    --end

    if self.ts == nil then
        self.ts = apolloengine.TextureStream();
    end
    local status = apollocore.CameraSystem:GetCameraStatus()
    if status == STATUS_PREVIEW then
        local res = apollocore.CameraSystem:UpdateTextureStream(self.ts)
        LOG("UpdateTextureStream res = " .. tostring(res))
        if res == true then
            if self.node == nil then
                self:_CreateBackgroundNode(nil, self.ts)
            else
                self.mCamTexEntity:ChangeTextureBuffer(self.ts)
            end
        else
            LOG("can not get texturestream")
        end
    end

    --self.cameraScene:Update(delts);

    _PROFILER_UPDATE()
    _COROUTINES_OFF()
end

function main:_CreateBackgroundNode(pngPath, ts)
    self.node = apollonode.QuadNode()
    self.node:CreateResource("comm:documents/material/imageblit.material");
    self.node:SetSequence(self.mainCamera:GetSequence());

    local tex = apolloengine.TextureEntity();

    if pngPath ~= nil then
        tex:PushMetadata(apollocore.TextureFileMetadata(
                apollocore.TextureEntity.TU_WRITE,
                apollocore.TextureEntity.PF_AUTO,1, false,
                apollocore.TextureEntity.TW_CLAMP_TO_EDGE,
                apollocore.TextureEntity.TW_CLAMP_TO_EDGE,
                apollocore.TextureEntity.TF_LINEAR,
                apollocore.TextureEntity.TF_LINEAR,
                pngPath)
        )
    else
        if ts ~= nil then
            tex:PushMetadata(apolloengine.TextureBufferMetadata(
                    apolloengine.TextureEntity.TU_WRITE,
                    1,
                    false,
                    apolloengine.TextureEntity.TW_CLAMP_TO_BORDER,
                    apolloengine.TextureEntity.TW_CLAMP_TO_BORDER,
                    apolloengine.TextureEntity.TF_LINEAR,
                    apolloengine.TextureEntity.TF_LINEAR,
                    ts)
            )
        end
    end
    --self.rgb_ts = apolloengine.TextureStream();
    --self.rgb_ts:SetStreamType(mathfunction.vector2(16,16), apolloengine.TextureEntity.PF_R8G8B8);

    tex:CreateResource();

    self.mCamTexEntity = tex;
    self.node:SetParameter(
            apolloengine.ShaderEntity.TEXTURE_DIFFUSE,
            self.mCamTexEntity);
end

function main:_CreateCameras(near, far, pos, lookat, up)
    local maincamera = apollonode.CameraNode() --新建摄像机
    maincamera:SetName("MainCamera")
    maincamera:CreateRealCameraProjection(near, far) --设置摄像机
    maincamera:LookAt(pos, lookat, up)
    maincamera:Recalculate() --手动更新矩阵
    maincamera:Activate();--激活主摄像机
    maincamera:SetSequenceCulling(false);
    maincamera:SetSequence(-100);
    return maincamera
end

function main:CamOpen()
    ERROR("[main:CamOpen]")
    local result = false;
    if apollocore.CameraSystem then
        local status = apollocore.CameraSystem:GetCameraStatus()
        ERROR("camera status = " .. tostring(status))
        apollocore.CameraSystem:Open("0");
        ERROR("main:CamOpen");
        result = true;
    end
    return result;
end

function main:CamClose()
    local result = false;
    if apollocore.CameraSystem then
        apollocore.CameraSystem:Close();
        result = true;
    end
    return result;
end

function main:CamStartPreview()
    ERROR("[main:CamStartPreview]")
    local result = false;
    if apollocore.CameraSystem then
        local status = apollocore.CameraSystem:GetCameraStatus()
        ERROR("camera status = " .. tostring(status))
        apollocore.CameraSystem:StartPreview();
        ERROR("main:StartPreview");
        result = true;
    end
    return result;
end

function main:GetCameraStatus()
    local result = -1;
    if apollocore.CameraSystem then
        result = apollocore.CameraSystem:GetCameraStatus();
    end
    return result;
end

function main:ExecuteParameter(parameter)
    local result = false;
    if apollocore.CameraSystem then
        result = apollocore.CameraSystem:ApplyParameter(parameter);
    end
    return result;
end

function main:CamGetDevice()
    local device = {};
    if apollocore.CameraSystem then
        device = apollocore.CameraSystem:GetCurrentDevice();
        ERROR("GetCameraDevice device:"..device);
        local character = deviceHelper.DevGetCameraCharacteristics(deviceHelper.LoadJsonString(device));
        ERROR("GetCameraDevice defaultZoom:"..tostring(deviceHelper.ChaGetDefaultZoom(character)));
        ERROR("GetCameraDevice maxZoom:"..tostring(deviceHelper.ChaGetMaxZoom(character)));
    end
    ERROR("main:CamGetDevice4")
    return device
end

function main:QueryParameter(parameterType)
    ERROR("CamQueryParameters parameterType:"..parameterType);
    local result = "";
    if apollocore.CameraSystem then
        result = apollocore.CameraSystem:QueryParameter(parameterType);
        ERROR("CamQueryParameters parameter:"..result);
    end
    return result;
end

function main:CamGetAPI()
    local api = 0;
    if apollocore.CameraSystem then
        api = apollocore.CameraSystem:GetCameraApi();
    end
    return api
end

function main:CamSetListener(listener)
    local res = false;
    ERROR("CamSetListener:"..tostring(listener))
    if apollocore.CameraSystem then
        res = apollocore.CameraSystem:SetCameraListener(listener);
    end
    return res;
end
return main
