local apolloDefine = require "apolloutility.defiend"
local apollocore = require "apollocore"
apolloDefine.default_scene_name = "test"
local SceneLoader = require "editorscene.sceneloader"
local venuscore = require "venuscore"
local apolloengine = require "apolloengine"
local apollonode = require "apolloutility.apollonode"
local mathfunction = require "mathfunction"
local likeapp = require "likeapp"
local cameraSystem = require "cam.CameraSystem"
local renderqueue = require "apolloutility.renderqueue"
local TAG = "test"

local LAYER_FIRST = "Input"
local LAYER_LAST = "Output"

--暂时关闭JIT防止JIT失败导致的效率下降
if _PLATFORM_ANDROID or _PROFILER then
    jit.off();
    jit.flush();
    LOG("turn off jit")
end

local main = {}

_NeedPause = function()
    main:_NeedPause();
end

if _KRATOSEDITOR then
    main._Agent = main
else
    g_callbackhandle = main
    gMain = main

    --初始化回调
    main.servicecallbacks = {}
    main.callbackindex = 1
    setmetatable(main.servicecallbacks, {__mode = "v"}) --弱引用
    --以下为固定接口
    function main:RegisterCallback(func)
        local index = self.callbackindex
        self.callbackindex = self.callbackindex + 1
        self.servicecallbacks[index] = func
        return self._Agent, index
    end

    function main:CallbackFunction(index, ...)
        --_COROUTINES_ON();
        local func = self.servicecallbacks[index]
        local res = true
        if func then
            res = func(...)
        end
        --_COROUTINES_OFF();
        return res
    end
end

function main:Initialize()
    LOG(string.format("[%s::initialize]", TAG))
    _COROUTINES_ON()
    self.begintime = venuscore.ITimerSystem:GetTimevalue();
    self.cumtime = 0;
    self.fps = 0;
    
    --创建一个场景
    self.cameraScene = apolloengine.SceneManager:GetOrCreateScene(apolloDefine.default_scene_name);
    self.mainCamera = self:CreateCR(0.01, 20, mathfunction.vector3(0,0,0), mathfunction.vector3(0,0,-1), mathfunction.vector3(0,1,0));

    SceneLoader:Initialize();

    renderqueue:Initialize(self.mainCamera);
    renderqueue:Activate(LAYER_FIRST);
    renderqueue:Activate(LAYER_LAST);

    --self.mainCamera:SetRenderWindow(apolloengine.IGraphicSystem:GetMainWindow());

    apolloengine.Framework:AddSynchronizeUpdateCallback(self._Agent, "SyncUpdate"); --添加同步更新
    
    self.mCamTexEntity = apolloengine.TextureEntity();
    self.mCamTexEntity:PushMetadata(apolloengine.TextureReferenceMetadata("DEVICE_CAPTURE"));
    self.mCamTexEntity:CreateResource();
    
    self:linkEmptyTexture()
    
    self:_CreateBackgroundNode();
    LOG(string.format("[%s::initialize] done", TAG))
    _COROUTINES_OFF()

    return true
end

function main:CreateCR(near, far, pos, lookat, up)
    local maincamera = apollonode.CameraNode();--新建摄像机
    --maincamera:CreatePerspectiveProjection(near, far);
    maincamera:SetName("MainCamera")
    maincamera:CreateRealCameraProjection(near, far);--设置摄像机
    maincamera:LookAt(pos, lookat, up);
    maincamera:Recalculate();--手动更新矩阵
    maincamera:SetClearColor(mathfunction.Color(0.0,0.0,1.0,1));
    maincamera:Activate();--激活主摄像机
    return maincamera;
end

function main:Timespan()
    local now = venuscore.ITimerSystem:GetTimevalue();
    local def = now - self.begintime;
    self.begintime = now;
  
    self.cumtime = self.cumtime + def;
    self.fps = self.fps + 1;
    if self.cumtime > 5 then
      local fps = self.fps / self.cumtime;
      self.fps = 0;
      self.cumtime = 0;
      LOG("logic fps "..fps);
    end
    --防止间隔过长导致效果突变太大效果不自然
    def = def < 0.1 and def or 0.03;
    return def;
end

function main:Exit()
    apolloengine.Framework:RemoveSynchronizeUpdateCallback(self._Agent, "SyncUpdate")
    apolloengine.SceneManager:DeleteScene(apolloDefine.default_scene_name)
    
    apolloengine.IGraphicSystem:RemoveRenderWindow(self.testwin)
    likeapp.VenusViewProxy:DestorySubWindow("test_window")
    cameraSystem.CamClose();
    if self.stickerLoaded then
        SceneLoader:ReleaseSticker();
        self.stickerLoaded = false;
    end
    WARNING("*************************************************************main_test:Exit");
    return true
end

--function main:AttachRenderTarget(fbo, texture)
--    LOG(string.format("[%s::AttachRenderTarget]platform android, fbo: %d", TAG, fbo));
--    self.rendertarget = apolloengine.RenderTargetEntity(); --创建一个FBO
--    self.rendertarget:PushMetadata(
--    --设置FBO格式
--            apolloengine.RenderTargetResourceMetadata(
--                    apolloengine.RenderTargetEntity.RT_RENDER_TARGET_2D,
--                    apolloengine.Framework:GetResolution(),
--            --分辨率
--                    fbo
--            )
--    );
--    self.outputtexture = self.rendertarget:MakeTextureAttachment(apolloengine.RenderTargetEntity.TA_COLOR_0); --增加color0纹理
--    self.outputtexture:PushMetadata(
--        apolloengine.TextureResourceMetadata(apolloengine.Framework:GetResolution(), texture)   --创建纹理
--    );
--    self.rendertarget:CreateResource();
--
--    renderqueue:SetOutputRenderTarget(self.rendertarget, self.outputtexture)
--
--    return true
--end

function main:SyncUpdate()
    --_COROUTINES_ON();
    local def = self:Timespan();
    self.def = def;
    self:_Running()
    --_COROUTINES_OFF();
end

function main:Update()
    self:_Running()
end


function main:_Running()
    local def = self.def;
    if self.stickerLoaded then
        SceneLoader:UpdateSticker(def);
    end
    self.cameraScene:Update(0.1);

    --self:_SetupSubViewAndScene();
end

function main:linkEmptyTexture()
    self.redirectTex = apolloengine.TextureEntity();
    apolloengine.DeviceResource:PushDeviceResource(apolloengine.DeviceResource.DEVICE_CAPTURE,self.redirectTex);
end

function main:_CreateBackgroundNode()
    self.node = apollonode.QuadNode() -- 挂在facecute scene上
    self.node:CreateResource("comm:documents/material/imageblit.material");
    self.node:SetSequence(self.mainCamera:GetSequence());
    self.node:SetParameter(
            apolloengine.ShaderEntity.TEXTURE_DIFFUSE,
            self.mCamTexEntity);
    -- self.node:SetLayer("Input");
    renderqueue:Queue(LAYER_FIRST, self.node); 
end

--function main:_CreateCameras(near, far, pos, lookat, up)
--    local maincamera = apollonode.CameraNode() --新建摄像机
--    maincamera:SetName("MainCamera")
--    maincamera:CreateRealCameraProjection(near, far) --设置摄像机
--    maincamera:LookAt(pos, lookat, up)
--    maincamera:Recalculate() --手动更新矩阵
--    maincamera:Activate();--激活主摄像机
--    maincamera:SetSequenceCulling(false);
--    maincamera:SetSequence(-100);
--    maincamera:SetClearColor( mathfunction.Color(0.0,1.0,0.0,1))
--    return maincamera
--end

--function main:_CreateCamera(layer,clearcolor)
--    local camera = self.cameraScene:CreateNode(apolloengine.Node.CT_NODE)
--    local cameraComponent = camera:CreateComponent(apolloengine.Node.CT_CAMERA)
--
--    local near = 0.1
--    local far = 20
--    local pos = mathfunction.vector3(0,0,1)
--    local resolution = mathfunction.vector2(320,420)
--    local lookat = mathfunction.vector3(0,0,0)
--    local up = mathfunction.vector3(0,1,0)
--
--    cameraComponent:AddLayerMask(layer)
--    --cameraComponent:FixedResolution(resolution)
--    cameraComponent:CreatePerspectiveProjection(near,far)
--    cameraComponent:LookAt(pos,lookat,up)
--    cameraComponent:Recalculate()
--    cameraComponent:Activate()
--    cameraComponent:SetClearColor(clearcolor)
--
--    return cameraComponent
--end


--function main:_CreateQuad(layer,renderOrder)
--    local quad = self.cameraScene:CreateNode(apolloengine.Node.CT_NODE)
--    local trans = quad:CreateComponent(apolloengine.Node.CT_TRANSFORM)
--    local render = quad:CreateComponent(apolloengine.Node.CT_RENDER)
--    quad:SetLayer(layer)
--
--    render:PushMetadata(
--        --这里的quad shader有修改过
--        apolloengine.RenderObjectMaterialMetadata(apolloengine.PathMetadata("comm:documents/shaders/opaque/imagecopy.material"))
--    )
--
--    render:PushMetadata(
--        apolloengine.RenderObjectMeshFileMetadate("comm:documents/basicobjects/quad/data/plane001.mesh")
--    )
--    render:CreateResource();
--
--    render:SetRenderOrder(renderOrder)
--
--    return quad
--end

function main:CamOpen(deviceId)
    return cameraSystem:CamOpen(deviceId);
end

function main:CamClose()
    return cameraSystem:CamClose();
end

function main:CamStartPreview()
    return cameraSystem:CamStartPreview();
end

function main:GetCameraStatus()
    return cameraSystem:GetCameraStatus();
end

function main:ExecuteParameter(parameter)
    return cameraSystem:ExecuteParameter(parameter);
end

function main:CamGetDevice()
    return cameraSystem:CamGetDevice();
end

function main:QueryParameter(parameterType)
    return cameraSystem:QueryParameter(parameterType);
end

function main:CamGetAPI()
    return cameraSystem:CamGetAPI();
end

function main:CamSetListener(listener)
    return cameraSystem:CamSetListener(listener);
end

-- 按照设计原意, 后面要根据 "数据源" 来区分 把前处理参数 设置到 哪个ProProcessPipeline 目前都设置到 cameraSystem
function main:SetOutputSize(width, height)
    cameraSystem:SetOutputSize(width, height);
end

function main:SetFaceSmoothParams(strength, modelLevel)
    cameraSystem:SetFaceSmoothParams(strength, modelLevel);
end

function main:SetTheaParams(enableVnr, vnrLightValue, enableVls, vlsScale, vlsFullRange)
    cameraSystem:SetTheaParams(enableVnr, vnrLightValue, enableVls, vlsScale, vlsFullRange);
end

function main:FacecuteLoadResource(path)
    local res = false;
    if venuscore.IFileSystem:isFileExist(path) then
        self.stickerLoaded = SceneLoader:LoadAsSticker(path)
        --防止时间间隔过长导致第一次update曲线取值跳跃太大
        self.begintime = venuscore.ITimerSystem:GetTimevalue();
        --self.facecuteScene:End("Sticker");
        collectgarbage();
        res = self.stickerLoaded;
    end
    return res;
end

function main:FacecuteUnloadResource()
    ERROR("main_test FacecuteUnLoadSticker");
    SceneLoader:ReleaseSticker();
    self.stickerLoaded = false;
    return true;
end

--------------------------------------------------------------------------------------------------------------
--- 创建子View和简单场景
local addViewCounter = 0 ;
local subView   = nil
local subCam    = nil
local subScene  = nil
local oneNodeTrans = nil

function main:_SetupSubViewAndScene()

    addViewCounter = addViewCounter + 1 ;

    if addViewCounter == 400 then

        WARNING("CreateSubWindow -- test_window")

        subView = likeapp.VenusViewProxy:CreateSubWindow("test_window")
        local scene = apolloengine.SceneManager:GetOrCreateScene("SubViewScene");
        local node = scene:GetRootNode();
        subCam = node:CreateComponent(apolloengine.Node.CT_CAMERA);
        subCam:SetSequenceCulling(true);
        subCam:CreateRealCameraProjection(0.01, 20);
        subCam:LookAt(mathfunction.vector3(0,0,1), mathfunction.vector3(0,0,0), mathfunction.vector3(0,1,0));
        subCam:Recalculate();
        subCam:SetClearColor(mathfunction.Color(1.0, 1.0, 0.0, 1.0));
        subCam:Activate();
        subCam:SetRenderWindow(subView); -- 窗口和摄像机关联
        apolloengine.IGraphicSystem:RegisterRenderWindow(subView); -- 需要格外调用这个?? 还是C++ ??

        local oneNode = scene:CreateNode(apolloengine.Node.CT_NODE);
        local oneNodeRender = oneNode:CreateComponent(apolloengine.Node.CT_RENDER);
        oneNodeTrans  = oneNode:CreateComponent(apolloengine.Node.CT_TRANSFORM)
        oneNodeRender:PushMetadata(
          apolloengine.RenderObjectMaterialMetadata(
          --  apolloengine.PathMetadata("comm:documents/shaders/opaque/unlit.material")
             apolloengine.PathMetadata("comm:documents/editor_shaders/3d/standardpbr.material")
          )
        );
        oneNodeRender:PushMetadata(
          apolloengine.RenderObjectMeshFileMetadate("comm:documents/basicobjects/box/data/box001.mesh")
        )
        oneNodeRender:CreateResource();


        local oneNodeTexture = apolloengine.TextureEntity();
        oneNodeTexture:SetJobType(venuscore.IJob.JT_SYNCHRONOUS);
        oneNodeTexture:PushMetadata(apolloengine.TextureFileMetadata(
          apolloengine.TextureEntity.TU_STATIC,
          apolloengine.TextureEntity.PF_AUTO,1, false,
          apolloengine.TextureEntity.TW_CLAMP_TO_EDGE,
          apolloengine.TextureEntity.TW_CLAMP_TO_EDGE,
          apolloengine.TextureEntity.TF_NEAREST,
          apolloengine.TextureEntity.TF_NEAREST,
          "comm:documents/texture/particle/likexin.png"
        ));
        oneNodeTexture:CreateResource();

        oneNodeRender:SetParameter("_MainTex", oneNodeTexture);

        local rotate = mathfunction.Mathutility:RotateAxis(
          mathfunction.vector3(1.414/2, 1.414/2, 0), 45.0 / 180.0 * 3.14 );
        oneNodeTrans:SetLocalRotation(rotate);
        oneNodeTrans:SetLocalScale(mathfunction.vector3(2.5, 2.5, 2.5));


        local lightNode = scene:CreateNode(apolloengine.Node.CT_NODE);
        lightNode:CreateComponent(apolloengine.Node.CT_TRANSFORM)
        local light = lightNode:CreateComponent(apolloengine.Node.CT_LIGHT);
        light:SetLightType(apolloengine.LightComponent.LT_DIRECTIONAL);
        light:SetLocalDirection(mathfunction.vector3(0, 0, -1)); -- 0.577
        light:SetColor(mathfunction.vector3(1,1,1));

        subScene = scene ;

    elseif subScene ~= nil then
        subScene:Update(0.03);

        local rotation = oneNodeTrans:GetLocalRotation();
        local rotate = mathfunction.Mathutility:RotateAxis(mathfunction.vector3(0.0, 1.0, 0.0), 2.0 / 180.0 * 3.14);
        rotation = rotate * rotation;
        oneNodeTrans:SetLocalRotation(rotation);

    end
end
--------------------------------------------------------------------------------------------------------------

return main
