local venuscore = require "venuscore"
local apollocore = require "apollocore"
local apolloengine = require "apolloengine"
local SceneLoader = require "editorscene.sceneloader"
local renderqueue = require "apolloutility.renderqueue"
local apolloDefine = require "apolloutility.defiend"
local ProfileScope = venuscore.ProfileScope

local LAYER_STICKER = "Sticker"
local LAYER_LAST = "Output"

local main = {}

_NeedPause = function()
    main:_NeedPause();
end

function main:Initialize() 
    _COROUTINES_ON();

    self.facecuteScene = apolloengine.SceneManager:GetOrCreateScene(apolloDefine.default_scene_name);

    SceneLoader:Initialize();

    self.resourceNodesId = {};
    if _PLATFORM_WINDOWS or _PLATFORM_MACOS then     
     self:FacecuteLoadResource("D:/Filters/VRTest/scene.json");
    end

    apolloengine.Framework:AddSynchronizeUpdateCallback(self._Agent, "SyncUpdate");--添加同步更新
    apolloengine.Framework:AddKeyboardCallback(self._Agent, "OnKeyboard");
    apolloengine.Framework:AddTouchCallback(self._Agent, "OnTouch");

    self.running = false;   

    self.begintime = venuscore.ITimerSystem:GetTimevalue();
    self.cumtime = 0;
    self.fps = 0; 

    -- self.leftEyeRenderTargetPool = {};
    -- self.rightEyeRenderTargetPool = {};

    LOG("initialize done");
    _COROUTINES_OFF();
    return true;
end

function main:SyncUpdate()
    --_COROUTINES_ON();
    local def = self:Timespan();
    self.def = def; 
    self:_Running();
    --_COROUTINES_OFF();
end

function main:_Running()
    _COROUTINES_ON();
    
    local def = self.def;

    if self.isloaded and not self.running then
      self.running = true;         
      SceneLoader:UpdateVRScene(def);
      self.running = false;
    end
    _COROUTINES_OFF();
end

function main:Timespan()
    local now = venuscore.ITimerSystem:GetTimevalue();
    local def = now - self.begintime;
    self.begintime = now;
  
    self.cumtime = self.cumtime + def;
    self.fps = self.fps + 1;
    if self.cumtime > 5 then
      local fps = self.fps / self.cumtime;
      self.fps = 0;
      self.cumtime = 0;
      LOG("logic fps "..fps);
    end
    --防止间隔过长导致效果突变太大效果不自然
    def = def < 0.1 and def or 0.03;
    return def;
end

function main:OnResizeView(x,y,w,h)
end

function main:OnTouch(touchinfo)
end

function main:OnKeyboard(key,status)
end

function main:AttachRenderTarget(fbo, texture)
    -- LOG("platform android, fbo:"..fbo);

    -- if self.leftEyeRenderTargetPool[fbo] == nil then
    --     self.leftEyeRenderTargetPool[fbo] = apolloengine.RenderTargetEntity();--创建一个FBO
    --     self.leftEyeRenderTargetPool[fbo]:PushMetadata(--设置FBO格式
    --             apolloengine.RenderTargetResourceMetadata(
    --                     apolloengine.RenderTargetEntity.RT_RENDER_TARGET_2D,
    --                     apolloengine.Framework:GetResolution(),--分辨率
    --                     fbo));
    --     self.outputtexture = self.leftEyeRenderTargetPool[fbo]:MakeTextureAttachment(
    --             apolloengine.RenderTargetEntity.TA_COLOR_0);--增加color0纹理
    --     self.outputtexture:PushMetadata(--创建纹理
    --             apolloengine.TextureResourceMetadata(
    --                     apolloengine.Framework:GetResolution(),
    --                     texture));
    --     self.leftEyeRenderTargetPool[fbo]:CreateResource();
    -- end
    -- renderqueue:SetLeftEyeOutRenderTarget(self.leftEyeRenderTargetPool[fbo], self.outputtexture)
    -- return true ; 
end

function main:AttachExtraRenderTarget(fbo, texture)
    -- LOG("platform android, fbo:"..fbo);
    
    -- if self.rightEyeRenderTargetPool[fbo] == nil then
    --     self.rightEyeRenderTargetPool[fbo] = apolloengine.RenderTargetEntity();--创建一个FBO
    --     self.rightEyeRenderTargetPool[fbo]:PushMetadata(--设置FBO格式
    --             apolloengine.RenderTargetResourceMetadata(
    --                     apolloengine.RenderTargetEntity.RT_RENDER_TARGET_2D,
    --                     apolloengine.Framework:GetResolution(),--分辨率
    --                     fbo));
    --     self.outputtexture_extra = self.rightEyeRenderTargetPool[fbo]:MakeTextureAttachment(
    --             apolloengine.RenderTargetEntity.TA_COLOR_0);--增加color0纹理
    --     self.outputtexture_extra:PushMetadata(--创建纹理
    --             apolloengine.TextureResourceMetadata(
    --                     apolloengine.Framework:GetResolution(),
    --                     texture));
    --     self.rightEyeRenderTargetPool[fbo]:CreateResource();
    -- end
    -- renderqueue:SetRightEyeOutRenderTarget(self.rightEyeRenderTargetPool[fbo], self.outputtexture_extra)
    -- return true; 
end

function main:FacecuteLoadResource(path)
    _COROUTINES_ON();
    
    if not venuscore.IFileSystem:isFileExist(path) then
      ERROR("FacecuteLoadResource "..tostring(path).." not exist");
      return false ;
    end
  
    
    local res = SceneLoader:LoadVRScene(path);
      
    self.isloaded = res;
    
    --防止时间间隔过长导致第一次update曲线取值跳跃太大
    self.begintime = venuscore.ITimerSystem:GetTimevalue();
    collectgarbage();
  
    _COROUTINES_OFF();
    return self.isloaded;
end

function main:FacecuteUnloadResource()
    _COROUTINES_ON();
    self.isloaded = false;
    self.recordStartCnt = 0;
    self.recordStopCnt = 0 ;
    
    SceneLoader:ReleaseVRScene();
    venuscore.IFileSystem:SetResourcePath("");
    collectgarbage();
    _COROUTINES_OFF();
    return true;
end

function main:Exit()
    --WARNING("*************************************************************main recoder");
    apolloengine.Framework:RemoveSynchronizeUpdateCallback(self._Agent, "SyncUpdate");
    apolloengine.Framework:RemoveTouchCallback(self._Agent,"OnTouch");
    apolloengine.Framework:RemoveKeyboardCallback(self._Agent,"OnKeyboard");
  

    local sceneID = self.facecuteScene:GetStaticID();
    apolloengine.SceneManager:DeleteSceneByID(sceneID);
    SceneLoader:ReleaseAllResource();

    -- self.leftEyeRenderTargetPool = {};
    -- self.rightEyeRenderTargetPool = {};
    return true;
end


main.SyncUpdate                     = ProfileScope(main.SyncUpdate,                              "Lua#main_vr:SyncUpdate")
main.Timespan                       = ProfileScope(main.Timespan,                                "Lua#main_vr:Timespan")
main.AttachRenderTarget             = ProfileScope(main.AttachRenderTarget,                      "Lua#main_vr:AttachRenderTarget")
main.AttachExtraRenderTarget        = ProfileScope(main.AttachExtraRenderTarget,                 "Lua#main_vr:AttachExtraRenderTarget")
main._Running                       = ProfileScope(main._Running,                                "Lua#main_vr:_Running")
SceneLoader.UpdateVRScene           = ProfileScope(SceneLoader.UpdateVRScene,                    "Lua#SceneLoader:UpdateVRScene")

return main;