local venuscore = require "venuscore"
local textureloader = require "apolloutility.asynctexture.textureloader"
local apollonode = require "apolloutility.apollonode"
local emptyimage = require "apolloutility.emptyimage"
local apolloengine = require "apolloengine"
local mathfunction = require "mathfunction"
local likeapp = require "likeapp"
local venusjson = require "venusjson"
require "utility"
local videodefined = require"videodecet.defined"

local SceneLoader = require "editorscene.sceneloader"
-- maybe used by another script
local makeup = require"beauty.facecute_makeup"
local UserCall = require "event.user_call_event"
local Keyboard = require "apolloengine.inputdelegate.keyboarddelegate"
local ED = require "bluecore.eventdispatcher"
local BD = require "bluecore.bluedefined"
local apolloDefine = require "apolloutility.defiend"
local apollocore = require "apollocore"
local BundleSystem = require "venuscore.bundle.bundlesystem"
local VtuberMsgHub = require "behavior.vtuber_behavior.vtuber_msg_hub"

math.randomseed(tostring(os.time()):reverse():sub(1, 6))

--暂时关闭JIT防止JIT失败导致的效率下降
if _PLATFORM_ANDROID or _PROFILER then
  jit.off();
  jit.flush();
  LOG("turn off jit")
end

if _PLATFORM_WINDOWS or _PLATFORM_MACOS then
  collectgarbage("setpause",200);
  collectgarbage("setstepmul",3000);
end

local LAYER_EDITOR_MAKE_UP   = "EditorMakeUp"
local LAYER_MAKE_UP = "MakeUp"
local LAYER_STYLISH = "Stylish"
local LAYER_STICKER = "Sticker"
local LAYER_LAST = "Output"

local RESOURCE_TYPE_FILTER = 1;
local RESOURCE_TYPE_MAKEUP = 2;
local RESOURCE_TYPE_STICKER = 3;
local RESOURCE_TYPE_STYLISH = 3;

local main = {}

_NeedPause = function()
  main:_NeedPause();
end

if _KRATOSEDITOR then
  main._Agent = main;
else
  g_callbackhandle = main;
  gMain = main

  --初始化回调
  main.servicecallbacks = {}
  main.callbackindex = 1;
  setmetatable(main.servicecallbacks, {__mode = "v"}) --弱引用
  --以下为固定接口
  function main:RegisterCallback(func)
    local index = self.callbackindex;
    self.callbackindex = self.callbackindex + 1;
    self.servicecallbacks[index] = func;
    return self._Agent, index;
  end

  function main:CallbackFunction(index, ...)
    local func = self.servicecallbacks[index];
    local res = true;
    if func then
      res = func(...);
    end
    return res;
  end

  function main:Exec(command)
    local f = load("return gMain:" .. command)
    if f == nil then
      return false
    end
    return f()
  end
end

function main:InitBlit()

  if self.blitMaterialEntity == nil then
    local path = venuscore.IFileSystem:PathAssembly("comm:documents/material/imageblit_offset.material");
    local pathMeta = apolloengine.PathMetadata(path);
    self.blitMaterialEntity = apolloengine.MaterialEntity();
    self.blitMaterialEntity:PushMetadata(pathMeta);
    self.blitMaterialEntity:SetJobType(venuscore.IJob.JT_SYNCHRONOUS);
    self.blitMaterialEntity:CreateResource();
  end

  if self.blitRT == nil then
    -- setup fbo
    local fbo_size      = mathfunction.vector2(1024, 1024);
    self.blitRT  = apolloengine.RenderTargetEntity();
    self.blitRT:PushMetadata(apolloengine.RenderTargetMetadata(apolloengine.RenderTargetEntity.RT_RENDER_TARGET_2D,
                                                                  apolloengine.RenderTargetEntity.ST_SWAP_UNIQUE,
                                                                  mathfunction.vector4(0, 0, 1024, 1024),
                                                                  fbo_size));
    self.blitImg = self.blitRT:MakeTextureAttachment(apolloengine.RenderTargetEntity.TA_COLOR_0);
    self.blitImg:PushMetadata(apolloengine.TextureBufferMetadata(fbo_size,
                                                                   apolloengine.TextureEntity.TT_TEXTURE2D,
                                                                   apolloengine.TextureEntity.TU_STATIC,
                                                                   apolloengine.TextureEntity.PF_R8G8B8A8,
                                                                   1, false,
                                                                   apolloengine.TextureEntity.TW_CLAMP_TO_EDGE,
                                                                   apolloengine.TextureEntity.TW_CLAMP_TO_EDGE,
                                                                   apolloengine.TextureEntity.TF_LINEAR,
                                                                   apolloengine.TextureEntity.TF_LINEAR));
    self.blitRT:SetJobType(venuscore.IJob.JT_SYNCHRONOUS);
    self.blitRT:CreateResource();

  end
end

function main:Initialize()
  WARNING("main_vtuber Initialize");
  
  if _PLATFORM_WINDOWS or _PLATFORM_MACOS then
    local debuger = require "mobdebug/mobdebug";
    debuger.start();
    _COROUTINES_ON = debuger.on;
    _COROUTINES_OFF = debuger.off;
  end
  
  _COROUTINES_ON();
  self.isloaded = false;

  --初始化服务回掉注册体系
  textureloader:Initialize();
  emptyimage:Initialize();
  SceneLoader:Initialize();

  apolloengine.Framework:AddSynchronizeUpdateCallback(self._Agent, "SyncUpdate");--添加同步更新
  apolloengine.Framework:AddKeyboardCallback(self._Agent, "OnKeyboard");
  apolloengine.Framework:AddTouchCallback(self._Agent, "OnTouch");
  apolloengine.Framework:AddAfterRenderCallback(self._Agent, "AfterRender");

  self.running = false;

  self.begintime = venuscore.ITimerSystem:GetTimevalue();
  self.cumtime = 0;
  self.fps = 0;
  self._curMode = "";
  
  self:InitBlit();

  LOG("initialize done");

  if _PLATFORM_WINDOWS or _PLATFORM_MACOS then
    local testpath =  venuscore.IFileSystem:PathAssembly("test:../../vtuberscene2nd/vtuberscene/");
    self:SetRootPath(testpath,"",0,"",2,"",1);
    self:SetConfig({});
  end
  
    -- 创建一个伪摄像头数据,用于反序列化加载素材,在真正渲染之前会更新
  self.mCamTexStream = apolloengine.TextureStream();
  self.mCamTexStream:SetStreamType(mathfunction.vector2(16, 16), apolloengine.TextureEntity.PF_R8G8B8);
  self.mCamTexEntity = apolloengine.TextureEntity();
  self.mCamTexEntity:PushMetadata(
          apolloengine.TextureBufferMetadata(
                  apolloengine.TextureEntity.TU_WRITE,
                  1, false,
                  apolloengine.TextureEntity.TW_CLAMP_TO_EDGE,
                  apolloengine.TextureEntity.TW_CLAMP_TO_EDGE,
                  apolloengine.TextureEntity.TF_LINEAR,
                  apolloengine.TextureEntity.TF_LINEAR,
                  self.mCamTexStream ));
  self.mCamTexEntity:SetKeepSource(true);
  self.mCamTexEntity:CreateResource();
  apolloengine.DeviceResource:PushDeviceResource(
          apolloengine.DeviceResource.DEVICE_CAPTURE,
          self.mCamTexEntity);

  _PROFILER_START();
  _COROUTINES_OFF();
  
  return true;
end

function main:_SetRenderTarget()
  if self.rendertarget
  and self.defScene then
    local main_camera = self.defScene:GetMainCamera();
    main_camera:AttachRenderTarget(self.rendertarget);
  end  
end


function main:SetConfig(config)
  if config.basescene and config.basescene~=self.scenecache then
    self:SwitchScene(config.sandbox, config.basescene);
    self.scenecache = config.basescene;
  end
  
  if self.vtuberManager == nil then
    WARNING("main_vtuber setConfig vtuberManager is nil");
    return false;
  end
  if _PLATFORM_WINDOWS or _PLATFORM_MACOS then
    config.basemodel = venuscore.IFileSystem:PathAssembly("test:../../vtuberscene2nd/vtuberscene/");
    config.dress = {};
    config.dress.cloth = venuscore.IFileSystem:PathAssembly("test:../../vtuberscene2nd/dress/upperbase/");
    config.dress.skirt = venuscore.IFileSystem:PathAssembly("test:../../vtuberscene2nd/dress/skirt/");
    config.dress.socks = venuscore.IFileSystem:PathAssembly("test:../../vtuberscene2nd/dress/siwa/");
    config.dress.shoes = venuscore.IFileSystem:PathAssembly("test:../../vtuberscene2nd/dress/shoes/");
    config.hair = {};
    config.hair.wholehair = venuscore.IFileSystem:PathAssembly("test:../../vtuberscene2nd/hair/shorthair0/");
    config.eyematatt = {};
    config.eyematatt._Color0 = mathfunction.vector4(0.98039, 0.15686, 0.15686, 1.0);
    config.eyematatt._Color1 = mathfunction.vector4(0.08235, 0.0, 0.33725, 1.0);
    config.facebsatt = {};
    config.facebsatt = {};
    config.facebsatt.face_thin_fat = 1;
  end
  self.vtuberManager:LoadConfig(config);
  
  return true;
end

function main:SetConfigStr(configstr)
  collectgarbage();
  local config = self:String2Config(configstr);
  return self:SetConfig(config);
end

function main:SetSignalFlag(flag)
  if self.vtuberManager == nil then
    WARNING("main_vtuber SetSignalFlag vtuberManager is nil");
    return false;
  end
  return self.vtuberManager:SetSignalFlag(flag);
end
  
function main:String2Config(configstr)
  local config = venusjson.LoadJsonString(configstr);
  return config;
end

function main:GetConfig()
  if self.vtuberManager == nil then
    WARNING("main_vtuber vtuberManager is nil");
    return false;
  end
  
  return self.vtuberManager:GetVtuberConfig();
end

function main:GetConfigStr()
  if self.vtuberManager == nil then
    WARNING("main_vtuber vtuberManager is nil");
    return false;
  end
  
  return self.vtuberManager:GetVtuberConfigStr();
end

function main:SwitchScene(sandboxPath, bundlePath)
  local loadPath = bundlePath;
  if sandboxPath then
    loadPath = sandboxPath..bundlePath;
  end
  local projpath = loadPath.."/";
  local configpath = projpath.."scene.json";
  self.vtuberManager = nil;
  self.photoManager = nil;
  self.isloaded =  SceneLoader:LoadAsSticker( configpath);

  self.defScene = SceneLoader.sceneList["Sticker"].nativeScene;
  self.rootNode = self.defScene:GetRootNode();
  self:_SetRenderTarget();--给主相机添加FBO

  local scriptComs = {};
  self:GetAllScriptComponent(self.rootNode, scriptComs);
  self.scriptComs = scriptComs;
  for i = 1, #self.scriptComs do
    if self.scriptComs[i] ~= nil then
      local paraSet = self.scriptComs[i].Instances;
      for scrKey, scrValue in pairs(paraSet) do
        if string.find(tostring(scrKey), "vtuberManager_behavior") then
          self.vtuberManager = scrValue;
          WARNING("Initialize get vtuberManager");
        elseif string.find(tostring(scrKey), "vtuberPhotoMgr_behavior") then
          self.photoManager = scrValue;
          WARNING("Initialize get vtuberPhotoMgr");
        end
      end
    end
  end

  if self.vtuberManager == nil then
    WARNING("main_vtuber SwitchScene vtuberManager is nil");
    return false;
  end

  self:SwitchViewConfig("Prepare",0.0);

  return true;
end

function main:SetRootPath(scenePath, modelPath, modelidx, matPath, matidx, bgPath, bgidx)
  WARNING("main_vtuber SetRootPath: "..scenePath);
  return self:SwitchScene(nil, scenePath);
end

function main:_Running()
  _COROUTINES_ON();

  self:_DispatchVtuberTouchGesture();
  self:_DispatchScreenRotation();
  self:_DispatchTouchGesture();
  
  if self.isloaded and not self.running then
    self.running = true;
    SceneLoader:UpdateSticker(self.def);
    self.running = false;
  end

  _PROFILER_UPDATE();
  _COROUTINES_OFF();

end

function main:Ontime()
  self:_Running();
end

function main:Update()
  self:_Running();
end

function main:SyncUpdate()
  --_COROUTINES_ON();
  local def = self:Timespan();
  self.def = def;
  self:_Running();
  --_COROUTINES_OFF();
end

function main:AfterRender()
  _COROUTINES_ON();
  if self.isloaded and not self.running then
    self.running = true;
    SceneLoader:AfterRenderSticker(self.def);
    self.running = false;
  end
  _COROUTINES_OFF();
end

function main:GetAllScriptComponent(node, scriptComs)
  local childs = node:GetChildrens();
  for _,v in pairs(childs) do
    local scriptCom = v:GetComponent(apolloengine.Node.CT_SCRIPT);
    if scriptCom then
      table.insert(scriptComs, scriptCom);
    end
    self:GetAllScriptComponent(v, scriptComs);
  end
end

function main:Timespan()
  local now = venuscore.ITimerSystem:GetTimevalue();
  local def = now - self.begintime;
  self.begintime = now;

  self.cumtime = self.cumtime + def;
  self.fps = self.fps + 1;
  if self.cumtime > 5 then
    local fps = self.fps / self.cumtime;
    self.fps = 0;
    self.cumtime = 0;
    LOG("logic fps "..fps);
  end
  --防止间隔过长导致效果突变太大效果不自然
  def = def < 0.1 and def or 0.03;
  return def;
end

function main:OnResizeView(x,y,w,h)

end

function main:_DispatchVtuberTouchGesture()
  if self.vtuberManager==nil then
    return;
  end
  local viewcontroll = self.vtuberManager:GetViewController();
  if viewcontroll==nil then
    return;
  end
  local staticID = BD.Event.GestureScalePath
  local actionSystem = apolloengine.IActionSystem
  local sensorType = apolloengine.SensorsTrackingComponent.VENUS_SENSOR_GESTURE_SCALE
  local number = actionSystem:GetSensorDataCount(sensorType)
  if number==1 then 
    local result = actionSystem:GetSensorDataByIndex(sensorType, 1);
    local scale = result.z;
    if self.currentmove==nil then
     self.currentmove = self.startmove * scale;
    else
     self.currentmove = self.currentmove * scale;
    end
    local moved = self.currentmove - self.startmove;
    viewcontroll:MoveCamera(moved);
  else
   self.startmove = 1;
   self.currentmove = nil;
  end
end

function main:_DispatchScreenRotation()
  local staticID = BD.Event.ScreenRotationChangePath
  if ED:HasCompListener(staticID) then
    -- someone want rotation data
    local type = BD.Event.ScreenRotationEvent.SCREEN_ROTATION_CHANGE
    local hasRotation = ED:HasEventListener(staticID, type);
    if hasRotation then
      local rotation = likeapp.AI:GetScreenRotation();
      local event = {
        staticID = staticID;
        eventType = type;
        params = rotation;
      };
      ED:PushEvent(event);
    end
  end
end

-- 手势缩放 和 旋转
function main:_DispatchTouchGesture()
  local eBegin = BD.Event.GestureGlobalEvent.GESTURE_BEGIN
  local eKeep  = BD.Event.GestureGlobalEvent.GESTURE_KEEP
  local eEnd   = BD.Event.GestureGlobalEvent.GESTURE_END

  local tBegin = apolloengine.SensorsTrackingComponent.VALUE_GESTURE_BEGIN
  local tKeep = apolloengine.SensorsTrackingComponent.VALUE_GESTURE_KEEP
  local tEnd = apolloengine.SensorsTrackingComponent.VALUE_GESTURE_END

  local staticID = BD.Event.GestureScalePath
  local actionSystem = apolloengine.IActionSystem

  if ED:HasCompListener(staticID) then
    local sensorType = apolloengine.SensorsTrackingComponent.VENUS_SENSOR_GESTURE_SCALE
    local number = actionSystem:GetSensorDataCount(sensorType)
    for i = 1, number do
      local result = actionSystem:GetSensorDataByIndex(sensorType, i);
      local eventType
      if result.value == tBegin then
        eventType = eBegin
      elseif   result.value == tKeep then
        eventType = eKeep
      elseif result.value == tEnd then
        eventType = eEnd
      end
      if eventType ~= nil then
        local event = {
          staticID = staticID;
          eventType = eventType;
          params = result ; -- params is {value = KEEP, code = scaleValue, x = 0.0, y = 0.0 } -- scale center or rotate center
        };
        ED:PushEvent(event);
      end
    end
  end
  staticID = BD.Event.GestureRotatePath
  if ED:HasCompListener(staticID) then
    local sensorType = apolloengine.SensorsTrackingComponent.VENUS_SENSOR_GESTURE_ROTATE
    local number = actionSystem:GetSensorDataCount(sensorType)
    for i = 1, number do
      local result = actionSystem:GetSensorDataByIndex(sensorType, i);
      local eventType
      if result.value == tBegin then
        eventType = eBegin
      elseif   result.value == tKeep then
        eventType = eKeep
      elseif result.value == tEnd then
        eventType = eEnd
      end
      if eventType ~= nil then
        local event = {
          staticID = staticID;
          eventType = eventType;
          params = result ; -- params is {value = KEEP, code = scaleValue, x = 0.0, y = 0.0 } -- scale center or rotate center
        };
        ED:PushEvent(event);
      end
    end
  end
end

function main:VtuberHandleTouch(touchinfo)
  if self.CurrentPress==nil then
    self.CurrentPress = 0;
  end
  
  if self.vtuberManager==nil then
    return;
  end
  local viewcontroll = self.vtuberManager:GetViewController()
  if viewcontroll==nil then
    ERROR("view controll is nil, controll nothing!!!")
    return;
  end
  local isMove = false;
  
  self.CurrentPressPoint = {};
  
  local PointCount = touchinfo:GetPointCount();


  for i = 1, PointCount do
    local touchType = touchinfo:GetTouchType(i);
    local index = touchinfo:GetTouchID(i);
    local point = touchinfo:GetTouchPoint(i);

    --if touchType==apolloengine.TouchInfo.TT_PRESS or touchType==apolloengine.TouchInfo.TT_MOVE then
    table.insert(self.CurrentPressPoint,point);
    -- ERROR(" add a pressed point in to array ");      
    --end 
  end
  
  self.LastCtrlState = self.CurCtrlState;
  
  if PointCount==1 then
    self.CurCtrlState = 1;
  elseif PointCount==2 then
    self.CurCtrlState = 2;
  else
    self.CurCtrlState = nil;
  end
--[[
  if self.CurCtrlState==2  then
    if self.LastCtrlState~=2 then
      viewcontroll:SetMoveWay((self.CurrentPressPoint[2] + self.CurrentPressPoint[1])*0.5);
      self.resetdistance = (self.CurrentPressPoint[2] - self.CurrentPressPoint[1]):Length();
    end
    self.currentdistance = (self.CurrentPressPoint[2] - self.CurrentPressPoint[1]):Length();
    if self.lastdistance~= nil then
      local moved = self.currentdistance - self.lastdistance;
      viewcontroll:MoveCamera(moved);
    end
    self.lastdistance = self.currentdistance;
  else
    if self.LastCtrlState==2 then
      --viewcontroll:ClearMoveWay();
    end
    self.currentdistance = nil;
    self.lastdistance = nil;
    self.resetdistance = nil;
  end]]--
  
  if self.CurCtrlState==1  then
    local touchType = touchinfo:GetTouchType(1);
    if touchType~=apolloengine.TouchInfo.TT_RELEASE then
      self.currentpos = touchinfo:GetTouchPoint(1);
      if self.lastpos~=nil then
        local distance = self.currentpos - self.lastpos;
        viewcontroll:RotateAvatar(distance.mx);
        viewcontroll:MoveAvatar(distance.my);
      end
      self.lastpos = self.currentpos;
    else
      self.lastpos = nil;
      self.currentpos = nil;
    end
  end
end

function main:OnTouch(touchinfo)
  self:VtuberHandleTouch(touchinfo);
  
  if ED:HasCompListener(BD.Event.TouchGlobalPath) then

    local ePress = BD.Event.TouchGlobalEvent.TOUCH_PRESS
    local eMove = BD.Event.TouchGlobalEvent.TOUCH_MOVE
    local eRelease = BD.Event.TouchGlobalEvent.TOUCH_RELEASE
    local path = BD.Event.TouchGlobalPath

    local tPress = apolloengine.TouchInfo.TT_PRESS
    local tMove = apolloengine.TouchInfo.TT_MOVE
    local tRelease = apolloengine.TouchInfo.TT_RELEASE

    local hasPress = ED:HasEventListener(path, ePress)
    local hasMove = ED:HasEventListener(path, eMove)
    local hasRelease = ED:HasEventListener(path, eRelease )

    local PointCount = touchinfo:GetPointCount();
    for i = 1, PointCount do

      local touchType = touchinfo:GetTouchType(i);

      if touchType == tPress then

        if hasPress then
          local event = {
            staticID = BD.Event.TouchGlobalPath;
            eventType = ePress;
            params = {
              index = touchinfo:GetTouchID(i),
              point = touchinfo:GetTouchPoint(i) -- vector2f
            };
          };
          ED:PushEvent(event);
        end

      elseif touchType == tMove then

        if hasMove then
          local event = {
            staticID = BD.Event.TouchGlobalPath;
            eventType = eMove;
            params = {
              index = touchinfo:GetTouchID(i),
              point = touchinfo:GetTouchPoint(i) -- vector2f
            };
          };
          ED:PushEvent(event);
        end

      elseif touchType == tRelease then

        if hasRelease then
          local event = {
            staticID = BD.Event.TouchGlobalPath;
            eventType = eRelease;
            params = {
              index = touchinfo:GetTouchID(i),
              point = touchinfo:GetTouchPoint(i) -- vector2f
            };
          };
          ED:PushEvent(event);
        end

      end
    end
  end
end

function main:Exit()
  WARNING("main_vtuber Exit");
  apolloengine.Framework:RemoveSynchronizeUpdateCallback(self._Agent, "SyncUpdate");
  apolloengine.Framework:RemoveTouchCallback(self._Agent,"OnTouch");
  apolloengine.Framework:RemoveKeyboardCallback(self._Agent,"OnKeyboard");
  apolloengine.Framework:RemoveAfterRenderCallback(self._Agent,"AfterRender");

  --self:UnLoadFaceReshape();
  --self:UnLoadFaceBeauty();
  SceneLoader:ReleaseAllResource();
  --local sceneID = self.facecuteScene:GetStaticID();
  --apolloengine.SceneManager:DeleteSceneByID(sceneID);
  apolloengine.SceneManager:DeleteScene(apolloDefine.default_scene_name);
  return true;
end

--设置摄像机FBO
function main:AttachRenderTarget(fbo, texture)
  LOG("platform android, fbo:"..fbo);
  self.rendertarget = apolloengine.RenderTargetEntity();--创建一个FBO
  self.rendertarget:PushMetadata(--设置FBO格式
          apolloengine.RenderTargetResourceMetadata(
                  apolloengine.RenderTargetEntity.RT_RENDER_TARGET_2D,
                  apolloengine.Framework:GetResolution(),--分辨率
                  fbo));
  self.outputtexture = self.rendertarget:MakeTextureAttachment(
          apolloengine.RenderTargetEntity.TA_COLOR_0);--增加color0纹理
  self.outputtexture:PushMetadata(--创建纹理
          apolloengine.TextureResourceMetadata(
                  apolloengine.Framework:GetResolution(),
                  texture));
  self.rendertarget:CreateResource();
  self:_SetRenderTarget();
  return true ;
end

--设置摄像机采集图片尺寸和id
function main:SetVideoTexture(size, video, pixelFormat)
  local tex = nil --暂时直接拷贝过来，但是这块更合理的写法是放到c++层，根据平台自动给个TextureEntity
  if apolloengine.Framework:IsUsingOpenGL() then
    tex = apolloengine.TextureEntity();
    tex:PushMetadata(
            apolloengine.TextureResourceMetadata(
                    mathfunction.vector2(size[1], size[2]),
                    video,
                    pixelFormat,
                    true -- C++ ITextureResource::RefrenceTexture
            ));
    tex:CreateResource();
  else
    -- TA_COLOR_0: venus输出纹理
    -- TA_COLOR_1: venus输入纹理
    -- Metal渲染的时候，外部传入的texid和size都没有作用了，输入的尺寸变化走 IWindow:ResizeView
    tex = apolloengine.Framework:GetVenusInputTex();
  end

  self.mCamTexEntity = tex;
  self.mCamTexStream = tex:GetSourceStream();
  apolloengine.DeviceResource:PushDeviceResource(
    apolloengine.DeviceResource.DEVICE_CAPTURE,
    self.mCamTexEntity);
  likeapp.AI:OnUpdateStream(self.mCamTexStream);  
  collectgarbage();
  return true;
end

function main:ThreadExit()
    collectgarbage();
    return true ;
end

function main:LuaGC()
  collectgarbage();
end

-- function main:SwitchDress(dresspart, dresspath)
--   local dressmgr = self.vtuberManager:GetDressMgr()
--   if dressmgr then
--     local res = dressmgr:LoadDress(dresspart, dresspath);
--     return res;
--   end
--   return false;
-- end

-- function main:SwitchJewellery(jewelleryPart, jewelleryPath)
--   local jewelleryMgr = self.vtuberManager:GetJewelleryMgr();
--   if jewelleryMgr then
--     local res = jewelleryMgr:LoadRing(jewelleryPart, jewelleryPath);
--     return res;
--   end
--   return false;
-- end

-- function main:SwitchHair(hairPart, hairPath)
--   local hairMgr = self.vtuberManager:GetHairMgr();
--   if hairMgr then
--     local res = hairMgr:LoadHair(hairPart, hairPath);
--     return res;
--   end
--   return false;
-- end

function main:KneadFaceConfig(faceAttr)
  local faceMgr = self.vtuberManager:GetFaceMgr();
  if faceMgr then
    faceMgr:LoadConfig(faceAttr);
    return true;
  end
  return false;
end

function main:EnterEditFaceMode()
  local faceMgr = self.vtuberManager:GetFaceMgr();
  if faceMgr then
    faceMgr:EnterEditFaceMode();
    return true;
  end
  return false;
end

function main:ExitEditFaceMode()
  local faceMgr = self.vtuberManager:GetFaceMgr();
  if faceMgr then
    faceMgr:ExitEditFaceMode();
    return true;
  end
  return false;
end

function main:KneadFace(key, value)
  local faceMgr = self.vtuberManager:GetFaceMgr();
  if faceMgr then
    faceMgr:SetFaceValue(key, value);
    return true;
  end
  return false;
end

-- function main:ChangeMaterialParam(type,configstr)
--   local config = self:String2Config(configstr);
--   if type==0 then
--     return self:ChangeEyeParam(config);
--   elseif type==1 then
--     return self:ChangeEyeBrowParam(config);
--   elseif type==2 then
--     return self:ChangeHairParam(config);
--   elseif type==3 then
--     return self:ChangeFaceParam(config);
--   elseif type==4 then
--     return self:ChangeBodyParam(config);
--   else
--     ERROR("ChangeMaterialParam unknow type");
--     return false;
--   end
-- end

-- function main:ChangeEyeParam(eyeConfig)
--   local materialMgr = self.vtuberManager:GetMaterialMgr();
--   if materialMgr then
--     if self.vtuberManager.humanModel then
--       materialMgr:SetHumanMaterial(self.vtuberManager.humanModel);
--       return materialMgr:ChangeParam("eyematatt", eyeConfig);
--     else
--       ERROR("main_vtuber vtuberManager.humanModel is nil");
--     end
--   else
--     ERROR("main_vtuber materialMgr nil");
--   end
--   return false;
-- end

-- function main:ChangeEyeBrowParam(eyeBrowConfig)
--   local materialMgr = self.vtuberManager:GetMaterialMgr();
--   if materialMgr then
--     if self.vtuberManager.humanModel then
--       materialMgr:SetHumanMaterial(self.vtuberManager.humanModel);
--       return materialMgr:ChangeParam("eyebrowmatatt", eyeBrowConfig);
--     else
--       ERROR("main_vtuber vtuberManager.humanModel is nil");
--     end
--   else
--     ERROR("main_vtuber materialMgr nil");
--   end
--   return false;
-- end

-- function main:ChangeHairParam(hairConfig)
--   local materialMgr = self.vtuberManager:GetMaterialMgr();
--   if materialMgr then
--     local hairMgr = self.vtuberManager:GetHairMgr();
--     local hairNodeConfig;
--     if hairMgr then
--       hairNodeConfig = hairMgr:GetConfig();
--     end
--     if hairNodeConfig then
--       for k, v in pairs(hairNodeConfig) do
--         if materialMgr and v ~= "" then
--           materialMgr:SetHairMaterial(k, v);
--           return materialMgr:ChangeParam(k.."matatt", hairConfig);
--         end
--       end
--     else
--       ERROR("main_vtuber hairMaterial is nil");
--     end
--   else
--     ERROR("main_vtuber materialMgr nil");
--   end
--   return false;
-- end


-- function main:ChangeFaceParam(faceConfig)
--   local materialMgr = self.vtuberManager:GetMaterialMgr();
--   if materialMgr then
--     if self.vtuberManager.humanModel then
--       materialMgr:SetHumanMaterial(self.vtuberManager.humanModel);
--       return materialMgr:ChangeParam("facematatt", faceConfig);
--     else
--       ERROR("main_vtuber vtuberManager.humanModel is nil");
--     end
--   else
--     ERROR("main_vtuber materialMgr nil");
--   end
--   return false;
-- end

-- function main:ChangeBodyParam(bodyConfig)
--   local materialMgr = self.vtuberManager:GetMaterialMgr();
--   if materialMgr then
--     if self.vtuberManager.humanModel then
--       materialMgr:SetHumanMaterial(self.vtuberManager.humanModel);
--       return materialMgr:ChangeParam("bodymatatt", bodyConfig);
--     else
--       ERROR("main_vtuber vtuberManager.humanModel is nil");
--     end
--   else
--     ERROR("main_vtuber materialMgr nil");
--   end
--   return false;
-- end

-- function main:SwitchPart(type,part,path)
--   if type==0 then
--     return self:SwitchDress(part,path);
--   elseif type==1 then
--     return self:SwitchJewellery(part,path);
--   elseif type==2  then
--     return self:SwitchHair(part,path);
--   else
--     ERROR("unknow type,can't load!");
--     return false;
--   end
-- end


function main:SwitchGameMode(mode)
  local   PrepareState = 10;
  local   DressingState = 100;
  local   FaceEditState = 101;
  local   ViewBGState = 200;
  local   LivingState = 1000;
  
  local modekey = "Prepare";
  if mode==PrepareState then
    modekey =  "Prepare";
  elseif mode==DressingState then
    modekey =  "Dressing";
  elseif mode==ViewBGState then
    modekey =  "ViewBG";
  elseif mode==FaceEditState then
    modekey =  "FaceEdit";
  elseif mode==LivingState then
    modekey =  "Living";
  end
  

  self._curMode = mode;
  self:SwitchViewConfig(modekey, 0.5);
  -- if self.vtuberManager ~= nil then
  --     self.vtuberManager:SwitchBackground(self._curMode );
  -- end
  return true;
end

function main:SwitchViewConfig(modekey, time)
  if self.vtuberManager == nil then
    WARNING("SwitchViewConfig failed, vtuberManager == nil !");
    return false;
  end
  local viewcontroll = self.vtuberManager:GetViewController();
  if viewcontroll==nil then
    return false;
  end
  viewcontroll:SwitchViewConfig(modekey, time);
  return true;
end

function main:SwitchModel(modelPath, modelIndex)
  if self.modelIndex ~= nil and self.modelIndex == modelIndex then
    return true;
  end

  local res = self.vtuberManager:LoadModel(modelPath);
  self.modelIndex = modelIndex;
  self.matIndex = 0;
  return res;
end

-- function main:SwitchMaterialPreset(matPath, presetidx)
--   local rootPath = matPath.."/";
--   venuscore.IFileSystem:SetResourcePath(rootPath);
--   if self.vtuberManager == nil then
--     WARNING("main_vtuber vtuberManager is nil");
--     return false;
--   end

--   if self.matIndex ~= nil and self.matIndex == presetidx then
--     return true;
--   end

--   local res = self.vtuberManager:ChangeMaterial(matPath, presetidx, self.modelIndex);
--   if res == true then
--     self.matIndex = presetidx;
--   end
--   return res;
-- end

function main:PlayAnimation(aniIdx)
  if self.vtuberManager == nil then
    WARNING("main_vtuber PlayAnimation vtuberManager is nil");
    return false;
  end
  if aniIdx < 0 then
    WARNING("main_vtuber PlayAnimation aniIdx error")
    return false;
  end
  local res = self.vtuberManager:ChangeAnimation(aniIdx);
  return res;
end

function main:TakePhoto(configstr, width, height)
  WARNING("vtuber main TakePhoto");
  local res = false;
  if self.photoManager == nil then
    WARNING("vtuber main TakePhoto photoManager is nil");
    return res;
  end
  local config = self:String2Config(configstr);
  res = self.photoManager:TakePhoto(config, width, height);
  return res;
end

function main:GetTextureStream()
  WARNING("vtuber main GetTextureStream");
  local res = false;
  if self.photoManager ~= nil then
    res = self.photoManager:GetTextureStream();
  else
    WARNING("vtuber main GetTextureStream photoManager is nil");
  end
  return res;
end

function main:SwitchCameraType(typeidx)
  WARNING("main_vtuber: SwitchCameraType "..typeidx);
end

function main:GetVtuberSendingMsg()
  return VtuberMsgHub:Serialize(VtuberMsgHub.SendingId)
end

function main:PutVtuberReceivingMsg(binary)
  VtuberMsgHub:Deserialize(VtuberMsgHub.ReceivingId, binary)
end

function main:Sleep(isSleep)
   if self.vtuberManager == nil then
    WARNING("main_vtuber Sleep vtuberManager is nil");
    return false;
  end

  self.vtuberManager:DisplayIcon(isSleep);
  self.vtuberManager:IsResetBones(isSleep);
  self.vtuberManager:IsCloseEye(isSleep);
  return true
end

function main:SetTracking(isActive)
  if self.vtuberManager ~= nil and self.vtuberManager.Recognition ~= nil then
    self.vtuberManager.Recognition:GetHostNode().Active = isActive;
    return true;
  end
  return false;
end

function main:GetBlendShapeWeights()
  -- in case of vtuberManager is not initialized
  if self.vtuberManager == nil then
    return ""
  end

  --NOTE(Chen Wei): since lua can not return string-array or float-array to c++ layer,
  --                here we just pack all-bs-weights to one formated-string
  local weight_dict = self.vtuberManager:GetBlendShapeWeights()
  local weights_dict_str = ""
  for name,value in pairs(weight_dict) do
    weights_dict_str = weights_dict_str .. name .. " " .. value .. " "
  end
  return weights_dict_str
end

function main:GetBlendShapeNum()
  -- in case of vtuberManager is not initialized
  if self.vtuberManager == nil then
    return 0
  end

  local weight_dict = self.vtuberManager:GetBlendShapeWeights()
  return #weight_dict
end

return main;
