local enhance = {}

enhance.Scenes = {}
setmetatable(enhance.Scenes, {__mode = "k"}) --弱引用 

enhance.FT_UPDATE = 1;
enhance.FT_LATEUPDATE = 2;


function enhance:_GetOrInitTable(scene, ft)
  local coms ;
  if not self.Scenes[scene] then
    self.Scenes[scene] = {};
    local t1 = setmetatable({}, {__mode = "k"}) --弱引用
    self.Scenes[scene][enhance.FT_UPDATE] = t1;
    local t2 = setmetatable({}, {__mode = "k"})
    self.Scenes[scene][enhance.FT_LATEUPDATE] = t2;
    if ft == enhance.FT_UPDATE then
      coms = t1
    elseif ft == enhance.FT_LATEUPDATE then
      coms = t2
    end
  else
    if ft == enhance.FT_UPDATE then
      coms = self.Scenes[scene][enhance.FT_UPDATE]
    elseif ft == enhance.FT_LATEUPDATE then
      coms = self.Scenes[scene][enhance.FT_LATEUPDATE]
    end
  end
  return coms ;
end

--function enhance:_Insert(scene, com, ft, func)
--  local funcs = self:_GetOrInitTable(scene, com)
--  funcs[ft] = func;
--end

function enhance:Enhance(meta, enhances)  
  local fNativeAwake = meta.Awake;

  local _AwakeFunction = function(self)
    fNativeAwake(self);
    local node = self:GetHostNode();
    local scene = node:GetHostScene();
    if enhances[enhance.FT_UPDATE] ~= nil then
      local coms = enhance:_GetOrInitTable(scene, enhance.FT_UPDATE)
      coms[self] = enhances[enhance.FT_UPDATE];
    end
    if enhances[enhance.FT_LATEUPDATE] ~= nil then
      local coms = enhance:_GetOrInitTable(scene, enhance.FT_LATEUPDATE)
      coms[self] = enhances[enhance.FT_LATEUPDATE];
    end
  end
  rawset(meta,"Awake",_AwakeFunction)
end


function enhance:Update(scene, dt)
  local s = self.Scenes[scene] ;
  local coms = s and s[enhance.FT_UPDATE] or nil  ;
  local toDel
  if coms then
    for com, func in pairs(coms) do
      if getmetatable(com) ~= nil then -- vc.isNil(com)
          func(com, dt);
      else
        toDel = toDel or {}
        table.insert(toDel, com);
        WARNING("found isNil "..tostring(com))
      end
    end
  end

  if toDel then
    for _, com in pairs(toDel) do
      coms[com] = nil
    end
  end

end

function enhance:LateUpdate(scene, dt)
  local s = self.Scenes[scene] ;
  local coms = s and s[enhance.FT_LATEUPDATE] or nil  ;
  local toDel
  if coms then
    for com, func in pairs(coms) do
      if getmetatable(com) ~= nil then
          func(com, dt);
      else
        toDel = toDel or {}
        table.insert(toDel, com);
        WARNING("found isNil "..tostring(com))
      end
    end
  end

  if toDel then
    for _, com in pairs(toDel) do
      coms[com] = nil
    end
  end

end


return enhance;