--local apolloengine = require "apolloengine"
local cv = require "computervisionfunction"
local DeviceResource = require "apolloengine.deviceresource"
local vc = require "libvenuscore"
local enhance = require "apolloengine.components.enhance"

local _fNativeDestroy = cv.SegmentComponent.Destroy;

local allSegs = {}; --- all scenes use one segment result, destroy one of them, use remaining segment
local currentSeg = nil

local _nativeAwake = cv.SegmentComponent.Awake;
local AwakeFoo = function(self)
  _nativeAwake(self);   -- it may call first

  if self.Type == cv.SegmentComponent.cvHairSegment then
    local tex = self:GetMask();
    DeviceResource:PushDeviceResource(DeviceResource.DEVICE_SEGMENT_HAIR, tex);
  else
    if not (self.Type == cv.SegmentComponent.cvFacePars_Face or self.Type == cv.SegmentComponent.cvFacePars_Int_Mouth) then

      local tex = self:GetMask();

      local staticID =  self:GetContentPath();
      allSegs[staticID] = tex;

      if _KRATOSEDITOR then
        -- editor scene first and game scene later
        currentSeg = tex ;
      else
        if currentSeg ~= nil and not vc.isNil(currentSeg) then
          tex = currentSeg;
        else
          currentSeg = tex ;
        end
      end
        DeviceResource:PushDeviceResource(DeviceResource.DEVICE_SEGMENT, tex);
        DeviceResource:PushDeviceResource(DeviceResource.DEVICE_FACEPARSE_LEFT_BROW, tex);
        DeviceResource:PushDeviceResource(DeviceResource.DEVICE_FACEPARSE_RIGHT_BROW, tex);
        DeviceResource:PushDeviceResource(DeviceResource.DEVICE_FACEPARSE_LEFT_EYE, tex);
        DeviceResource:PushDeviceResource(DeviceResource.DEVICE_FACEPARSE_RIGHT_EYE, tex);
        DeviceResource:PushDeviceResource(DeviceResource.DEVICE_FACEPARSE_NOSE, tex);
        DeviceResource:PushDeviceResource(DeviceResource.DEVICE_FACEPARSE_EXT_MOUTH,tex);
        DeviceResource:PushDeviceResource(DeviceResource.DEVICE_FACEPARSE_INT_MOUTH, tex);
        DeviceResource:PushDeviceResource(DeviceResource.DEVICE_FACEPARSE_LIP, tex);
        DeviceResource:PushDeviceResource(DeviceResource.DEVICE_FACEPARSE_HEAD, tex);
    end
  end

end

local DestroyFoo = function (self)

  local staticID =  self:GetContentPath();
  allSegs[staticID] = nil;
  currentSeg = nil;

  local invalidTex = {}
  for id, tex in pairs(allSegs) do
    --WARNING("DestroyFoo id "..tostring(id)..", tex"..tostring(tex))
    if vc.isNil(tex) then
      table.insert(invalidTex, id);
    else
      DeviceResource:PushDeviceResource(DeviceResource.DEVICE_SEGMENT, tex);
      currentSeg = tex ;
      break;
    end
  end

  for _, id in pairs(invalidTex) do
    allSegs[id] = nil ;
  end

  _fNativeDestroy(self);
end


--此处必须用rawset，否则会把UPDATE函数设置到cv.SegmentComponent的元表的UPDATE中。
rawset(cv.SegmentComponent,"Awake", AwakeFoo)
rawset(cv.SegmentComponent,"Destroy", DestroyFoo);