local apolloengine = require "apolloengine"
local mathfunction = require "mathfunction"
local venuscore = require "venuscore"
--local filelfs = require "window.editor.widget.assets.fileutility"
require "utility"

local vyouBloomEffect = {}

vyouBloomEffect.Queue = 310;
-- Iterative sampling times
vyouBloomEffect.iterations = 4;
-- sampling scale factor
-- eg: factor = 2 means that the result of each down sampling is 1/2 of the input
vyouBloomEffect.factor = 2;

function vyouBloomEffect:InitializePrefilter(host, size)
  -- pass3: prefilter
  self.prefilterMA = host:CreateMaterial( "comm:documents/vyou_matetial/vyouPostEffect3.material" );
  self.prefilterRO  = host:CreateRenderObject();
  self.prefilterRT = host:CreateRenderTarget(
          apolloengine.RenderTargetMetadata(
                  apolloengine.RenderTargetEntity.RT_RENDER_TARGET_2D,
                  apolloengine.RenderTargetEntity.ST_SWAP_UNIQUE,
                  mathfunction.vector4(self.fbosize:x(),self.fbosize:y(),0,0),
                  self.fbosize,
                  mathfunction.vector2(1.0,1.0)),
          apolloengine.TextureRenderMetadata(
                  apolloengine.RenderTargetEntity.ST_SWAP_UNIQUE,
                  self.fbosize,
                  apolloengine.TextureEntity.PF_RG11B10FLOAT));
end

function vyouBloomEffect:InitializeDownSampleBlur(host, size)
  --pass3-1 -> pass6: down-sample blur

  self.downSampleParas = {mathfunction.vector2(0.00326, 0.00156), mathfunction.vector2(0.00654, 0.00313), mathfunction.vector2(0.01316, 0.00629)};
  self.blurParas = {mathfunction.vector2(0.00654, 0.00313), mathfunction.vector2(0.01316, 0.00629), mathfunction.vector2(0.02632, 0.01266)};

  local scale = 0.5;
  -- 1/2
  self.downSampleAndTuneMA = host:CreateMaterial("comm:documents/vyou_matetial/vyouPostEffect3-1.material" );
  self.downSampleAndTuneRO  = host:CreateRenderObject();
  self.DOWNSAMPLE_TUNE_PARA = apolloengine.IMaterialSystem:NewParameterSlot(apolloengine.ShaderEntity.UNIFORM, "Params");
  --self.downSampleTunePara = mathfunction.vector4(0.5675, 65472.00, 0.69207, 0.34604);
  --host:RegisterScriptParameter(self, "downSampleTunePara");

  self.bloomSoftKnee = mathfunction.vector1(0.5675);
  self.bloomClamp = mathfunction.vector1(65472.00);
  self.bloomThreshold = mathfunction.vector1(0.69207);
  self.bloomUserIntensity = mathfunction.vector1(0.34604);

  host:RegisterScriptParameter(self, "bloomSoftKnee");
  host:RegisterScriptParameter(self, "bloomClamp");
  host:RegisterScriptParameter(self, "bloomThreshold");
  host:RegisterScriptParameter(self, "bloomUserIntensity");

  --down-sample
  self.downSampleMA = host:CreateMaterial("comm:documents/vyou_matetial/vyouPostEffect3-2.material" );
  self.downSampleRO  = host:CreateRenderObject();
  self.downSampleRTs = {};

  --1/2, 1/4, 1/8, 1/16
  scale = 1.0;
  for i = 1, self.iterations  do
    scale = scale / self.factor;
    self.downSampleRTs[i] = host:CreateRenderTarget(
            apolloengine.RenderTargetMetadata(
                    apolloengine.RenderTargetEntity.RT_RENDER_TARGET_2D,
                    apolloengine.RenderTargetEntity.ST_SWAP_UNIQUE,
                    mathfunction.vector4(self.fbosize:x()*scale,self.fbosize:y()*scale,0,0),
                    self.fbosize*scale,
                    mathfunction.vector2(scale,scale)),
            apolloengine.TextureRenderMetadata(
                    apolloengine.RenderTargetEntity.ST_SWAP_UNIQUE,
                    self.fbosize*scale,
                    apolloengine.TextureEntity.PF_RG11B10FLOAT));
  end

  scale = 1.0;
  --up-sample
  self.upSampleMA = host:CreateMaterial( "comm:documents/vyou_matetial/vyouPostEffect4.material" );
  self.upSampleRO  = host:CreateRenderObject();
  self.upSampleRTs = {};
  --1/2, 1/4, 1/8
  for i = 1, self.iterations - 1 do
    scale = scale / self.factor;
    self.upSampleRTs[i] = host:CreateRenderTarget(
            apolloengine.RenderTargetMetadata(
                    apolloengine.RenderTargetEntity.RT_RENDER_TARGET_2D,
                    apolloengine.RenderTargetEntity.ST_SWAP_UNIQUE,
                    mathfunction.vector4(self.fbosize:x()*scale,self.fbosize:y()*scale,0,0),
                    self.fbosize*scale,
                    mathfunction.vector2(scale,scale)),
            apolloengine.TextureRenderMetadata(
                    apolloengine.RenderTargetEntity.ST_SWAP_UNIQUE,
                    self.fbosize*scale,
                    apolloengine.TextureEntity.PF_RG11B10FLOAT));
  end

  self.blurMA = host:CreateMaterial("comm:documents/vyou_matetial/vyouPostEffect3-3.material" );
  self.blurRO  = host:CreateRenderObject();
  self.blurRTs = {};
  --1/4, 1/8, 1/16
  scale = 0.5;
  for i = 1, self.iterations - 1 do
    scale = scale / self.factor;
    self.blurRTs[i] = host:CreateRenderTarget(
            apolloengine.RenderTargetMetadata(
                    apolloengine.RenderTargetEntity.RT_RENDER_TARGET_2D,
                    apolloengine.RenderTargetEntity.ST_SWAP_UNIQUE,
                    mathfunction.vector4(self.fbosize:x()*scale,self.fbosize:y()*scale,0,0),
                    self.fbosize*scale,
                    mathfunction.vector2(scale,scale)),
            apolloengine.TextureRenderMetadata(
                    apolloengine.RenderTargetEntity.ST_SWAP_UNIQUE,
                    self.fbosize*scale,
                    apolloengine.TextureEntity.PF_RG11B10FLOAT));
  end

  self._SourceTexLowMip = apolloengine.IMaterialSystem:NewParameterSlot(apolloengine.ShaderEntity.UNIFORM, "_SourceTexLowMip");
  self.SourceTex_TexelSize = apolloengine.IMaterialSystem:NewParameterSlot(apolloengine.ShaderEntity.UNIFORM, "SourceTex_TexelSize");
end

function vyouBloomEffect:InitializeBloomLut(host, size)
  self.LutTexEntity = apolloengine.TextureEntity();
 -- local metaPath = filelfs:GetTextureMeta("comm:documents/texture/posteffect/cel_lut.png");
 -- local rMetaPath = filelfs:ConverToRelativePath(metaPath);

  self.LutTexEntity:PushMetadata(apolloengine.TextureFileMetadata("comm:documents/texture/posteffect/cel_lut.png"));
  self.LutTexEntity:SetKeepSource(true);
  self.LutTexEntity:SetJobType(venuscore.IJob.JT_SYNCHRONOUS);
  self.LutTexEntity:CreateResource();

  host:RegisterScriptParameter(self, "LutTexEntity");

  --self.UnityLutTexEntity = apolloengine.TextureEntity();
  --self.UnityLutTexEntity:PushMetadata(apolloengine.TextureFileMetadata(
  --        apolloengine.TextureEntity.TU_STATIC,
  --        apolloengine.TextureEntity.PF_AUTO,1, false,
  --        apolloengine.TextureEntity.TW_CLAMP_TO_EDGE,
  --        apolloengine.TextureEntity.TW_CLAMP_TO_EDGE,
  --        apolloengine.TextureEntity.TF_NEAREST,
  --        apolloengine.TextureEntity.TF_NEAREST,
  --        "comm:documents/texture/material/7-unitydefault.png"));
  --self.UnityLutTexEntity:SetJobType(venuscore.IJob.JT_SYNCHRONOUS);
  --self.UnityLutTexEntity:CreateResource();

  --pass7:bloom + lut
  self.bloomLutMA = host:CreateMaterial( "comm:documents/vyou_matetial/vyouPostEffect7.material" );
  self.bloomLutRO  = host:CreateRenderObject();
  self.bloomLutRT = host:CreateRenderTarget(
          apolloengine.RenderTargetMetadata(
                  apolloengine.RenderTargetEntity.RT_RENDER_TARGET_2D,
                  apolloengine.RenderTargetEntity.ST_SWAP_UNIQUE,
                  mathfunction.vector4(self.fbosize:x(),self.fbosize:y(),0,0),
                  self.fbosize,
                  mathfunction.vector2(1.0,1.0)),
          apolloengine.TextureRenderMetadata(
                  apolloengine.RenderTargetEntity.ST_SWAP_UNIQUE,
                  self.fbosize*1.0,
                  apolloengine.TextureEntity.PF_RG11B10FLOAT));

  self._Bloom_Texture = apolloengine.IMaterialSystem:NewParameterSlot(apolloengine.ShaderEntity.UNIFORM, "_Bloom_Texture");
  self._InternalLut = apolloengine.IMaterialSystem:NewParameterSlot(apolloengine.ShaderEntity.UNIFORM, "_InternalLut");
  self._UserLut = apolloengine.IMaterialSystem:NewParameterSlot(apolloengine.ShaderEntity.UNIFORM, "_UserLut");

  self.Bloom_Params = apolloengine.IMaterialSystem:NewParameterSlot(apolloengine.ShaderEntity.UNIFORM, "Bloom_Params");
  self.Bloom_RGBM = apolloengine.IMaterialSystem:NewParameterSlot(apolloengine.ShaderEntity.UNIFORM, "Bloom_RGBM");
  self.Lut_Params = apolloengine.IMaterialSystem:NewParameterSlot(apolloengine.ShaderEntity.UNIFORM, "Lut_Params");
  self.UserLut_Params = apolloengine.IMaterialSystem:NewParameterSlot(apolloengine.ShaderEntity.UNIFORM, "UserLut_Params");
  self.Vignette_Params1 = apolloengine.IMaterialSystem:NewParameterSlot(apolloengine.ShaderEntity.UNIFORM, "Vignette_Params1");
  self.Vignette_Params2 = apolloengine.IMaterialSystem:NewParameterSlot(apolloengine.ShaderEntity.UNIFORM, "Vignette_Params2");


  self.bloomIntensity = mathfunction.vector1(1.0);
  self.bloomColor = mathfunction.vector3(1.09732, 0.97371, 0.97371);
  self.bloomFastMode = false;

  host:RegisterScriptParameter(self, "bloomIntensity");
  host:RegisterScriptParameter(self, "bloomColor");
  host:RegisterScriptParameter(self, "bloomFastMode");

  self.lutParas = mathfunction.vector4(0.00391, 0.0625, 15.00, 1.07177);
  --self.useLutParas = mathfunction.vector4(0.00, 0.00, 0.00, 0.00);

  host:RegisterScriptParameter(self, "lutParas");

  self.vignetteParas1 = mathfunction.vector4(0.00, 0.00, 0.00, 1.00);
  self.vignetteParas2 = mathfunction.vector4(0.50, 0.50, 0.00, 1.00);

  self.vignetteColor = mathfunction.vector3(0.00, 0.00, 0.00);
  self.vignetteIntensity = mathfunction.vector1(0.00);

  self.vignetteCenter = mathfunction.vector2(0.50, 0.50);
  self.vignetteEnable = false;
  self.vignetteRoundness = mathfunction.vector1(1.0);

  host:RegisterScriptParameter(self, "vignetteColor");
  host:RegisterScriptParameter(self, "vignetteIntensity");
  host:RegisterScriptParameter(self, "vignetteCenter");
  host:RegisterScriptParameter(self, "vignetteEnable");
  host:RegisterScriptParameter(self, "vignetteRoundness");

end

function vyouBloomEffect:InitializeFinal(host, size)
  --pass8:tune + linear2gamma
  self.FinalMA = host:CreateMaterial( "comm:documents/vyou_matetial/vyouPostEffect8.material" );
  self.FinalRO  = host:CreateRenderObject();
  self.FinalRT = host:CreateRenderTarget(
          apolloengine.RenderTargetMetadata(
                  apolloengine.RenderTargetEntity.RT_RENDER_TARGET_2D,
                  apolloengine.RenderTargetEntity.ST_SWAP_UNIQUE,
                  mathfunction.vector4(self.fbosize:x(),self.fbosize:y(),0,0),
                  self.fbosize,
                  mathfunction.vector2(1.0,1.0)),
          apolloengine.TextureRenderMetadata(
                  apolloengine.RenderTargetEntity.ST_SWAP_UNIQUE,
                  self.fbosize,
                  apolloengine.TextureEntity.PF_RG11B10FLOAT));
  self.SourceSize = apolloengine.IMaterialSystem:NewParameterSlot(apolloengine.ShaderEntity.UNIFORM, "SourceSize");
  self.EnableLinear2Gamma = apolloengine.IMaterialSystem:NewParameterSlot(apolloengine.ShaderEntity.UNIFORM, "EnableLinear2Gamma");
  self.enablelinear2gamma = true;
  host:RegisterScriptParameter(self, "enablelinear2gamma");
end

function vyouBloomEffect:Initialize(host, size)
  self.fbosize            = mathfunction.vector2(size:x(), size:y());
  self.Queue = 310;
  self._SourceTex = apolloengine.IMaterialSystem:NewParameterSlot(apolloengine.ShaderEntity.UNIFORM, "_SourceTex");
  self:InitializePrefilter(host, size);
  self:InitializeDownSampleBlur(host, size);
  self:InitializeBloomLut(host, size);
  self:InitializeFinal(host, size);

  return self.Queue;
end

function vyouBloomEffect:Resizeview(size)
  self.fbosize = mathfunction.vector2(size:x(), size:y());
end


function vyouBloomEffect:Process(context, Original, Scene, Output)
  
  
  --pass3 prefilter full-size
  context:BeginRenderPass(self.prefilterRT, apolloengine.RenderTargetEntity.CF_COLOR);
  self.prefilterMA:SetParameter(self._SourceTex, Scene:GetAttachment(apolloengine.RenderTargetEntity.TA_COLOR_0));
  context:Draw(self.prefilterRO, self.prefilterMA);
  context:EndRenderPass();

  --pass3-1 -> pass3-7 downSample
  context:BeginRenderPass(self.downSampleRTs[1], apolloengine.RenderTargetEntity.CF_COLOR);
  self.downSampleAndTuneMA:SetParameter(self._SourceTex, self.prefilterRT:GetAttachment(apolloengine.RenderTargetEntity.TA_COLOR_0));
  self.downSampleAndTuneMA:SetParameter(self.DOWNSAMPLE_TUNE_PARA, mathfunction.vector4(self.bloomSoftKnee.mx, self.bloomClamp.mx, self.bloomThreshold.mx, self.bloomUserIntensity.mx));
  context:Draw(self.downSampleAndTuneRO, self.downSampleAndTuneMA);
  context:EndRenderPass();

  local scaleInv = 1.0;
  for i = 2, self.iterations do
    scaleInv = scaleInv * 0.5;
    --downSample
    context:BeginRenderPass(self.downSampleRTs[i], apolloengine.RenderTargetEntity.CF_COLOR);
    self.downSampleMA:SetParameter(
            self._SourceTex,
            self.downSampleRTs[i - 1]:GetAttachment( apolloengine.RenderTargetEntity.TA_COLOR_0 ) );
    self.downSampleMA:SetParameter( self.SourceTex_TexelSize, mathfunction.vector4(self.downSampleParas[i-1].mx, self.downSampleParas[i-1].my, self.fbosize:x() * scaleInv, self.fbosize:y() * scaleInv ) );
    context:Draw(self.downSampleRO, self.downSampleMA);
    context:EndRenderPass();
    --blur
    context:BeginRenderPass(self.blurRTs[i-1], apolloengine.RenderTargetEntity.CF_COLOR);
    self.blurMA:SetParameter(
            self._SourceTex,
            self.downSampleRTs[i]:GetAttachment( apolloengine.RenderTargetEntity.TA_COLOR_0 ) );
    self.blurMA:SetParameter( self.SourceTex_TexelSize, mathfunction.vector4(self.blurParas[i-1].mx, self.blurParas[i-1].my, self.fbosize:x() * scaleInv*0.5, self.fbosize:y() * scaleInv*0.5 ) );
    context:Draw(self.blurRO, self.blurMA);
    context:EndRenderPass();
  end

  --upSample pass4 -> pass7
  --pass4
  context:BeginRenderPass(self.upSampleRTs[3], apolloengine.RenderTargetEntity.CF_COLOR);
  self.upSampleMA:SetParameter(self._SourceTex, self.blurRTs[2]:GetAttachment( apolloengine.RenderTargetEntity.TA_COLOR_0));
  self.upSampleMA:SetParameter(self._SourceTexLowMip, self.blurRTs[3]:GetAttachment( apolloengine.RenderTargetEntity.TA_COLOR_0));
  context:Draw(self.upSampleRO, self.upSampleMA);
  context:EndRenderPass();

  --pass5
  context:BeginRenderPass(self.upSampleRTs[2], apolloengine.RenderTargetEntity.CF_COLOR);
  self.upSampleMA:SetParameter(self._SourceTex, self.blurRTs[1]:GetAttachment( apolloengine.RenderTargetEntity.TA_COLOR_0));
  self.upSampleMA:SetParameter(self._SourceTexLowMip, self.upSampleRTs[3]:GetAttachment( apolloengine.RenderTargetEntity.TA_COLOR_0));
  context:Draw(self.upSampleRO, self.upSampleMA);
  context:EndRenderPass();

  --pass6
  context:BeginRenderPass(self.upSampleRTs[1], apolloengine.RenderTargetEntity.CF_COLOR);
  self.upSampleMA:SetParameter(self._SourceTex, self.downSampleRTs[1]:GetAttachment( apolloengine.RenderTargetEntity.TA_COLOR_0));
  self.upSampleMA:SetParameter(self._SourceTexLowMip, self.upSampleRTs[2]:GetAttachment( apolloengine.RenderTargetEntity.TA_COLOR_0));
  context:Draw(self.upSampleRO, self.upSampleMA);
  context:EndRenderPass();


  ----bloom+lut
  context:BeginRenderPass(self.bloomLutRT, apolloengine.RenderTargetEntity.CF_COLOR);
  self.bloomLutMA:SetParameter(self._SourceTex, self.prefilterRT:GetAttachment(apolloengine.RenderTargetEntity.TA_COLOR_0));
  self.bloomLutMA:SetParameter(self._Bloom_Texture, self.upSampleRTs[1]:GetAttachment(apolloengine.RenderTargetEntity.TA_COLOR_0));

  if self.LutTexEntity ~= self.LastLutTexEntity then
    self.bloomLutMA:SetParameter(self._InternalLut, self.LutTexEntity);
    self.LutTexEntity = self.LastLutTexEntity
  end

  self.bloomLutMA:SetParameter(self.Bloom_Params,mathfunction.vector4(self.bloomIntensity.mx, self.bloomColor.mx, self.bloomColor.my, self.bloomColor.mz) );
  self.bloomLutMA:SetParameter(self.Bloom_RGBM, self.bloomFastMode and mathfunction.vector1(1.0) or mathfunction.vector1(0.0) );
  self.bloomLutMA:SetParameter(self.Lut_Params,  self.lutParas );
  --self.bloomLutMA:SetParameter(self.UserLut_Params, self.useLutParas );
  self.bloomLutMA:SetParameter(self.Vignette_Params1 ,mathfunction.vector4(self.vignetteColor.mx,self.vignetteColor.my,self.vignetteColor.mz,self.vignetteIntensity.mx) );
  self.bloomLutMA:SetParameter(self.Vignette_Params2, mathfunction.vector4(self.vignetteCenter.mx, self.vignetteCenter.my, self.vignetteEnable and 1.0 or 0.0, self.vignetteRoundness.mx) );

  context:Draw(self.bloomLutRO, self.bloomLutMA);
  context:EndRenderPass();

  --final pass
  context:BeginRenderPass(Output, apolloengine.RenderTargetEntity.CF_COLOR);
  self.FinalMA:SetParameter(self._SourceTex, self.bloomLutRT:GetAttachment(apolloengine.RenderTargetEntity.TA_COLOR_0));
  self.FinalMA:SetParameter(self.SourceSize, mathfunction.vector4(self.fbosize:x(),  self.fbosize:y(), 0.0, 0.0));
  self.FinalMA:SetParameter(self.EnableLinear2Gamma, self.enablelinear2gamma and mathfunction.vector1(1.0) or mathfunction.vector1(0.0));
  context:Draw(self.FinalRO, self.FinalMA);
  context:EndRenderPass();

end

return vyouBloomEffect;