local apolloengine = require "apolloengine"
local mathfunction = require "mathfunction"
local venuscore = require "venuscore"

require "utility"

local bloomEffect_v2 = {}

bloomEffect_v2.light_prefilterRO = 0;
bloomEffect_v2.light_prefilterMA = 0;

bloomEffect_v2.down_sampleRO = 0;
bloomEffect_v2.down_sampleMA = 0;

bloomEffect_v2.up_sampleRO = 0;
bloomEffect_v2.up_sampleMA = 0;

bloomEffect_v2.accumulationRO = 0;
bloomEffect_v2.accumulationMA = 0;

bloomEffect_v2.intensity = 1.0; -- Min(0f)
bloomEffect_v2.threshold = 0.692; -- Min(0f)
bloomEffect_v2.softKnee = 0.5; -- (0f, 1f)
bloomEffect_v2.clamp = 65472.0
bloomEffect_v2.diffusion = 7; -- (1f, 10f)
bloomEffect_v2.anamorphicRatio = 0.0; --(-1f, 1f)

--Dirtness

bloomEffect_v2.Queue = 100;
-- Iterative sampling times
bloomEffect_v2.iterations = 4;
-- sampling scale factor
-- eg: factor = 2 means that the result of each down sampling is 1/2 of the input
bloomEffect_v2.factor = 4;

local function log2(x)
  return math.log( x ) / math.log( bloomEffect_v2.factor );
end

local function GammaToLinearSpace(c)
  return c * (c * (c * 0.305306011 + 0.682171111) + 0.012522878);
end

local function Exp2(x)
  return math.exp(x * 0.69314718055994530941723212145818);
end

function bloomEffect_v2:Initialize(host, size)

  self.fbosize            = mathfunction.vector2(size:x(), size:y());
  self.intensity          = mathfunction.vector1(1.0);
  self.threshold          = mathfunction.vector1(0.692);
  self.softKnee      = mathfunction.vector1(0.5);
  self.clamp = mathfunction.vector1(65472.0)
  self.diffusion = mathfunction.vector1(7); -- (1f, 10f)
  self.anamorphicRatio = mathfunction.vector1(0.0); --(-1f, 1f)
  self.color  = mathfunction.vector3(1.0, 1.0, 1.0);
  self.dirtIntensity = mathfunction.vector1(0.0);  -- Min(0f)
  self.blueNoiseSize  = mathfunction.vector2(64.0, 64.0);
  self.m_NoiseTextureIndex = 0;

  self.sampleScale = mathfunction.vector1(0.0);

  self.k_MaxPyramidSize = 8;
  self.fastMode = true;
  self.qualityOffset = self.fastMode and 2 or 1;

  self.prefilterMA = {};
  self.down_sampleMA = {};
  self.up_sampleMA = {};

  self.iterations = self.k_MaxPyramidSize;


  self.whiteTex     = apolloengine.TextureEntity();
  self.whiteTex:PushMetadata(apolloengine.TextureFileMetadata(
          apolloengine.TextureEntity.TU_STATIC,
          apolloengine.TextureEntity.PF_AUTO,1, false,
          apolloengine.TextureEntity.TW_CLAMP_TO_EDGE,
          apolloengine.TextureEntity.TW_CLAMP_TO_EDGE,
          apolloengine.TextureEntity.TF_LINEAR,
          apolloengine.TextureEntity.TF_LINEAR,
          "comm:documents/texture/material/blank.jpg"));
  self.whiteTex:SetJobType(venuscore.IJob.JT_SYNCHRONOUS);
  self.whiteTex:CreateResource();

  self.blackTex     = apolloengine.TextureEntity();
  self.blackTex:PushMetadata(apolloengine.TextureFileMetadata(
          apolloengine.TextureEntity.TU_STATIC,
          apolloengine.TextureEntity.PF_AUTO,1, false,
          apolloengine.TextureEntity.TW_CLAMP_TO_EDGE,
          apolloengine.TextureEntity.TW_CLAMP_TO_EDGE,
          apolloengine.TextureEntity.TF_LINEAR,
          apolloengine.TextureEntity.TF_LINEAR,
          "comm:documents/texture/material/black.png"));
  self.blackTex:SetJobType(venuscore.IJob.JT_SYNCHRONOUS);
  self.blackTex:CreateResource();

  self._Params = apolloengine.IMaterialSystem:NewParameterSlot(apolloengine.ShaderEntity.UNIFORM, "_Params");
  self._Threshold = apolloengine.IMaterialSystem:NewParameterSlot(apolloengine.ShaderEntity.UNIFORM, "_Threshold");
  self._MainTex_TexelSize = apolloengine.IMaterialSystem:NewParameterSlot(apolloengine.ShaderEntity.UNIFORM, "_MainTex_TexelSize");
  self._RenderViewportScaleFactor = apolloengine.IMaterialSystem:NewParameterSlot(apolloengine.ShaderEntity.UNIFORM, "_RenderViewportScaleFactor");
  self._SampleScale = apolloengine.IMaterialSystem:NewParameterSlot(apolloengine.ShaderEntity.UNIFORM, "_SampleScale");
  self._ColorIntensity = apolloengine.IMaterialSystem:NewParameterSlot(apolloengine.ShaderEntity.UNIFORM, "_ColorIntensity");


  self._Bloom_Color = apolloengine.IMaterialSystem:NewParameterSlot(apolloengine.ShaderEntity.UNIFORM, "_Bloom_Color");
  self._Bloom_Settings = apolloengine.IMaterialSystem:NewParameterSlot(apolloengine.ShaderEntity.UNIFORM, "_Bloom_Settings");
  self._Bloom_DirtTileOffset = apolloengine.IMaterialSystem:NewParameterSlot(apolloengine.ShaderEntity.UNIFORM, "_Bloom_DirtTileOffset");
  self._BloomTex_TexelSize = apolloengine.IMaterialSystem:NewParameterSlot(apolloengine.ShaderEntity.UNIFORM, "_BloomTex_TexelSize");

  self._MainTex = apolloengine.IMaterialSystem:NewParameterSlot(apolloengine.ShaderEntity.UNIFORM, "_MainTex");
  self._BloomTex = apolloengine.IMaterialSystem:NewParameterSlot(apolloengine.ShaderEntity.UNIFORM, "_BloomTex");
  self._Bloom_DirtTex = apolloengine.IMaterialSystem:NewParameterSlot(apolloengine.ShaderEntity.UNIFORM, "_Bloom_DirtTex");
  --self._DitheringTex = apolloengine.IMaterialSystem:NewParameterSlot(apolloengine.ShaderEntity.UNIFORM, "_DitheringTex");

  self._BloomMode = apolloengine.IMaterialSystem:NewParameterSlot(apolloengine.ShaderEntity.UNIFORM, "_BloomMode");

  --uniform sampler2D _MainTex;
  --uniform vec4 _Params;
  --uniform vec4 _Threshold;
  --uniform vec4 _MainTex_TexelSize;
  --uniform float _RenderViewportScaleFactor;
  self.prefilterMA[1] = host:CreateMaterial( "comm:documents/material/unity_bloom/prefilter13.material" );
  self.prefilterMA[2] = host:CreateMaterial( "comm:documents/material/unity_bloom/prefilter4.material" );
  self.prefilterRO = host:CreateRenderObject();

  --uniform sampler2D _MainTex;
  --uniform vec4 _MainTex_TexelSize;
  --uniform float _RenderViewportScaleFactor;
  self.down_sampleMA[1] = host:CreateMaterial( "comm:documents/material/unity_bloom/Downsample13.material" );
  self.down_sampleMA[2] = host:CreateMaterial( "comm:documents/material/unity_bloom/Downsample4.material" );
  self.down_sampleRO = host:CreateRenderObject();

  --uniform sampler2D _BloomTex;
  --uniform sampler2D _MainTex;
  --uniform float _SampleScale;
  --uniform vec4 _MainTex_TexelSize;
  --uniform float _RenderViewportScaleFactor;
  self.up_sampleMA[1] = host:CreateMaterial( "comm:documents/material/unity_bloom/UpsampleTent.material" );
  self.up_sampleMA[2] = host:CreateMaterial( "comm:documents/material/unity_bloom/UpsampleBox.material" );
  self.up_sampleRO = host:CreateRenderObject();

  --uniform vec4 _Dithering_Coords;
  --uniform vec3 _Bloom_Color;
  --uniform vec3 _Bloom_Settings;
  --uniform vec4 _Bloom_DirtTileOffset;
  --uniform vec4 _BloomTex_TexelSize;
  --uniform sampler2D _DitheringTex;
  --uniform sampler2D _Bloom_DirtTex;
  --uniform sampler2D _BloomTex;
  --uniform sampler2D _MainTex;
  --uniform float _RenderViewportScaleFactor;
  self.uberMA     = host:CreateMaterial( "comm:documents/material/unity_bloom/Uber.material" );
  self.uberRO     = host:CreateRenderObject();

  self.CopyKillNanMA     = host:CreateMaterial( "comm:documents/material/unity_bloom/Copy.material" );
  self.CopyKillNanRO     = host:CreateRenderObject();

  self._BloomMipDown = {}
  self._BloomMipUp = {}

  local scale = 1.0;
  self._TargetPool0 = host:CreateRenderTarget(
          apolloengine.RenderTargetMetadata(
                  apolloengine.RenderTargetEntity.RT_RENDER_TARGET_2D,
                  apolloengine.RenderTargetEntity.ST_SWAP_UNIQUE,
                  mathfunction.vector4(self.fbosize:x()*scale,self.fbosize:y()*scale,0,0),
                  self.fbosize*scale,
                  mathfunction.vector2(scale,scale)),
          apolloengine.TextureRenderMetadata(
                  apolloengine.RenderTargetEntity.ST_SWAP_UNIQUE,
                  self.fbosize*scale,
                  apolloengine.TextureEntity.PF_RG11B10FLOAT));

  for i = 1, self.iterations do
    scale = scale / self.factor;
    self._BloomMipDown[i] = host:CreateRenderTarget(
    apolloengine.RenderTargetMetadata(
        apolloengine.RenderTargetEntity.RT_RENDER_TARGET_2D,
        apolloengine.RenderTargetEntity.ST_SWAP_UNIQUE,
        mathfunction.vector4(self.fbosize:x()*scale,self.fbosize:y()*scale,0,0),
        self.fbosize*scale,
        mathfunction.vector2(scale,scale)),
    apolloengine.TextureRenderMetadata(
      apolloengine.RenderTargetEntity.ST_SWAP_UNIQUE,
      self.fbosize*scale,
      apolloengine.TextureEntity.PF_RG11B10FLOAT));
  end

  scale = 1.0;
  for i = 1, self.iterations - 1 do
    scale = scale / self.factor;
    self._BloomMipUp[i] = host:CreateRenderTarget(
    apolloengine.RenderTargetMetadata(
        apolloengine.RenderTargetEntity.RT_RENDER_TARGET_2D,
        apolloengine.RenderTargetEntity.ST_SWAP_UNIQUE,
        mathfunction.vector4(self.fbosize:x()*scale,self.fbosize:y()*scale,0,0),
        self.fbosize*scale,
        mathfunction.vector2(scale,scale)),
    apolloengine.TextureRenderMetadata(
      apolloengine.RenderTargetEntity.ST_SWAP_UNIQUE,
      self.fbosize*scale,
      apolloengine.TextureEntity.PF_RG11B10FLOAT));
  end

  host:RegisterScriptParameter(self, "intensity");
  host:RegisterScriptParameter(self, "threshold");
  host:RegisterScriptParameter(self, "softKnee");
  host:RegisterScriptParameter(self, "fastMode");
  host:RegisterScriptParameter(self, "clamp");
  host:RegisterScriptParameter(self, "diffusion");
  host:RegisterScriptParameter(self, "anamorphicRatio");
  host:RegisterScriptParameter(self, "color");

  return self.Queue;
end

function bloomEffect_v2:Resizeview(size)
  self.fbosize = mathfunction.vector2(size:x(), size:y());
end

function bloomEffect_v2:Process(context, Original, Scene, Output)

  if self.fbosize.mx > 16 and self.fbosize.my > 16 then
    -- Negative anamorphic ratio values distort vertically - positive is horizontal
    local ratio = math.clamp(self.anamorphicRatio.mx, -1, 1);
    local rw = ratio < 0 and -ratio or 0;
    local rh = ratio > 0 and  ratio or 0;

    -- Do bloom on a half-res buffer, full-res doesn't bring much and kills performances on
    -- fillrate limited platforms
    local tw = math.floor(self.fbosize.mx / (2 - rw));
    local th = math.floor(self.fbosize.my / (2 - rh));

    -- Determine the iteration count
    local s = math.max(tw, th);
    local logs = log2(s) + math.min(self.diffusion.mx, 10.0) - 10.0;
    local logs_i = math.floor(logs);
    self.iterations = math.clamp(logs_i, 2, self.k_MaxPyramidSize);

    self.sampleScale.mx = 0.5 + logs - logs_i;
  end

  context:BeginRenderPass(self._TargetPool0, apolloengine.RenderTargetEntity.CF_COLOR);
  self.CopyKillNanMA:SetParameter(self._MainTex, Scene:GetAttachment(apolloengine.RenderTargetEntity.TA_COLOR_0));
  context:Draw(self.CopyKillNanRO, self.CopyKillNanMA);
  context:EndRenderPass();

  local MainTexTexelSize =  mathfunction.vector4(1.0 / self.fbosize.mx, 1.0 / self.fbosize.my, self.fbosize.mx, self.fbosize.my);

  -- Prefiltering parameters
  local lthresh = GammaToLinearSpace(self.threshold.mx);
  local knee = lthresh * self.softKnee.mx + 1e-5;
  local lclamp = GammaToLinearSpace(self.clamp.mx);

  --light_prefilter, select pixels by brightness
  --get filted RTa[1], size by 1/2
  context:BeginRenderPass(self._BloomMipDown[1], apolloengine.RenderTargetEntity.CF_COLOR);
  self.prefilterMA[self.qualityOffset]:SetParameter(self._MainTex, self._TargetPool0:GetAttachment(apolloengine.RenderTargetEntity.TA_COLOR_0));
  self.prefilterMA[self.qualityOffset]:SetParameter( self._MainTex_TexelSize, MainTexTexelSize);
  self.prefilterMA[self.qualityOffset]:SetParameter( self._Params, mathfunction.vector4(lclamp, 0, 0, 0));
  --self.prefilterMA[self.qualityOffset]:SetParameter( self._Threshold, mathfunction.vector4(0.43711, 0.21855, 0.43713, 1.14382));
  self.prefilterMA[self.qualityOffset]:SetParameter( self._Threshold, mathfunction.vector4(lthresh, lthresh - knee, knee * 2, 0.25 / knee));
  self.prefilterMA[self.qualityOffset]:SetParameter( self._RenderViewportScaleFactor, mathfunction.vector1(1.0));

  --_Threshold 0.43711, 0.21855, 0.43713, 1.14382 float4

  context:Draw(self.prefilterRO, self.prefilterMA[self.qualityOffset]);
  context:EndRenderPass();

  -- downsample
  for i = 2, self.iterations do
    context:BeginRenderPass(self._BloomMipDown[i], apolloengine.RenderTargetEntity.CF_COLOR);
    local mainTex = self._BloomMipDown[i - 1]:GetAttachment(apolloengine.RenderTargetEntity.TA_COLOR_0);
    local _size = mainTex:GetSize();
    self.down_sampleMA[self.qualityOffset]:SetParameter(self._MainTex, mainTex);
    self.down_sampleMA[self.qualityOffset]:SetParameter( self._MainTex_TexelSize, mathfunction.vector4(1.0/_size.mx, 1.0/_size.my, _size.mx, _size.my));
    self.down_sampleMA[self.qualityOffset]:SetParameter( self._RenderViewportScaleFactor, mathfunction.vector1(1.0));
    context:Draw(self.down_sampleRO, self.down_sampleMA[self.qualityOffset]);
    context:EndRenderPass();
  end

  -- Upsample:
  local mainTex = self._BloomMipDown[self.iterations]:GetAttachment(apolloengine.RenderTargetEntity.TA_COLOR_0);
  local _size = mainTex:GetSize();
  context:BeginRenderPass(self._BloomMipUp[self.iterations - 1], apolloengine.RenderTargetEntity.CF_COLOR);
  self.up_sampleMA[self.qualityOffset]:SetParameter(self._MainTex, mainTex);
  self.up_sampleMA[self.qualityOffset]:SetParameter(self._BloomTex, self._BloomMipDown[self.iterations - 1]:GetAttachment(apolloengine.RenderTargetEntity.TA_COLOR_0));
  self.up_sampleMA[self.qualityOffset]:SetParameter( self._MainTex_TexelSize, mathfunction.vector4(1.0/_size.mx, 1.0/_size.my, _size.mx, _size.my) );
  self.up_sampleMA[self.qualityOffset]:SetParameter( self._RenderViewportScaleFactor, mathfunction.vector1(1.0));
  self.up_sampleMA[self.qualityOffset]:SetParameter( self._SampleScale, self.sampleScale);
  context:Draw(self.up_sampleRO, self.up_sampleMA[self.qualityOffset]);
  context:EndRenderPass();

  for i = self.iterations - 1, 2, -1 do
    context:BeginRenderPass(self._BloomMipUp[i - 1], apolloengine.RenderTargetEntity.CF_COLOR);
    mainTex = self._BloomMipUp[i]:GetAttachment(apolloengine.RenderTargetEntity.TA_COLOR_0);
    _size = mainTex:GetSize();
    self.up_sampleMA[self.qualityOffset]:SetParameter(self._MainTex, mainTex);
    self.up_sampleMA[self.qualityOffset]:SetParameter(self._BloomTex, self._BloomMipDown[i - 1]:GetAttachment(apolloengine.RenderTargetEntity.TA_COLOR_0));
    self.up_sampleMA[self.qualityOffset]:SetParameter( self._MainTex_TexelSize, mathfunction.vector4(1.0/_size.mx, 1.0/_size.my, _size.mx, _size.my) );
    self.up_sampleMA[self.qualityOffset]:SetParameter( self._RenderViewportScaleFactor, mathfunction.vector1(1.0));
    self.up_sampleMA[self.qualityOffset]:SetParameter( self._SampleScale, self.sampleScale);
    context:Draw(self.up_sampleRO, self.up_sampleMA[self.qualityOffset]);
    context:EndRenderPass();
  end

  -- Bloom
  local bloomTex = self._BloomMipUp[1]:GetAttachment(apolloengine.RenderTargetEntity.TA_COLOR_0);
  _size = bloomTex:GetSize();
  context:BeginRenderPass(Output, apolloengine.RenderTargetEntity.CF_COLOR);
  self.uberMA:SetParameter(self._MainTex, self._TargetPool0:GetAttachment(apolloengine.RenderTargetEntity.TA_COLOR_0));
  self.uberMA:SetParameter(self._BloomTex, bloomTex);
  self.uberMA:SetParameter(self._Bloom_DirtTex, self.blackTex);
  --self.uberMA:SetParameter(self._DitheringTex, self.blackTex);
  self.uberMA:SetParameter( self._RenderViewportScaleFactor, mathfunction.vector1(1.0));
  self.uberMA:SetParameter( self._BloomTex_TexelSize, mathfunction.vector4(1.0/_size.mx, 1.0/_size.my, _size.mx, _size.my) );
  self.uberMA:SetParameter( self._BloomMode, mathfunction.vector1(self.qualityOffset));

  local dirtTexSize = self.blackTex:GetSize();

  local dirtRatio = dirtTexSize.mx / dirtTexSize.my;
  local screenRatio = self.fbosize.mx / self.fbosize.my;
  local dirtTileOffset = mathfunction.vector4(1, 1, 0, 0);
  if dirtRatio > screenRatio then
    dirtTileOffset.mx = screenRatio / dirtRatio;
    dirtTileOffset.mz = (1 - dirtTileOffset.mx) * 0.5;
  elseif screenRatio > dirtRatio then
    dirtTileOffset.my = dirtRatio / screenRatio;
    dirtTileOffset.mw = (1 - dirtTileOffset.my) * 0.5;
  end

  local linearColor = self.color;
  local intensity = Exp2(self.intensity.mx / 10) - 1;
  local shaderSettings = mathfunction.vector3(self.sampleScale.mx, intensity, self.dirtIntensity.mx);

  --self.uberMA:SetParameter( self._Dithering_Coords, dirtTileOffset);

  self.uberMA:SetParameter( self._Bloom_Color, self.color);
  self.uberMA:SetParameter( self._Bloom_Settings, shaderSettings);
  self.uberMA:SetParameter( self._Bloom_DirtTileOffset, dirtTileOffset);
  context:Draw(self.uberRO, self.uberMA);
  context:EndRenderPass();

end

return bloomEffect_v2;