local apolloengine = require "apolloengine"
local mathfunction = require "mathfunction"
local venuscore = require "venuscore"

local lut_user = {}

function lut_user:Initialize(host, size)
  self.DoMaterial  = 0;
  self.DoRenderObj = 0;
  self.Queue = 310;

  self.contribution = 0.858;

  self.LutParams = mathfunction.vector4(0.00391, 0.0625, 15, self.contribution);
  self._UserLutParams = apolloengine.IMaterialSystem:NewParameterSlot(apolloengine.ShaderEntity.UNIFORM, "_UserLutParams");

  --self.lutPath = "docs:lutpics/lut.png";
  --self.lastLutPath = "docs:lutpics/lut.png";
  self.lutPath = "comm:documents/texture/posteffect/uerlut.png";
  self.lastLutPath = "comm:documents/texture/posteffect/uerlut.png";
  self.LutTexEntity = apolloengine.TextureEntity();
  self.LutTexEntity:PushMetadata(apolloengine.TextureFileMetadata(
          apolloengine.TextureEntity.TU_STATIC,
          apolloengine.TextureEntity.PF_AUTO,1, false,
          apolloengine.TextureEntity.TW_CLAMP_TO_EDGE,
          apolloengine.TextureEntity.TW_CLAMP_TO_EDGE,
          apolloengine.TextureEntity.TF_LINEAR,
          apolloengine.TextureEntity.TF_LINEAR,
          "comm:documents/texture/lut/uerlut.png", false, false));
  self.LutTexEntity:SetJobType(venuscore.IJob.JT_SYNCHRONOUS);
  self.LutTexEntity:CreateResource();

  self.LastLutTexEntity = self.LutTexEntity;

  self.LOOKUPTABLE = apolloengine.IMaterialSystem:NewParameterSlot(
      apolloengine.ShaderEntity.UNIFORM,
      "LOOKUPTABLE");
  
  local pathout = venuscore.IFileSystem:PathAssembly("comm:documents/material/unity_bloom/userLut.material");
  self.DoMaterial = host:CreateMaterial(pathout);
  self.DoRenderObj = host:CreateRenderObject();
  self.DoMaterial:SetParameter(self.LOOKUPTABLE, self.LutTexEntity);
  self.DoMaterial:SetParameter(self._UserLutParams, self.LutParams);

  host:RegisterScriptParameter(self, "LutTexEntity");
  host:RegisterScriptParameter(self, "contribution");
  return self.Queue;
end



function lut_user:Resizeview(size)
end

function lut_user:Process(context, Original, Scene, Output)
 
  if self.LutTexEntity ~= self.LastLutTexEntity then
    self.DoMaterial:SetParameter(self.LOOKUPTABLE, self.LutTexEntity);
    self.LastLutTexEntity = self.LutTexEntity;
  end
  
  context:BeginRenderPass(Output, apolloengine.RenderTargetEntity.CF_COLOR);
  self.DoMaterial:SetParameter(
		apolloengine.ShaderEntity.TEXTURE_DIFFUSE,
		Scene:GetAttachment( apolloengine.RenderTargetEntity.TA_COLOR_0 ) );

  local _size = self.LutTexEntity:GetSize();
  self.DoMaterial:SetParameter(self._UserLutParams, mathfunction.vector4(1.0/_size.mx, 1.0/_size.my, _size.my - 1, self.contribution));

  context:Draw(self.DoRenderObj, self.DoMaterial);
  context:EndRenderPass();
end

return lut_user;