local apolloengine = require "apolloengine"
local mathfunction = require "mathfunction"
local venuscore = require "venuscore"

local lut_vyou = {}

function lut_vyou:Initialize(host, size)
  self.DoMaterial  = 0;
  self.DoRenderObj = 0;
  self.Queue = 310;

  self.LutParams = mathfunction.vector4(0.00391, 0.0625, 15, 1.07177);
  self.Lut_Params = apolloengine.IMaterialSystem:NewParameterSlot(apolloengine.ShaderEntity.UNIFORM, "Lut_Params");

  --self.lutPath = "docs:lutpics/lut.png";
  --self.lastLutPath = "docs:lutpics/lut.png";
  self.lutPath = "comm:documents/texture/posteffect/lut.png";
  self.lastLutPath = "comm:documents/texture/posteffect/lut.png";
  self.LutTexEntity = apolloengine.TextureEntity();
  self.LutTexEntity:PushMetadata(apolloengine.TextureFileMetadata(
          apolloengine.TextureEntity.TU_STATIC,
          apolloengine.TextureEntity.PF_AUTO,1, false,
          apolloengine.TextureEntity.TW_CLAMP_TO_EDGE,
          apolloengine.TextureEntity.TW_CLAMP_TO_EDGE,
          apolloengine.TextureEntity.TF_LINEAR,
          apolloengine.TextureEntity.TF_LINEAR,
          "comm:documents/texture/lut/pass7-lut.png", false, true));
  self.LutTexEntity:SetJobType(venuscore.IJob.JT_SYNCHRONOUS);
  self.LutTexEntity:CreateResource();

  self.LastLutTexEntity = self.LutTexEntity;

  self.LOOKUPTABLE = apolloengine.IMaterialSystem:NewParameterSlot(
      apolloengine.ShaderEntity.UNIFORM,
      "LOOKUPTABLE");
  
  local pathout = venuscore.IFileSystem:PathAssembly("comm:documents/material/unity_bloom/VyouLut.material");
  self.DoMaterial = host:CreateMaterial(pathout);
  self.DoRenderObj = host:CreateRenderObject();
  self.DoMaterial:SetParameter(self.LOOKUPTABLE, self.LutTexEntity);
  self.DoMaterial:SetParameter(self.Lut_Params, self.LutParams);

  host:RegisterScriptParameter(self, "LutTexEntity");
  host:RegisterScriptParameter(self, "LutParams");
  return self.Queue;
end



function lut_vyou:Resizeview(size)
end

function lut_vyou:Process(context, Original, Scene, Output)
 
  if self.LutTexEntity ~= self.LastLutTexEntity then
    self.DoMaterial:SetParameter(self.LOOKUPTABLE, self.LutTexEntity);
    self.LastLutTexEntity = self.LutTexEntity;
  end
  
  context:BeginRenderPass(Output, apolloengine.RenderTargetEntity.CF_COLOR);
  self.DoMaterial:SetParameter(
		apolloengine.ShaderEntity.TEXTURE_DIFFUSE,
		Scene:GetAttachment( apolloengine.RenderTargetEntity.TA_COLOR_0 ) );
  context:Draw(self.DoRenderObj, self.DoMaterial);
  context:EndRenderPass();
end

return lut_vyou;